package no.miles.mpl.plugins;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import no.miles.mpl.plugins.jira.exception.RemoteException;

import com.atlassian.jira.issue.Issue;
import com.atlassian.jira.issue.index.DocumentConstants;
import com.atlassian.jira.issue.search.SearchException;
import com.atlassian.jira.issue.search.SearchProvider;
import com.atlassian.jira.issue.search.SearchRequest;
import com.atlassian.jira.issue.search.SearchResults;
import com.atlassian.jira.issue.search.parameters.lucene.AbsoluteDateRangeParameter;
import com.atlassian.jira.issue.search.parameters.lucene.ProjectParameter;
import com.atlassian.jira.project.ProjectManager;
import com.atlassian.jira.security.JiraAuthenticationContext;
import com.atlassian.jira.web.bean.PagerFilter;
import com.opensymphony.user.User;

public class SearchServiceExtension {


//	private final PermissionManager permissionManager;
//	private final ConstantsManager constantsManager;
//    private final CustomFieldManager customFieldManager;
	private final SearchProvider searchProvider;
//	private final ApplicationProperties applicationProperties;
//	private final SearchRequestManager searchRequestManager;
//	private final VersionManager versionManager;
	private final ProjectManager projectManager;
	private final JiraAuthenticationContext authenticationContext;



	public SearchServiceExtension(JiraAuthenticationContext authenticationContext, 
			SearchProvider searchProvider, 
			ProjectManager projectManager)  {
		this.authenticationContext = authenticationContext;
//		this.permissionManager = permissionManager;
//		this.constantsManager = constantsManager;
		this.searchProvider = searchProvider;
//		this.applicationProperties = applicationProperties;
//		this.searchRequestManager = searchRequestManager;
//		this.versionManager = versionManager;
		this.projectManager = projectManager;
//		this.customFieldManager = customFieldManager;
		

	}

	/**
	 * Get all
	 * @param user
	 * @param projectKeys
	 * @return
	 * @throws Exception
	 */
	public String[] getIssueKeysWithProject(User user, String[] projectKeys) throws RemoteException{

		SearchRequest searchRequest = new SearchRequest(user);		
		
		if (projectKeys != null && projectKeys.length > 0)
        {
            searchRequest.addParameter(new ProjectParameter(convertKeysToIds(projectKeys)));
        }
		
		PagerFilter pagerFilter = PagerFilter.getUnlimitedFilter();
		
		try {
			
			SearchResults sr = searchProvider.search(searchRequest, user, pagerFilter);
			List issues = sr.getIssues();
			
			String[] issueKeys = new String[issues.size()];
			
			for (int i = 0; i < issues.size(); i++) {
				
				Issue issue = (Issue) issues.get(i);
				
				issueKeys[i] = issue.getKey();				
			}
			
			return issueKeys;
			
		} catch (SearchException e) {
			// TODO Auto-generated catch block
			throw new RemoteException("Error occurred during searching", e);
		}	
		
	}




	/**
	 * Get issues keys for issues updated after the specific date
	 * @param user
	 * @param projectKeys
	 * @param date
	 * @return
	 * @throws Exception
	 */
	public String[] getUpdatedIssueKeysWithProject(User user, String[] projectKeys, Date date) throws RemoteException{

		SearchRequest searchRequest = new SearchRequest(user);		
		searchRequest.addParameter(new AbsoluteDateRangeParameter(DocumentConstants.ISSUE_UPDATED, date, null));
		
		if (projectKeys != null && projectKeys.length > 0)
        {
            searchRequest.addParameter(new ProjectParameter(convertKeysToIds(projectKeys)));
        }
		
		PagerFilter pagerFilter = PagerFilter.getUnlimitedFilter();
		
		try {
			
			SearchResults sr = searchProvider.search(searchRequest, user, pagerFilter);
			List issues = sr.getIssues();
			
			String[] issueKeys = new String[issues.size()];
			
			for (int i = 0; i < issues.size(); i++) {
				
				Issue issue = (Issue) issues.get(i);
				
				issueKeys[i] = issue.getKey();				
			}
			
			return issueKeys;
			
		} catch (SearchException e) {
			// TODO Auto-generated catch block
			throw new RemoteException("Error occurred during searching", e);
		}	
		
	}
	
	/**
	 * Borrowed from the jira-rpc plugin..
	 * @param projectKeys
	 * @return
	 */
    private List convertKeysToIds(String[] projectKeys)
    {
        if (projectKeys == null || projectKeys.length == 0)
            return Collections.EMPTY_LIST;

        List list = new ArrayList();
        for (int i = 0; i < projectKeys.length; i++)
        {
            String projectKey = projectKeys[i];
            list.add(projectManager.getProjectObjByKey(projectKey).getId());
        }
        return list;
    }

}
