
package no.miles.mpl.plugins.jira.auth;

import com.atlassian.core.user.UserUtils;
import com.atlassian.core.util.DateUtils;
import com.atlassian.core.util.RandomGenerator;
import com.atlassian.jira.security.PermissionManager;
import com.atlassian.jira.security.Permissions;
import com.opensymphony.user.EntityNotFoundException;
import com.opensymphony.user.User;
import com.opensymphony.util.TextUtils;

import java.util.Map;

import no.miles.mpl.plugins.jira.exception.RemoteAuthenticationException;
import no.miles.mpl.plugins.jira.exception.RemoteException;
import no.miles.mpl.plugins.jira.exception.RemotePermissionException;

public class TokenManagerImpl implements TokenManager
{
    public static long DEFAULT_TIMEOUT = 30 * DateUtils.MINUTE_MILLIS;
    Map tokens;
    private PermissionManager permissionManager;

    public TokenManagerImpl(PermissionManager permissionManager)
    {
        this(DEFAULT_TIMEOUT, permissionManager);
    }

    public TokenManagerImpl(long timeout, PermissionManager permissionManager)
    {
        this.permissionManager = permissionManager;
        tokens = new TokenMap(timeout);
    }

    public String login(String username, String password) throws RemoteException, RemoteAuthenticationException
    {
        try
        {
            User user = UserUtils.getUser(username);
            if (user != null)
            {
                boolean authenticated = user.authenticate(password);

                if (authenticated)
                {
                    return createToken(user);
                }
            }
        }
        catch (EntityNotFoundException e)
        {
        }

        // thrown if user not found, user is null or the password didn't match.
        throw new RemoteAuthenticationException("Invalid username or password.");
    }

    public boolean logout(String token)
    {
        synchronized (tokens)
        {
            return (tokens.remove(token) != null);
        }
    }

    private String createToken(User user) throws RemoteException
    {
        String token = null;

        synchronized (tokens)
        {
            token = RandomGenerator.randomString(10);

            int count = 0;
            while (tokens.containsKey(token) && count++ < 10)
            {
                token = RandomGenerator.randomString(10);
            }

            if (count >= 10)
            {
                throw new RemoteException("Error generating authentication token after 10 attempts?");
            }

            tokens.put(token, user);
        }

        return token;
    }   

    public User retrieveUserNoPermissionCheck(String token) throws RemoteAuthenticationException, RemotePermissionException
    {
        if (!TextUtils.stringSet(token))
            return null;

        User user = (User) tokens.get(token);

        if (user == null)
        {
            throw new RemoteAuthenticationException("User not authenticated yet, or session timed out.");
        }
        return user;

    }
}
