package no.miles.mpl.plugins.hudson.appclient;

import org.kohsuke.stapler.StaplerRequest;

import hudson.model.AbstractProject;
import hudson.model.Job;
import hudson.model.JobProperty;
import hudson.model.JobPropertyDescriptor;

public class MplProjectIssueFeedProperty extends JobProperty<AbstractProject<?,?>> {

	public final String issueFeed;

    /**
     * @stapler-constructor
     */
	public MplProjectIssueFeedProperty(String issueFeed) {
		// normalize
		if(issueFeed==null || issueFeed.length()==0)
			issueFeed=null;
		else {
			if(!issueFeed.endsWith("/"))
				issueFeed += '/';
		}
		this.issueFeed = issueFeed;
	}

	public DescriptorImpl getDescriptor() {
		return DESCRIPTOR;
	}

	public static final DescriptorImpl DESCRIPTOR = new DescriptorImpl();

	public static final class DescriptorImpl extends JobPropertyDescriptor {
		// no longer in use but kept for backward compatibility
		private transient String issueFeed;

		public DescriptorImpl() {
			super(MplProjectIssueFeedProperty.class);
			load();   
		}

		public boolean isApplicable(Class<? extends Job> jobType) {
			return AbstractProject.class.isAssignableFrom(jobType);
		}

		public String getDisplayName() {
			return "Miles Platform Issue Feed";
		}

		public JobProperty<?> newInstance(StaplerRequest req) throws FormException {
			MplProjectIssueFeedProperty mplProjectIssueFeed = req.bindParameters(MplProjectIssueFeedProperty.class, "mpl.");
			if(mplProjectIssueFeed.issueFeed == null)
				mplProjectIssueFeed = null; // not configured
			return mplProjectIssueFeed;
		}
	}

}
