package org.eclipse.mylyn.mpl.ui;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import no.miles.mpl.wrapper.datastructures.Issue;
import no.miles.mpl.wrapper.datastructures.IssueTypes;
import no.miles.mpl.wrapper.datastructures.PriorityTypes;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.mylyn.tasks.core.RepositoryTaskData;

import sun.security.x509.IssuerAlternativeNameExtension;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.Resource;
import com.sun.syndication.feed.atom.Content;
import com.sun.syndication.feed.atom.Link;
import com.sun.syndication.propono.atom.client.AtomClientFactory;
import com.sun.syndication.propono.atom.client.ClientCollection;
import com.sun.syndication.propono.atom.client.ClientEntry;
import com.sun.syndication.propono.utils.ProponoException;

public class MplAtomLib {


	public static boolean validateUrl(String username, String password, String repositoryUrl){

		try {
			ClientCollection atomClient = AtomClientFactory.getCollection(repositoryUrl, username, password);

			if(atomClient != null)
				return true;

		} catch (ProponoException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();

		}
		return false;		

	}

	private static String endWithSlashes(String url)
	{
		if(!url.endsWith("/"))
			return url + "/";

		return url;
	}

	/**
	 * Post a task
	 * @param username
	 * @param password
	 * @param repositoryUrl
	 * @param taskdata
	 * @return
	 */
	public static String postTask(String username, String password, String repositoryUrl, RepositoryTaskData taskdata){

		try {

			ClientCollection atomClient = AtomClientFactory.getCollection(repositoryUrl, username, password);	

			ClientEntry entry = atomClient.createEntry();

			entry.setTitle("New title");

			String projectKey = getProjectKey(repositoryUrl);

			Content summary = new Content();
			summary.setType("text");
			summary.setValue("summary...");

			entry.setSummary(summary);

			Issue issue = new Issue();
			issue.setSummary("summary..");
			issue.setPriority(PriorityTypes.PRIORITY_MAJOR);

			issue.setType(IssueTypes.ISSUE_TYPE_ISSUE);

			issue.setAssigned_to(username);

			Content content = new Content();
			content.setType("text/rdf+n3");
			content.setValue(issue.toN3());
			entry.setContent(content);

			atomClient.addEntry(entry);	
			String url = entry.getId();

			List<Link> links = entry.getAlternateLinks();

			if(links != null && links.size() > 0){
				for (Link link : links) {
					System.out.println(link.getRel() + " : " + link.getHref());
				}
			}

			//TODO: Fix ++
			Issue postedIssue = Issue.parseOntology(entry.getContent().getValue(), projectKey);

			System.out.println("entry id: " + entry.getId());

			System.out.println(entry.getEditURI());

//			String number = url.substring(url.lastIndexOf("/") + 1);
//			String taskId = number + " : " + entry.getTitle();

			return postedIssue.getKey() + " : " + postedIssue.getSummary();

		} catch (ProponoException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			System.out.println("problem parsing the issue..");
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;




	}

	private static String getProjectKey(String repositoryUrl){
//		/projects/{projectkey}/issues/		

//		System.out.println("trying to match repo: " + repositoryUrl);

		String regex = "/projects/(\\w+)/issues";
		Pattern pattern = Pattern.compile(regex);		
		Matcher matcher = pattern.matcher(repositoryUrl);		
		boolean found = matcher.find();

//		System.out.println("Found a match? " + found);

//		System.out.println(matcher.group(0));

		String projectKey = matcher.group(1);

//		System.out.println("ProjectKey: "+ projectKey);

		return projectKey;
	}

	/**
	 * Get the issuekey from the url..
	 * @param issueUrl
	 * @return
	 */
	private static String getIssueKey(String issueUrl){
//		/projects/{projectkey}/issues/		

		System.out.println("trying to match issueKey from : " + issueUrl);

		String regex = "/projects/(\\w+)/issues/(\\d+)";
		Pattern pattern = Pattern.compile(regex);		
		Matcher matcher = pattern.matcher(issueUrl);		
		boolean found = matcher.find();

		String issueKey = matcher.group(2);

//		System.out.println("IssueKey: "+ issueKey);

		return issueKey;
	}

	/**
	 * Get the issuekey from the url..
	 * @param issueUrl
	 * @return
	 */
	private static String getIssueKeyFromTaskId(String taskId){
//		/projects/{projectkey}/issues/		

//		System.out.println("trying to match issueUrl: " + issueUrl);

		String regex = "(\\d+)";
		Pattern pattern = Pattern.compile(regex);		
		Matcher matcher = pattern.matcher(taskId);		
		boolean found = matcher.find();

		String issueKey = matcher.group(1);

//		System.out.println("IssueKey: "+ issueKey);

		return issueKey;
	}

	/**
	 * Fetch an issue from the ++
	 * @param username
	 * @param password
	 * @param repositoryUrl
	 * @param taskId
	 * @return
	 */
	public static Issue fetchIssue(String username, String password, String repositoryUrl, String taskId){

		if(!repositoryUrl.endsWith("/"))
			repositoryUrl = repositoryUrl + "/";

		String issueKey = getIssueKeyFromTaskId(taskId);
		String issueUrl = repositoryUrl + issueKey;

		String projectKey = getProjectKey(repositoryUrl);

		System.out.println("Trying to fetch issue: " + issueUrl);

		try {
			ClientCollection atomClient = AtomClientFactory.getCollection(repositoryUrl, username, password);
			ClientEntry entry = atomClient.getEntry(issueUrl);

			String viewIssueUrl = null;
			List<Link> alternateLinks = entry.getAlternateLinks();
			if(alternateLinks != null && alternateLinks.size() > 0)
				viewIssueUrl = alternateLinks.get(0).getHref();	

			issueKey = getIssueKey(entry.getId());

			//Use the Issue parser
			Content content = entry.getContent();

			if(content != null && content.getType().equalsIgnoreCase("text/rdf+n3") && content.getValue() != null){

//				System.out.println(content.getValue());

				Issue issue = Issue.parseOntology(content.getValue(), projectKey);

				issue.setKey(issueKey);

				if(viewIssueUrl != null){
					issue.setViewissueurl(viewIssueUrl);
//					System.out.println("viewIssueUrl: " + viewIssueUrl);
				}
				
				return issue;
			}



		} catch (ProponoException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}



		return null;
	}


	/**
	 * Fetch tasks from the repository
	 * @param username
	 * @param password
	 * @param repositoryUrl
	 * @return
	 * @throws Exception 
	 */
	public static ArrayList<MplTask> fetchTasks(String username, String password, String repositoryUrl) {	

		try{

			long startupTime = 0;
			long fetchedEntriesTime = 0;
			long parsedEntriesTime = 0;

			ArrayList<MplTask> tasks = new ArrayList<MplTask>();

			String projectKey = getProjectKey(repositoryUrl);

			System.out.println("Fetching tasks from atom.." + repositoryUrl);

			ClientCollection atomClient;


			startupTime = System.currentTimeMillis();
//			System.out.println("Fetching atom tasks " + System.currentTimeMillis());
			atomClient = AtomClientFactory.getCollection(repositoryUrl, username, password);
			Iterator<ClientEntry> entries = atomClient.getEntries();
			fetchedEntriesTime = System.currentTimeMillis();

			while(entries.hasNext())
			{				



				ClientEntry entry = entries.next();
				String viewIssueUrl = null;
				List<Link> alternateLinks = entry.getAlternateLinks();
				if(alternateLinks != null && alternateLinks.size() > 0){
					viewIssueUrl = alternateLinks.get(0).getHref();						


					String issueKey = getIssueKey(entry.getId());

					//Use the Issue parser
					Content content = entry.getContent();

					if(content != null && content.getType().equalsIgnoreCase("text/rdf+n3") && content.getValue() != null){


//						System.out.println(content.getValue());

						Issue issue = Issue.parseOntology(content.getValue(), projectKey);

						issue.setKey(issueKey);

						if(viewIssueUrl != null){
							issue.setViewissueurl(viewIssueUrl);
//							System.out.println("viewIssueUrl: " + viewIssueUrl);
						}

//						System.out.println("building taskid..");

						String taskId = issue.getKey() + " : " + issue.getSummary();

//						System.out.println("taskid: " + taskId);
//						System.out.println("summary: " + issue.getSummary());

						MplTask task = new MplTask(repositoryUrl, taskId, issue.getSummary());

						tasks.add(task);
					}
				}

//				System.out.println("Parsed atom tasks " + new Date().);

			}

			parsedEntriesTime = System.currentTimeMillis();

			System.out.println("******************");
			System.out.println("Time spent fetching entries: " + (fetchedEntriesTime - startupTime));
			System.out.println("Time spent parsing entries: " + (parsedEntriesTime - fetchedEntriesTime));
			System.out.println("******************");



			return tasks;

		} catch (Throwable e) {

			e.printStackTrace();
			// TODO: handle exception
			return null;
		}
	}
}
