package org.eclipse.mylyn.mpl.ui;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.mylyn.tasks.core.AbstractAttributeFactory;
import org.eclipse.mylyn.tasks.core.RepositoryTaskAttribute;

public class MplAttributeFactory extends AbstractAttributeFactory {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2052597301414549854L;
	
	private static Map<String, Attribute> attributeByMplKey = new HashMap<String, Attribute>();
	
	private static Map<String, String> mplKeyByTaskKey = new HashMap<String, String>();

	
	public enum Attribute {
//		ASSIGNED_TO("assigned_to", "Assigned to:", RepositoryTaskAttribute.USER_CC, true, false), 
		UPDATED("updated","Last Modification:", RepositoryTaskAttribute.DATE_MODIFIED, true, true), 
		COMPONENT("component","Component:", RepositoryTaskAttribute.PRODUCT), 
		DESCRIPTION("description", "Description:", RepositoryTaskAttribute.DESCRIPTION, false, false), 
		KEY("key", "<used by search engine>", null, true), 
//		KEYWORDS(Key.KEYWORDS, "Keywords:", RepositoryTaskAttribute.KEYWORDS), 
//		MILESTONE(Key.MILESTONE, "Milestone:",	null), 
//		NEW_CC(RepositoryTaskAttribute.NEW_CC, "Add CC:"),
		ASSIGNED_TO("assigned_to", "Assigned to:", RepositoryTaskAttribute.USER_ASSIGNED, false, false),
		PRIORITY("priority", "Priority:", null), 
		REPORTER("reporter", "Reporter:", RepositoryTaskAttribute.USER_REPORTER, true, true),
		STATE("state", "State:", RepositoryTaskAttribute.STATUS, false, false), 
//		SEVERITY(Key.SEVERITY, "Severity:", null), 
//		STATUS(Key.STATUS, "Status:", RepositoryTaskAttribute.STATUS, false, true), 
		SUMMARY("summary", "Summary:", RepositoryTaskAttribute.SUMMARY, true), 
		CREATED("created", "Created:",RepositoryTaskAttribute.DATE_CREATION, true, true), 
		TYPE("type", "Type:", null), 
		VERSION("version","Version:", null),
		DUE_DATE("due_date","Due date:", RepositoryTaskAttribute.TYPE_DATE);
		//Project key.. + viewissueurl..
		
		
		private final boolean isHidden;

		private final boolean isReadOnly;

		private final String mplKey;

		private final String prettyName;

		private final String taskKey;

		Attribute(String mplKey, String prettyName, String taskKey, boolean hidden, boolean readonly) {
			this.mplKey = mplKey;
			this.taskKey = taskKey;
			this.prettyName = prettyName;
			this.isHidden = hidden;
			this.isReadOnly = readonly;
			
//			Issue issue = new Issue();			
//			issue.get

			attributeByMplKey.put(mplKey, this);
			if (taskKey != null) {
				mplKeyByTaskKey.put(taskKey, mplKey);
			}
		}

		Attribute(String key, String prettyName, String taskKey, boolean hidden) {
			this(key, prettyName, taskKey, hidden, false);
		}

		Attribute(String key, String prettyName, String taskKey) {
			this(key, prettyName, taskKey, false, false);
		}

		/**
		 * This is for Mylyn attributes that do not map to Mpl attributes.
		 */
		Attribute(String taskKey, String prettyName) {
			this(taskKey, prettyName, taskKey, true, false);
		}

		public String getTaskKey() {
			return taskKey;
		}

		public String getMplKey() {
			return mplKey;
		}

		public boolean isHidden() {
			return isHidden;
		}

		public boolean isReadOnly() {
			return isReadOnly;
		}

		@Override
		public String toString() {
			return prettyName;
		}
	}
	
	static {
		// make sure hash maps get initialized when class is loaded
		Attribute.values();
	}

	@Override
	public Date getDateForAttributeType(String attributeKey, String dateString) {
		// TODO Auto-generated method stub
		
		//TODO: Finish this!!
		Date date = new Date();		
		return date;
	}

	@Override
	public String getName(String key) {
		Attribute attribute = attributeByMplKey.get(key);
		// TODO if attribute == null it is probably a custom field: need 
		// to query custom field information from repository
		return (attribute != null) ? attribute.toString() : key;
	}

	@Override
	public boolean isHidden(String key) {
		// TODO Auto-generated method stub
		if (isInternalAttribute(key)) {
			return true;
		}

		Attribute attribute = attributeByMplKey.get(key);
		return (attribute != null) ? attribute.isHidden() : false;
	}

	@Override
	public boolean isReadOnly(String key) {
		// TODO Auto-generated method stub
		Attribute attribute = attributeByMplKey.get(key);
		return (attribute != null) ? attribute.isReadOnly() : false;
	}

	@Override
	public String mapCommonAttributeKey(String key) {
		String mplKey = mplKeyByTaskKey.get(key);
		return (mplKey != null) ? mplKey : key;
	}
	
	static boolean isInternalAttribute(String id) {
		return RepositoryTaskAttribute.COMMENT_NEW.equals(id) || RepositoryTaskAttribute.REMOVE_CC.equals(id)
				|| RepositoryTaskAttribute.NEW_CC.equals(id) || RepositoryTaskAttribute.ADD_SELF_CC.equals(id);
	}

}
