package org.eclipse.mylyn.mpl.ui;

import java.util.Date;

import no.miles.mpl.wrapper.datastructures.Issue;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Status;
import org.eclipse.mylyn.mpl.ui.MplAttributeFactory.Attribute;
import org.eclipse.mylyn.tasks.core.AbstractAttributeFactory;
import org.eclipse.mylyn.tasks.core.AbstractTaskDataHandler;
import org.eclipse.mylyn.tasks.core.RepositoryTaskAttribute;
import org.eclipse.mylyn.tasks.core.RepositoryTaskData;
import org.eclipse.mylyn.tasks.core.TaskRepository;


public class MplTaskDataHandler extends AbstractTaskDataHandler {

	private static final MplAttributeFactory attributeFactory = new MplAttributeFactory();

	private MplRepositoryConnector connector;

	public MplTaskDataHandler(MplRepositoryConnector mplRepositoryConnector) {
		// TODO Auto-generated constructor stub
		this.connector = mplRepositoryConnector;
	}

	@Override
	public AbstractAttributeFactory getAttributeFactory(String repositoryUrl,
			String repositoryKind, String taskKind) {
		// TODO Auto-generated method stub
		return attributeFactory;
	}

	@Override
	public AbstractAttributeFactory getAttributeFactory(
			RepositoryTaskData taskData) {
		// TODO Auto-generated method stub
		return getAttributeFactory(taskData.getRepositoryUrl(), taskData.getRepositoryKind(), taskData.getTaskKind());
	}

	@Override
	public RepositoryTaskData getTaskData(TaskRepository repository,
			String taskId, IProgressMonitor monitor) throws CoreException {
		// TODO Auto-generated method stub
		return downloadTaskData(repository, taskId);	
	}

	public RepositoryTaskData downloadTaskData(TaskRepository repository, String id) throws CoreException {
//		if (!MplRepositoryConnector.hasEditor(repository)) {
//		// offline mode is only supported for XML-RPC
//		return null;
//		}

		System.out.println("Should download taskdata..");
		try {
			RepositoryTaskData data = new RepositoryTaskData(attributeFactory, MplCorePlugin.REPOSITORY_KIND,
					repository.getUrl(), id + "");

			Issue issue = MplAtomLib.fetchIssue(repository.getUserName(), repository.getPassword(), repository.getUrl(), id);	
			createDefaultAttributes(repository, attributeFactory, data, true);
			updateTaskData(repository, attributeFactory, data, issue);
			return data;
		} catch (OperationCanceledException e) {
			e.printStackTrace();
			throw e;
		} catch (Exception e) {
			// TODO catch TracException
			e.printStackTrace();
			throw new CoreException(new Status(IStatus.ERROR, "mpl", "Couldn't download taskdata.."));	
//			throw new CoreException(TracCorePlugin.toStatus(e, repository));
		}
	}

	public static void updateTaskData(TaskRepository repository, AbstractAttributeFactory factory,
			RepositoryTaskData data, Issue issue) {
//		if (ticket.getCreated() != null) {
//		data.setAttributeValue(Attribute.TIME.getTracKey(), TracUtils.toTracTime(ticket.getCreated()) + "");
//		}

		System.out.println("updating task data..");

		Date lastChanged = null;
		if(issue.getUpdated() != null)
			lastChanged = issue.getUpdated();

//		data.addAttribute("description", RepositoryTaskAttribute.DESCRIPTION.);

//		data.addAttributeValue(Attribute.DESCRIPTION.getTaskKey(), issue.getDescription());
		data.addAttributeValue(Attribute.ASSIGNED_TO.getTaskKey(), issue.getAssigned_to());
		data.addAttributeValue(Attribute.PRIORITY.getTaskKey(), issue.getPriority());
		data.addAttributeValue(Attribute.STATE.getTaskKey(), issue.getState());
		data.addAttributeValue(Attribute.TYPE.getTaskKey(), issue.getType());		

		data.setDescription(issue.getDescription());

//		data.addAttribute(Attribute.DESCRIPTION.getTaskKey(), issue.getDescription());

//		Map<String, String> valueByKey = ticket.getValues();
//		for (String key : valueByKey.keySet()) {
//		if (Key.CC.getKey().equals(key)) {
//		StringTokenizer t = new StringTokenizer(valueByKey.get(key), CC_DELIMETER);
//		while (t.hasMoreTokens()) {
//		data.addAttributeValue(key, t.nextToken());
//		}
//		} else {
//		data.setAttributeValue(key, valueByKey.get(key));
//		}
//		}

//		TracComment[] comments = ticket.getComments();
//		if (comments != null) {
//		for (int i = 0; i < comments.length; i++) {
//		if (!"comment".equals(comments[i].getField()) || "".equals(comments[i].getNewValue())) {
//		continue;
//		}

//		TaskComment taskComment = new TaskComment(factory, data.getComments().size() + 1);
//		taskComment.setAttributeValue(RepositoryTaskAttribute.COMMENT_AUTHOR, comments[i].getAuthor());
//		taskComment.setAttributeValue(RepositoryTaskAttribute.COMMENT_DATE, comments[i].getCreated().toString());
//		taskComment.setAttributeValue(RepositoryTaskAttribute.COMMENT_TEXT, comments[i].getNewValue());
//		data.addComment(taskComment);
//		}
//		}

//		TracAttachment[] attachments = ticket.getAttachments();
//		if (attachments != null) {
//		for (int i = 0; i < attachments.length; i++) {
//		RepositoryAttachment taskAttachment = new RepositoryAttachment(factory);
//		taskAttachment.setCreator(attachments[i].getAuthor());
//		taskAttachment.setRepositoryKind(TracCorePlugin.REPOSITORY_KIND);
//		taskAttachment.setRepositoryUrl(repository.getUrl());
//		taskAttachment.setTaskId("" + ticket.getId());
//		taskAttachment.setAttributeValue(Attribute.DESCRIPTION.getTracKey(), attachments[i].getDescription());
//		taskAttachment.setAttributeValue(RepositoryTaskAttribute.ATTACHMENT_FILENAME,
//		attachments[i].getFilename());
//		taskAttachment.setAttributeValue(RepositoryTaskAttribute.USER_OWNER, attachments[i].getAuthor());
//		if (attachments[i].getCreated() != null) {
//		if (lastChanged == null || attachments[i].getCreated().after(lastChanged)) {
//		lastChanged = attachments[i].getCreated(); 
//		}

//		taskAttachment.setAttributeValue(RepositoryTaskAttribute.ATTACHMENT_DATE, attachments[i].getCreated()
//		.toString());
//		}
//		taskAttachment.setAttributeValue(RepositoryTaskAttribute.ATTACHMENT_URL, repository.getUrl()
//		+ ITracClient.TICKET_ATTACHMENT_URL + ticket.getId() + "/" + attachments[i].getFilename());
//		taskAttachment.setAttributeValue(RepositoryTaskAttribute.ATTACHMENT_ID, i + "");
//		data.addAttachment(taskAttachment);
//		}
//		}

//		String[] actions = ticket.getActions();
//		if (actions != null) {
//		// add operations in a defined order
//		List<String> actionList = new ArrayList<String>(Arrays.asList(actions));
//		addOperation(repository, data, ticket, actionList, "leave");
//		addOperation(repository, data, ticket, actionList, "accept");
//		addOperation(repository, data, ticket, actionList, "resolve");
//		addOperation(repository, data, ticket, actionList, "reassign");
//		addOperation(repository, data, ticket, actionList, "reopen");
//		}

		if (lastChanged != null) {
			data.setAttributeValue(Attribute.UPDATED.getMplKey(), lastChanged.toString());
		}
	}

	@Override
	public boolean initializeTaskData(TaskRepository repository,
			RepositoryTaskData data, IProgressMonitor monitor)
	throws CoreException {
		// TODO Auto-generated method stub
		try {

			System.out.println("Should initialize task data..");
//			ITracClient client = connector.getClientManager().getRepository(repository);
//			client.updateAttributes(new NullProgressMonitor(), false);
			createDefaultAttributes(repository, attributeFactory, data, false);
			return true;
		} catch (OperationCanceledException e) {
			throw e;
		} catch (Exception e) {
			// TODO catch TracException
			e.printStackTrace();
			throw new CoreException(new Status(IStatus.ERROR, "mpl", "Couldn't initialize task.."));
//			throw new CoreException(TracCorePlugin.toStatus(e, repository));
		}
	}

	@Override
	public String postTaskData(TaskRepository repository,
			RepositoryTaskData taskData, IProgressMonitor monitor)
	throws CoreException {
		// Should post a new task to the Atom feed

		if(taskData.isNew()){

			try{

				String id = MplAtomLib.postTask(repository.getUserName(), repository.getPassword(), repository.getUrl(), taskData);
				System.out.println("id: " + id);
				monitor.setTaskName(id);
				return id;

			} catch (Exception e) {
				e.printStackTrace();
				throw new CoreException(new Status(IStatus.ERROR, "mpl", "error posting task " + e.getMessage()));
			}

		} else {

			System.out.println("Should update the task!");

			//TODO: Update the task!!
			return null;
		}
//		return null;
	}

	public static void createDefaultAttributes(TaskRepository repository, AbstractAttributeFactory factory, RepositoryTaskData data, boolean existingTask) {
//		TracTicketField[] fields = client.getTicketFields();

//		if (existingTask) {
//		createAttribute(factory, data, Attribute.STATUS, client.getTicketStatus());
//		createAttribute(factory, data, Attribute.RESOLUTION, client.getTicketResolutions());
//		}

//		createAttribute(factory, data, Attribute.COMPONENT, client.getComponents());
//		createAttribute(factory, data, Attribute.VERSION, client.getVersions(), true);
//		createAttribute(factory, data, Attribute.PRIORITY, client.getPriorities());
//		createAttribute(factory, data, Attribute.SEVERITY, client.getSeverities());

//		createAttribute(factory, data, Attribute.TYPE, client.getTicketTypes());
//		RepositoryTaskAttribute attribute = createAttribute(factory, data, Attribute.OWNER);
//		if (!existingTask) {
//		attribute.setReadOnly(false);
//		}
//		createAttribute(factory, data, Attribute.MILESTONE, client.getMilestones(), true);
//		if (existingTask) {
//		createAttribute(factory, data, Attribute.REPORTER);
//		}

//		if (existingTask) {
//		createAttribute(factory, data, Attribute.NEW_CC);
//		}
//		createAttribute(factory, data, Attribute.CC);
//		createAttribute(factory, data, Attribute.KEYWORDS);

//		if (!existingTask) {
//		createAttribute(factory, data, Attribute.SUMMARY);
//		createAttribute(factory, data, Attribute.DESCRIPTION);
//		}

//		if (fields != null) {
//		for (TracTicketField field : fields) {
//		if (field.isCustom()) {
//		createAttribute(data, field);
//		}
//		}
//		}
	}

	private static RepositoryTaskAttribute createAttribute(AbstractAttributeFactory factory, RepositoryTaskData data,
			Attribute attribute) {

		RepositoryTaskAttribute attr = new RepositoryTaskAttribute(attribute.getMplKey(), attribute.name(), false);


		return attr;
//		return createAttribute(factory, data, attribute, values, false);
	}

//	private static void createAttribute(RepositoryTaskData data, TracTicketField field) {
//	RepositoryTaskAttribute attr = new RepositoryTaskAttribute(field.getName(), field.getLabel(), false);
//	if (field.getType() == TracTicketField.Type.CHECKBOX) {
//	// attr.addOption("True", "1");
//	// attr.addOption("False", "0");
//	attr.addOption("1", "1");
//	attr.addOption("0", "0");

//	if (field.getDefaultValue() != null) {
//	attr.setValue(field.getDefaultValue());
//	}
//	} else if (field.getType() == TracTicketField.Type.SELECT || field.getType() == TracTicketField.Type.RADIO) {
//	String[] values = field.getOptions();
//	if (values != null && values.length > 0) {
//	if (field.isOptional()) {
//	attr.addOption("", "");
//	}
//	for (int i = 0; i < values.length; i++) {
//	attr.addOption(values[i], values[i]);
//	}

//	if (field.getDefaultValue() != null) {
//	try {
//	int index = Integer.parseInt(field.getDefaultValue());
//	if (index > 0 && index < values.length) {
//	attr.setValue(values[index]);
//	}
//	} catch (NumberFormatException e) {
//	for (int i = 0; i < values.length; i++) {
//	if (field.getDefaultValue().equals(values[i].toString())) {
//	attr.setValue(values[i]);
//	break;
//	}
//	}
//	}
//	}
//	}
//	} else {
//	if (field.getDefaultValue() != null) {
//	attr.setValue(field.getDefaultValue());
//	}
//	}
//	data.addAttribute(attr.getId(), attr);
//	}

//	private static RepositoryTaskAttribute createAttribute(AbstractAttributeFactory factory, RepositoryTaskData data,
//	Attribute attribute, Object[] values, boolean allowEmtpy) {
//	RepositoryTaskAttribute attr = factory.createAttribute(attribute.getTracKey());
//	if (values != null && values.length > 0) {
//	if (allowEmtpy) {
//	attr.addOption("", "");
//	}
//	for (int i = 0; i < values.length; i++) {
//	attr.addOption(values[i].toString(), values[i].toString());
//	}
//	} else {
//	attr.setHidden(true);
//	attr.setReadOnly(true);
//	}
//	data.addAttribute(attribute.getTracKey(), attr);
//	return attr;
//	}

//	private static RepositoryTaskAttribute createAttribute(AbstractAttributeFactory factory, RepositoryTaskData data,
//	Attribute attribute) {
//	RepositoryTaskAttribute attr = factory.createAttribute(attribute.getTracKey());
//	data.addAttribute(attribute.getTracKey(), attr);
//	return attr;
//	}

//	private static RepositoryTaskAttribute createAttribute(AbstractAttributeFactory factory, RepositoryTaskData data,
//	Attribute attribute, Object[] values) {
//	return createAttribute(factory, data, attribute, values, false);
//	}

}
