package no.miles.mpl.layers.app;

import no.miles.mpl.wrapper.*;

import java.util.ArrayList;
import java.util.Properties;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.sql.*;

public class Projects {

	public static ArrayList<Project> getProjects()
	{
		ArrayList<Project> projects = new ArrayList<Project>();

		Connection conn = getConnection();
		try {
			Statement st = conn.createStatement();
			ResultSet rs = st.executeQuery("SELECT * FROM projects");
			while (rs.next()) {
				String name = rs.getString("name");
				String bugtracker = rs.getString("bugtracker");
				String url = rs.getString("url");

				Project project = new Project(name, name);
				project.setBugtrackerName(bugtracker);
				project.setBugtrackerUrl(url);
				
				projects.add(project);
				
				System.out.println(name + " " + bugtracker + " " + url);
			}
			rs.close();
			st.close();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();			
		}		
		return projects;
	}

	public static Project getProject(String projectName)
	{
		Project project = new Project(projectName, projectName);		

		project.setIssues(getIssues(project));		

		return project;
	}


	public static ArrayList<Issue> getIssues(Project project){

		BugTrackerWrapperInterface bugTracker = null;
		Connection conn = getConnection();
		PreparedStatement st;
		try {
			st = conn.prepareStatement("SELECT * FROM projects WHERE name = ?");
			st.setString(1, project.getKey());
			ResultSet rs = st.executeQuery();
			while (rs.next()) {
				String name = rs.getString("name");
				String bugtracker = rs.getString("bugtracker");
				String url = rs.getString("url");
				
				if(bugtracker.equals("jira")){
					bugTracker = new JiraWrapper(url);
				}
				else if(bugtracker.equals("trac")){
					bugTracker = new TracWrapper(url);
				}
				
				System.out.println(name + " " + bugtracker + " " + url);
			}
			rs.close();
			st.close();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}		
		
		ArrayList<Issue> issues = bugTracker.getIssues("test", "test", project.getKey());		
		return issues;
	}


	public static boolean projectExists(String projectName){

		Connection conn = getConnection();
		PreparedStatement st;
		try {
			st = conn.prepareStatement("SELECT name FROM projects WHERE name = ?");

			st.setString(1, projectName);
			ResultSet rs = st.executeQuery();
			while (rs.next()) {
				if(rs.getString("name").equals(projectName))
					return true;
			}
			rs.close();
			st.close();

		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return false;
	}

	private static Connection getConnection()
	{
		Connection conn;
		try {
			Class.forName("org.postgresql.Driver");

//			Properties props = new Properties();		
//			props.load(new FileInputStream("/target/no.miles.mpl.layers.app-1.0-SNAPSHOT/WEB-INF/classes/db.properties"));			
//			String url = props.getProperty("url");
//			String user = props.getProperty("user");
//			String password = props.getProperty("password");

			String url = "jdbc:postgresql:mpldb";
			String user = "mpl";
			String password = "mpl";
			conn = DriverManager.getConnection(url, user, password);

		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}		

		return conn;
	}



}
