package no.miles.mpl.restful;

import java.io.IOException;

import org.restlet.Context;
import org.restlet.data.MediaType;
import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.data.Status;
import org.restlet.resource.DomRepresentation;
import org.restlet.resource.Representation;
import org.restlet.resource.Resource;
import org.restlet.resource.Variant;
import org.w3c.dom.Document;

public class Config extends Resource {	

	private String projectKey;

	public Config(Context context, Request request, Response response) {
		super(context, request, response);

		this.projectKey = (String) request.getAttributes().get("project");			
		getVariants().add(new Variant(MediaType.TEXT_XML));
	}

	@Override
	public boolean allowPut() {
		// TODO Auto-generated method stub
		return true;
	}

	@Override
	public void put(Representation entity) {

		if(entity.getMediaType().equals(MediaType.TEXT_XML))
		{
//			out.println(entity);			
			DomRepresentation domRep = new DomRepresentation(entity);	


			domRep.setNamespaceAware(true);

			Document configDoc;
			try {
				configDoc = domRep.getDocument();

				boolean valid = Common.validateDocument(configDoc, Common.XSD_URI);

				if(!valid) {
					getResponse().setStatus(Status.CLIENT_ERROR_NOT_ACCEPTABLE, "Validation error");
					return;
				}

				boolean storedSuccessfully = Common.storeDocument(configDoc, Common.ROOT_PATH + projectKey + "_config.xml");

				if(!storedSuccessfully)	{
					getResponse().setStatus(Status.CLIENT_ERROR_NOT_ACCEPTABLE, "Could not save to file");
					return;
				}		

			} catch (IOException e) {
				e.printStackTrace();
				getResponse().setStatus(Status.CLIENT_ERROR_BAD_REQUEST);
			}
		}
	}

	@Override
	public Representation getRepresentation(Variant variant) {
		Representation result = null;

		if(variant.getMediaType().equals(MediaType.TEXT_XML)){

			Document doc = Common.getDocument(Common.ROOT_PATH + projectKey + "_config.xml");

			if(doc == null){
				doc = Common.getDocument(Common.CONFIG_PATH);

				if(doc == null){
					getResponse().setStatus(Status.CLIENT_ERROR_NOT_FOUND);
					return result;
				}				
			}
			DomRepresentation domRep = new DomRepresentation(MediaType.TEXT_XML, doc);
			result = domRep;
		}
		return result;
	}

}
