package no.miles.mpl.restful;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Properties;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import no.miles.mpl.wrapper.BugTrackerWrapperInterface;
import no.miles.mpl.wrapper.datastructures.*;
import no.miles.mpl.wrapper.exception.BadRequestException;
import no.miles.mpl.wrapper.exception.InternalServerErrorException;
import no.miles.mpl.wrapper.JiraWrapper;
import no.miles.mpl.wrapper.TracWrapper;

import org.restlet.Context;
import org.restlet.data.MediaType;
import org.restlet.data.Reference;
import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.data.Status;
import org.restlet.resource.DomRepresentation;
import org.restlet.resource.Representation;
import org.restlet.resource.Resource;
import org.restlet.resource.StringRepresentation;
import org.restlet.resource.Variant;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
public class RestIssues extends Resource{

	protected BugTrackerWrapperInterface bugTracker;
	protected Properties properties;
	protected String uri;
	protected String projectKey;
	protected String username;
	protected String password;
	protected String root;

	/**
	 * Constructor
	 * @param context
	 * @param request
	 * @param response
	 */
	public RestIssues(Context context, Request request, Response response) {
		super(context, request, response);

		this.projectKey = (String) request.getAttributes().get("project");	

		properties = new Properties();
		try {
			properties.load(new FileInputStream("target/classes/bugtracker.properties"));
		} catch (IOException e) {
		}

		this.username = properties.getProperty("username");
		this.password = properties.getProperty("password");

		initBugTracker();

		getVariants().add(new Variant(MediaType.TEXT_XML));

		this.uri = request.getResourceRef().toString();
		this.root = request.getRootRef().toString();
		
		System.out.println(this.root + " " + this.uri);
	}

	/**
	 * Initialize the chosen bugtracker
	 */
	protected void initBugTracker()
	{	
		bugTracker = null;
		
//		Document configDoc = Common.getDocument(Common.ROOT_PATH + projectKey + "_config.xml");		
		
		Document configDoc = Common.getDocument(Common.LOCALHOST + Common.PROJECTS + projectKey + Common.CONFIG);
		
		System.out.println(configDoc.getDocumentElement().getTextContent());
		
		String configBugTracker = Common.getNodeValue(configDoc, "/mpl:config/mpl:bugtracker/mpl:name", Common.MPL_PREFIX, Common.MPL_NS);	
		String projectRoot = Common.getNodeValue(configDoc, "/mpl:config/mpl:bugtracker/mpl:rooturl", Common.MPL_PREFIX, Common.MPL_NS);	

		
		//Initialize Jira
		if(configBugTracker.equalsIgnoreCase("jira"))
		{
			bugTracker = new JiraWrapper(projectRoot);				
			System.out.println("Valgt Jira som bugTracker");
		}
		//Initialize Trac
		else if(configBugTracker.equalsIgnoreCase("trac"))
		{
			bugTracker = new TracWrapper(projectRoot);
			System.out.println("Valgt Trac som bugTracker");
		}
		else {
			System.out.println("Not initialised");
		}
	}


	@Override
	public Representation getRepresentation(Variant variant) {
		Representation result = null;

		if(variant.getMediaType().equals(MediaType.TEXT_XML)){

			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

			try {
				DocumentBuilder db = dbf.newDocumentBuilder();
				Document doc = db.newDocument();
				Node issuesNode = doc.createElement("issues");
				doc.appendChild(issuesNode);

				if(bugTracker != null)
				{
					Issue[] issues = bugTracker.getIssues(username, password, projectKey);
					for (Issue issue : issues) {

						Node issueNode = doc.createElement("issue");

//						Attr id = doc.createAttribute("id");
//						id.setTextContent(issue.getKey());
//						issueNode.getAttributes().item(0).appendChild(id);
//						issueNode.getAttributes().setNamedItem(id);

						Node urlNode = doc.createElement("url");
						urlNode.setTextContent(uri + issue.getKey());

						issueNode.appendChild(urlNode);

						Node summary = doc.createElement("summary");
						summary.setTextContent(issue.getSummary());

						issueNode.appendChild(summary);     		        			
						issuesNode.appendChild(issueNode);

					}
				} else {
					getResponse().setStatus(Status.CLIENT_ERROR_NOT_FOUND);
					return result;
				}
				

				DomRepresentation domRep = new DomRepresentation(MediaType.TEXT_XML, doc);
				result = domRep;

			} catch (ParserConfigurationException e1) {
				e1.printStackTrace();
			} catch (BadRequestException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InternalServerErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		else {
			result = new StringRepresentation("Du ba om " + variant.getMediaType().toString());
		}

		return result;
	}


	@Override
	public void handleGet() {
		// Make sure that the Uri ends with a "/" without changing the query.
		Reference ref = getRequest().getResourceRef();
		if (!ref.getPath().endsWith("/")) {
			ref.setPath(ref.getPath() + "/");
			getResponse().redirectPermanent(ref);
		} else {
			super.handleGet();
		}
	}
}
