package no.miles.mpl.semant.app.endpoint.datahandlers;

import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Element;
import no.miles.mpl.semantic.app.endpoint.entries.IssueEntry;
import no.miles.mpl.wrapper.BugTrackerWrapperInterface;
import no.miles.mpl.wrapper.datastructures.Issue;
import no.miles.mpl.wrapper.datastructures.IssueStates;
import no.miles.mpl.wrapper.datastructures.Project;
import no.miles.mpl.wrapper.exception.BadRequestException;
import no.miles.mpl.wrapper.exception.InternalServerErrorException;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.InfModel;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.ModelMaker;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.reasoner.Reasoner;
import com.hp.hpl.jena.reasoner.rulesys.OWLFBRuleReasonerFactory;
import com.hp.hpl.jena.reasoner.rulesys.RDFSFBRuleReasoner;
import com.hp.hpl.jena.reasoner.rulesys.RDFSFBRuleReasonerFactory;
import com.hp.hpl.jena.shared.DoesNotExistException;
import com.hp.hpl.jena.shared.Lock;
import com.sun.syndication.feed.atom.Content;
import com.sun.syndication.feed.atom.Entry;
import com.sun.syndication.feed.atom.Link;

public class Issues {

	public static String MODELPREFIX = "cachedissuesfor";

	public static String EHCACHEPREFIX = "ehcachedissuesforproject";

	public static String EH_ENTRY_CACHEPREFIX = "ehcachedentriesforproject";

	/**
	 * Delete issue - general method
	 * @param username
	 * @param password
	 * @param projectKey
	 * @param issueKey
	 * @throws BadRequestException
	 * @throws InternalServerErrorException
	 */
	public static void deleteIssue(String username, String password, String projectKey, String issueKey) throws BadRequestException, InternalServerErrorException{

		if(projectKey == null || issueKey == null)
			throw new BadRequestException("Bad title/key!");		

		if(getIssue(username, password, projectKey, issueKey) == null)
			return; //Already deleted?

		BugTrackerWrapperInterface bugTracker = Projects.getBugTracker(null, projectKey, username, password);
		if(bugTracker != null){

			bugTracker.deleteIssue(username, password, projectKey, issueKey);
			return;
		}

		throw new BadRequestException("Bad projectkey?");

	}

	/**
	 * Update issue from (default n3)
	 * @param username
	 * @param password
	 * @param projectKey
	 * @param entry
	 * @return
	 * @throws BadRequestException
	 * @throws InternalServerErrorException
	 */
	public static Issue updateIssue(String username, String password, String projectKey, Entry entry) throws BadRequestException, InternalServerErrorException
	{
		Content content = (Content)entry.getContents().get(0);

		if(content != null && content.getValue() != null && content.getType().equalsIgnoreCase(Content.TEXT)){

			String n3 = content.getValue();			

			Issue issue;
			try {
				issue = Issue.parseOntology(n3, projectKey, entry.getTitle(), null);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new BadRequestException(e.getMessage());
			}

			BugTrackerWrapperInterface bugTracker = Projects.getBugTracker(null, projectKey, username, password);

			Issue addedIssue = bugTracker.updateIssue(username, password, projectKey, issue);

			return addedIssue;
		}

		return null;
	}

	/**
	 * Update issue from XML
	 * @param username
	 * @param password
	 * @param projectKey
	 * @param entry
	 * @return
	 * @throws BadRequestException
	 * @throws InternalServerErrorException
	 */
	public static Issue updateIssueFromXml(String username, String password, String projectKey, Entry entry) throws BadRequestException, InternalServerErrorException
	{
		Content content = (Content)entry.getContents().get(0);

		if(content != null && content.getValue() != null && content.getType().equalsIgnoreCase(Content.XML)){

//			String n3 = content.getValue();			

//			Issue issue = Issue.parseOntology(n3, projectKey, entry.getTitle(), null);

//			BugTrackerWrapperInterface bugTracker = Projects.getBugTracker(projectKey);

//			Issue addedIssue = bugTracker.updateIssue(username, password, projectKey, issue);

//			return addedIssue;
		}

		return null;
	}

	/**
	 * Add issue (from N3)
	 * @param username
	 * @param password
	 * @param projectKey
	 * @param entry
	 * @return
	 * @throws BadRequestException
	 * @throws InternalServerErrorException
	 */
	public static Issue addIssue(String username, String password, String projectKey, Entry entry) throws BadRequestException, InternalServerErrorException
	{

		BugTrackerWrapperInterface bugTracker = Projects.getBugTracker(null, projectKey, username, password);

		/**
		 * If no content - try to use pure atom properties
		 */
		if(entry.getContents() == null || entry.getContents().size() == 0) {

			if(entry.getTitle() != null){

				Issue issue = new Issue();
				issue.setSummary(entry.getTitle());


				if(entry.getSummary() != null){
					issue.setDescription(entry.getSummary().getValue());
				}

				issue.setReported_by(username);

				try {
					issue.setState(IssueStates.STATE_NEW);
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					throw new InternalServerErrorException(e.getMessage());
				}

				issue.setCreated(new Date());

				Issue addedIssue = bugTracker.addIssue(username, password, projectKey, issue);
				return addedIssue;				
			}			

			throw new BadRequestException("Content is empty, also missing title");
		}

		Content content = (Content)entry.getContents().get(0);

		/**
		 * If RDF content - parse the issue..
		 */
		if(content != null && content.getValue() != null && content.getType().equalsIgnoreCase("text/rdf+n3")){

//			System.out.println("Trying to parse n3 content: ");

			String n3 = content.getValue();	

//			System.out.println(n3);

			Issue issue;
			try {
				issue = Issue.parseOntology(n3, projectKey);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new BadRequestException(e.getMessage());
			}



			Issue addedIssue = bugTracker.addIssue(username, password, projectKey, issue);

			return addedIssue;
		}	

		/**
		 * If XML content - parse the issue..
		 */
		else if(content != null && content.getValue() != null && content.getType().equalsIgnoreCase("application/xml")){

//			System.out.println("Trying to parse content");

//			String xml = content.getValue();			

//			Issue issue;
//			try {
//			issue = Issue.parseXml(xml);
//			} catch (Exception e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//			throw new BadRequestException(e.getMessage());
//			}

//			Issue addedIssue = bugTracker.addIssue(username, password, projectKey, issue);

//			return addedIssue;

			throw new BadRequestException("XML support not implemented!");

		}	

		throw new BadRequestException("Content is empty or not RDF or XML!");
	}

	/**
	 * Get an issue
	 * @param username
	 * @param password
	 * @param projectKey
	 * @param issueKey
	 * @return
	 * @throws BadRequestException
	 * @throws InternalServerErrorException
	 */
	public static Issue getIssue(String username, String password, String projectKey, String issueKey) throws BadRequestException, InternalServerErrorException
	{
		BugTrackerWrapperInterface bugTracker = Projects.getBugTracker(null, projectKey, username, password);		
		Issue issue = bugTracker.getIssue(username, password, projectKey, issueKey);
		return issue;
	}

	/**
	 * Get all issues for a project
	 * @param username
	 * @param password
	 * @param project
	 * @return
	 * @throws BadRequestException
	 * @throws InternalServerErrorException
	 */
	public static Issue[] getIssues(String username, String password, Project project) throws BadRequestException, InternalServerErrorException{

		BugTrackerWrapperInterface bugTracker = Projects.getBugTracker(null, project.getName(), username, password);

//		System.out.println("Fetching issues..."  + new Date().toString());

		Issue[] issues = bugTracker.getIssues(username, password, project.getName());	

//		updateCache(issues, project);

		return issues;

	}

	public static ArrayList<Entry> getEntries(String username, String password, Project project, String uriPrefix) throws InternalServerErrorException, BadRequestException{

		ArrayList<Entry> entries = new ArrayList<Entry>();

		BugTrackerWrapperInterface bugTracker = Projects.getBugTracker(null, project.getName(), username, password);

		String cacheName = EH_ENTRY_CACHEPREFIX + project.getName();

		CacheManager singletonManager = CacheManagerFactory.getEntryCacheManager();


		ArrayList<Entry> allEntries = null;

		/**
		 * Instantiate the cache if it needs to be done..
		 */
		if(!singletonManager.cacheExists(cacheName)){

			allEntries = new ArrayList<Entry>();

			System.out.println("Creating cache");			

			Cache issueCache = new Cache(cacheName, 5000, false, true, 0, 0);

			singletonManager.addCache(issueCache);

			Issue[] allIssues = bugTracker.getIssues(username, password, project.getName());	

			for (Issue issue : allIssues) {

				Entry entry = new IssueEntry(uriPrefix + project.getName() + "/issues/", issue);

				allEntries.add(entry);

			}			
		} 	


		Cache cachedEntries = singletonManager.getCache(cacheName);

		if(allEntries != null){

			for (Entry entry : allEntries) {

//				String key = issue.getKey();

				String entryId = entry.getId();

				String issueEntryPattern = "/projects/\\w+/issues/(\\d+)";

				Pattern pattern = Pattern.compile(issueEntryPattern);

//				System.out.println("entryId for match: " + entryId);

				Matcher matcher = pattern.matcher(entryId);

				while(matcher.find()){

					String issueId = matcher.group(1);

//					System.out.println("mathced: " + issueId);

					Element entryElement = new Element(issueId, entry);
					cachedEntries.put(entryElement);

				}

			}

		} else {

			CacheManagerFactory.setLastRefreshedDate(cacheName, new Date());

			long beforedelete = System.currentTimeMillis();

			/**
			 * Remove deleted issues from cache
			 */

			ArrayList<String> allIssueIds = bugTracker.getIssueKeys(username, password, project.getName());		

			List<String> cachedKeys = cachedEntries.getKeys();
			for (String cachedKey : cachedKeys) {

				if(!allIssueIds.contains(cachedKey)){					
					cachedEntries.remove(cachedKey);					
				}
			}
			

			long afterdelete = System.currentTimeMillis();

			System.out.println("Spent " + (afterdelete - beforedelete) + " deleting issues");


			/**
			 * Update the cache..
			 */

			long beforeFecthingUpdatedIssues = System.currentTimeMillis();



			Date lr = CacheManagerFactory.getLastRefreshedDate(cacheName);


			ArrayList<String> updatedIssues = bugTracker.getUpdatedIssueKeys(username, password, project.getName(), lr);
			for (String updatedIssueKey : updatedIssues) {

				try {
					Issue issue = bugTracker.getIssue(username, password, project.getName(), updatedIssueKey);

					Entry entry = new IssueEntry(uriPrefix, issue);

					String key = issue.getKey();					
					Element entryElement = new Element(key, issue);					
					cachedEntries.put(entryElement);				

				} catch (BadRequestException e) {

					//Assume that this issue is deleted..					
					cachedEntries.remove(updatedIssueKey);
				}				
			}	

			CacheManagerFactory.setLastRefreshedDate(cacheName, new Date());

			long afterFecthingUpdatedIssues = System.currentTimeMillis();

			System.out.println("Spent " + (afterFecthingUpdatedIssues - beforeFecthingUpdatedIssues) + " fetching updated issues");

		}

		System.out.println("Fetching keys..?");

		List<Object> keys = cachedEntries.getKeys();

		for (Object key : keys) {

			Element elem = cachedEntries.get(key);

			Entry entry = (Entry) elem.getObjectValue();

//			System.out.println(key + " : "+ entry.getId());

			entries.add(entry);

		}

		return entries;		
	}


	public static Issue[] getEhCachedIssues(String username, String password, Project project) throws InternalServerErrorException, BadRequestException {

		BugTrackerWrapperInterface bugTracker = Projects.getBugTracker(null, project.getName(), username, password);

//		System.out.println("Fetching issues..."  + new Date().toString());

		String cacheName = EHCACHEPREFIX + project.getName();

		CacheManager singletonManager = CacheManagerFactory.getCacheManager();

		Issue[] allIssues = null;

		/**
		 * Instantiate the cache if it needs to be done..
		 */
		if(!singletonManager.cacheExists(cacheName)){

			System.out.println("Instantiating cache");			

			Cache issueCache = new Cache(cacheName, 5000, false, true, 0, 0);

			singletonManager.addCache(issueCache);

			allIssues = bugTracker.getIssues(username, password, project.getName());			

		} 

		Cache cachedIssues = singletonManager.getCache(cacheName);

		if(allIssues != null){

			for (Issue issue : allIssues) {

				String key = issue.getKey();

				Element issueElement = new Element(key, issue);

				cachedIssues.put(issueElement);

			}

			CacheManagerFactory.setLastRefreshedDate(cacheName, new Date());


		} else {	

			long beforedelete = System.currentTimeMillis();

			/**
			 * Remove deleted issues from cache
			 */

			ArrayList<String> allIssueIds = bugTracker.getIssueKeys(username, password, project.getName());		

			List<String> cachedKeys = cachedIssues.getKeys();
			for (String cachedKey : cachedKeys) {

				if(!allIssueIds.contains(cachedKey)){					
					cachedIssues.remove(cachedKey);					
				}


			}

			long afterdelete = System.currentTimeMillis();

			System.out.println("Spent " + (afterdelete - beforedelete) + " deleting issues");


			/**
			 * Update the cache..
			 */

			long beforeFecthingUpdatedIssues = System.currentTimeMillis();



			Date lr = CacheManagerFactory.getLastRefreshedDate(cacheName);


			ArrayList<String> updatedIssues = bugTracker.getUpdatedIssueKeys(username, password, project.getName(), lr);
			for (String updatedIssueKey : updatedIssues) {

				try {
					Issue issue = bugTracker.getIssue(username, password, project.getName(), updatedIssueKey);

					String key = issue.getKey();					
					Element issueElement = new Element(key, issue);					
					cachedIssues.put(issueElement);				

				} catch (BadRequestException e) {

					//Assume that this issue is deleted..					
					cachedIssues.remove(updatedIssueKey);
				}				
			}	

			CacheManagerFactory.setLastRefreshedDate(cacheName, new Date());

			long afterFecthingUpdatedIssues = System.currentTimeMillis();

			System.out.println("Spent " + (afterFecthingUpdatedIssues - beforeFecthingUpdatedIssues) + " fetching updated issues");

		}


		List<Object> keys = cachedIssues.getKeys();

		Issue[] issues = new Issue[keys.size()];		

		for(int i = 0; i < keys.size(); i++){

			Object issueKey = keys.get(i);

			Element elem = cachedIssues.get(issueKey);

			Issue issue = (Issue) elem.getObjectValue();

			issues[i] = issue;

		}


//		updateCache(issues, project);

		return issues;



	}


	/**
	 * Get cached URIs..
	 * @param modelName
	 * @return
	 * @throws InternalServerErrorException
	 * @throws BadRequestException
	 */
	private static ArrayList<String> getCachedIssueUris(String modelName) throws InternalServerErrorException, BadRequestException{

//		System.out.println("Fetching cached uris..	");

		ModelMaker maker = JenaHandler.getModelMaker();
		Model model = maker.getModel(modelName);

		ArrayList<String> issueUris = new ArrayList<String>();

		if(model == null){
			throw new BadRequestException("No issues cached for this project");
		}

		model.enterCriticalSection(Lock.READ);	

//		OutputStream out = new ByteArrayOutputStream();
//		BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
//		model.write(bufferedOut, "N3");
//		String n3 = out.toString();	


		/**
		 * We don't want to overwrite 
		 */
		Model metaModel = ModelFactory.createOntologyModel();

		Model unionModel = ModelFactory.createUnion(model, metaModel);

//		System.out.println("Model " + modelName + " before inference.." + n3);

		try{

			JenaHandler.loadN3(metaModel, JenaHandler.BAETLEN3);			

			Reasoner reasoner = RDFSFBRuleReasonerFactory.theInstance().create(null);
//			Reasoner reasoner = OWLFBRuleReasonerFactory.theInstance().create(null);
			InfModel inf = ModelFactory.createInfModel(reasoner, unionModel);

			if(inf == null){
				throw new InternalServerErrorException("Could not create inferred model");
			}
//			inf.abort();

			String fetchIssueKeys = 		
				"PREFIX rdf: <" + Issue.RDFNS + ">" +
				"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" +
				"PREFIX owl:  <http://www.w3.org/2002/07/owl#>" +
				"PREFIX baetle: <" + Issue.BAETLENS + ">" +
				"PREFIX mpl: <" + Issue.MPLNS + ">" +
				"PREFIX foaf: <" + Issue.FOAFNS +">" +
				"PREFIX doap: <" + Issue.DOAPNS +">" +
				"PREFIX sioc: <" + Issue.SIOCNS +">" +
				"PREFIX wf: <" + Issue.WFNS +">" +
				"SELECT ?issue " +
				"WHERE {" +
				" ?issue rdf:type baetle:Issue" +
				"} " +
				"";	

			Query query = QueryFactory.create(fetchIssueKeys);

			// Execute the query and obtain results
			QueryExecution qe = QueryExecutionFactory.create(query, inf);
//			QueryExecution qe = QueryExecutionFactory.create(query, model);
			ResultSet rs = qe.execSelect();

			try{

				while(rs.hasNext()){
					QuerySolution qs = rs.nextSolution();
					Resource r = qs.getResource("issue");
					String issueUri = r.getURI();
					issueUris.add(issueUri);
				}	

//				out = new ByteArrayOutputStream();
//				bufferedOut = new BufferedOutputStream(out);
//				model.write(bufferedOut, "N3");
//				n3 = out.toString();	

//				System.out.println("Model " + modelName + " after inference.." + n3);

			} finally {
				qe.close();

				metaModel.close();
				unionModel.close();				
				inf.close();
			}

		} finally {

			model.leaveCriticalSection();				
			model.close();
			maker.close();

		}

		return issueUris;
	}


	public static ArrayList<Entry> getCachedIssueEntries(String username, String password, String baseUri, Project project) throws InternalServerErrorException, BadRequestException{

//		System.out.println("Fetching cached entries..	");

		ArrayList<Entry> entries = new ArrayList<Entry>();

		ArrayList<Issue> issueList = new ArrayList<Issue>();

		if(project != null && project.getName() != null){

			String modelName = MODELPREFIX + project.getName();

			ModelMaker maker = JenaHandler.getModelMaker();
			Model model = maker.getModel(modelName);

			ArrayList<String> issueUris = new ArrayList<String>();

			if(model == null){
				throw new BadRequestException("No issues cached for this project");
			}

			model.enterCriticalSection(Lock.READ);	

//			OutputStream out = new ByteArrayOutputStream();
//			BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
//			model.write(bufferedOut, "N3");
//			String n3 = out.toString();	


			/**
			 * We don't want to overwrite 
			 */
			Model metaModel = ModelFactory.createOntologyModel();

			Model unionModel = ModelFactory.createUnion(model, metaModel);

//			System.out.println("Model " + modelName + " before inference.." + n3);

			try{

				JenaHandler.loadN3(metaModel, JenaHandler.BAETLEN3);	

				model.enterCriticalSection(Model.READ);

				Reasoner reasoner = RDFSFBRuleReasonerFactory.theInstance().create(null);

//				Reasoner reasoner = OWLFBRuleReasonerFactory.theInstance().create(null);
				InfModel inf = ModelFactory.createInfModel(reasoner, unionModel);

				if(inf == null){
					throw new InternalServerErrorException("Could not create inferred model");
				}

				String fetchIssueProperties = 		
					"PREFIX rdf: <" + Issue.RDFNS + ">" +
					"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" +
					"PREFIX owl:  <http://www.w3.org/2002/07/owl#>" +
					"PREFIX baetle: <" + Issue.BAETLENS + ">" +
					"PREFIX mpl: <" + Issue.MPLNS + ">" +
					"PREFIX foaf: <" + Issue.FOAFNS +">" +
					"PREFIX doap: <" + Issue.DOAPNS +">" +
					"PREFIX sioc: <" + Issue.SIOCNS +">" +
					"PREFIX wf: <" + Issue.WFNS +">" +
					"SELECT ?issue ?title ?description ?prioritytype " +
					"WHERE {" +
//					" { " +
					" ?issue rdf:type baetle:Issue ." + 
					" OPTIONAL { " + 
					" 	?issue baetle:title ?title " +
					" } ." +
					" OPTIONAL { " +
					" 	?issue baetle:description ?description " +
					" } " +
					" OPTIONAL { " +
					" 	?issue baetle:priority ?priority . " +
					" 	?priority rdf:type ?prioritytype . " +
					" 	?prioritytype rdfs:subClassOf baetle:Priority " +
					" } " +
					" } " +
					"";	

				Query query = QueryFactory.create(fetchIssueProperties);

				// Execute the query and obtain results
//				QueryExecution qe = QueryExecutionFactory.create(query, inf);
				QueryExecution qe = QueryExecutionFactory.create(query, inf);
				ResultSet rs = qe.execSelect();

				try{

					while(rs.hasNext()){
						QuerySolution qs = rs.nextSolution();

//						System.out.println(qs);


						Resource issueR = qs.getResource("issue");
						String issueUri = issueR.getURI();
						issueUris.add(issueUri);


						Resource priorityR = qs.getResource("prioritytype");





						Literal titleR = qs.getLiteral("title");
						Literal descR = qs.getLiteral("description");

						Issue issue = new Issue();

						Entry entry = new Entry();

//						if(issueUri != null){

//						String describeIssue = 		
//						"PREFIX rdf: <" + Issue.RDFNS + ">" +
//						"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" +
//						"PREFIX owl:  <http://www.w3.org/2002/07/owl#>" +
//						"PREFIX baetle: <" + Issue.BAETLENS + ">" +
//						"PREFIX mpl: <" + Issue.MPLNS + ">" +
//						"PREFIX foaf: <" + Issue.FOAFNS +">" +
//						"PREFIX doap: <" + Issue.DOAPNS +">" +
//						"PREFIX sioc: <" + Issue.SIOCNS +">" +
//						"PREFIX wf: <" + Issue.WFNS +">" +
//						"DESCRIBE <" + issueUri +">" +
//						"";	

//						Query issueQuery = QueryFactory.create(describeIssue);


//						QueryExecution issueQueryqueryEx = QueryExecutionFactory.create(issueQuery, model);			
//						Model issueModel = issueQueryqueryEx.execDescribe();

//						try{
//						ByteArrayOutputStream out = new ByteArrayOutputStream();
//						BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
//						issueModel.write(bufferedOut, "N3");
//						String issuen3 = out.toString();





//						} finally {
//						issueQueryqueryEx.close();
//						issueModel.close();
//						} 

//						System.out.println("Fetching n3 for issue " + issueUri);

////						String n3 = getN3Representation(issueUri, modelName);

//						}

						Link alternate = new Link();
						alternate.setHref(issueUri);

						ArrayList<Link> alternateLinks = new ArrayList<Link>();

						alternateLinks.add(alternate);

						issue.setViewissueurl(issueUri);


						if(priorityR != null){

							String priority = priorityR.getURI();

							if(priority != null)
								issue.setPriority(priority);

						}

						if(titleR != null){
							entry.setTitle(titleR.getString());
							issue.setSummary(titleR.toString());
						} else {
							issue.setSummary("");
						}

						if(descR != null){
							Content summary = new Content();
							summary.setType("text");
							summary.setValue(descR.toString());
							issue.setDescription(descR.toString());
							entry.setSummary(summary);
						} else {
							issue.setDescription("");
						}

						Content content = new Content();
						content.setType("text/rdf+n3");

						String issuen3 = issue.toN3();

						content.setValue(issuen3);

						ArrayList<Content> contents = new ArrayList<Content>();
						contents.add(content);

						entry.setContents(contents);

						entry.setAlternateLinks(alternateLinks);

						entries.add(entry);
					}	

//					out = new ByteArrayOutputStream();
//					bufferedOut = new BufferedOutputStream(out);
//					model.write(bufferedOut, "N3");
//					n3 = out.toString();	

//					System.out.println("Model " + modelName + " after inference.." + n3);

				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} finally {
					qe.close();
					metaModel.close();
					unionModel.close();				
					inf.close();
				}

			} finally {

				model.leaveCriticalSection();				
				model.close();
				maker.close();

			}

//			/**
//			* Add the n3 representation to the content..
//			*/
//			for (Entry entry : entries) {

//			List<Link> alternateLinks = entry.getAlternateLinks();

//			Link alternate = alternateLinks.get(0);

//			if(alternate != null){

//			String issueUri = alternate.getHref();

//			String n3 = getN3Representation(issueUri, modelName);

//			Content content = new Content();
//			content.setType("text/rdf+n3");
//			content.setValue(n3);

//			ArrayList<Content> contents = new ArrayList<Content>();
//			contents.add(content);

//			entry.setContents(contents);

//			}

//			}

		} else {
			throw new BadRequestException("Project not valid");
		}





		return entries;
	}


	/**
	 * Get all cached issues for a project
	 * @param username
	 * @param password
	 * @param project
	 * @return
	 * @throws BadRequestException
	 * @throws InternalServerErrorException
	 */
	public static Issue[] getCachedIssues(String username, String password, Project project) throws BadRequestException, InternalServerErrorException{

		Issue[] issues = null;

		ArrayList<Issue> issueList = new ArrayList<Issue>();

		if(project != null && project.getName() != null){

			String modelName = MODELPREFIX + project.getName();

			ArrayList<String> issueUris = getCachedIssueUris(modelName);

			ModelMaker maker = JenaHandler.getModelMaker();
			Model model = maker.getModel(modelName);

			try {

//				OutputStream out = new ByteArrayOutputStream();
//				BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
//				model.write(bufferedOut, "N3");
//				String n3 = out.toString();	

//				System.out.println("Model " + modelName + "before describe: " + n3);


				for (String issueUri : issueUris) {

					String describeIssue = 		
						"PREFIX rdf: <" + Issue.RDFNS + ">" +
						"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" +
						"PREFIX owl:  <http://www.w3.org/2002/07/owl#>" +
						"PREFIX baetle: <" + Issue.BAETLENS + ">" +
						"PREFIX mpl: <" + Issue.MPLNS + ">" +
						"PREFIX foaf: <" + Issue.FOAFNS +">" +
						"PREFIX doap: <" + Issue.DOAPNS +">" +
						"PREFIX sioc: <" + Issue.SIOCNS +">" +
						"PREFIX wf: <" + Issue.WFNS +">" +
						"DESCRIBE <" + issueUri +">" +
//						"DESCRIBE ?issue " +
//						"WHERE {" +
//						"	?issue rdf:about <" + issueUri + "> . " +
//						"} " +
						"";	


//					System.out.println(describeIssue);

					Query issueQuery = QueryFactory.create(describeIssue);


					model.enterCriticalSection(Model.READ);
					try{

						QueryExecution issueQueryqueryEx = QueryExecutionFactory.create(issueQuery, model);			
						Model issueModel = issueQueryqueryEx.execDescribe();

						try{
							OutputStream out = new ByteArrayOutputStream();
							BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
							issueModel.write(bufferedOut, "N3");
							String issuen3 = out.toString();	

//							System.out.println("Model after describe: ");
//							System.out.println(issuen3);

							int ls = issueUri.lastIndexOf("/");
							String issueKey = issueUri.substring(ls + 1);

							Issue issue;
							try {
								issue = Issue.parseOntology(issuen3, project.getName(), issueKey, issueUri);
								issueList.add(issue);

							} catch (Exception e) {
								// TODO Auto-generated catch block
//								e.printStackTrace();
								System.out.println("Couldn't parse the issue or something..");
							}


						} finally {
							issueQueryqueryEx.close();
							issueModel.close();
						} 

					} finally {

						model.leaveCriticalSection();

					}

				}


			} finally {


				model.close();
				maker.close();
			}

		} else {

			throw new BadRequestException("Project null or without name");
		}

		issues = new Issue[issueList.size()];

		for(int i = 0; i < issueList.size(); i++){
			issues[i] = issueList.get(i);
		}

		return issues;

	}


	private static String getN3Representation(String issueUri, String modelName) throws InternalServerErrorException{

		String issuen3 = null;		

		ModelMaker maker = JenaHandler.getModelMaker();
		Model model = maker.getModel(modelName);

		try {

			String describeIssue = 		
				"PREFIX rdf: <" + Issue.RDFNS + ">" +
				"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" +
				"PREFIX owl:  <http://www.w3.org/2002/07/owl#>" +
				"PREFIX baetle: <" + Issue.BAETLENS + ">" +
				"PREFIX mpl: <" + Issue.MPLNS + ">" +
				"PREFIX foaf: <" + Issue.FOAFNS +">" +
				"PREFIX doap: <" + Issue.DOAPNS +">" +
				"PREFIX sioc: <" + Issue.SIOCNS +">" +
				"PREFIX wf: <" + Issue.WFNS +">" +
				"DESCRIBE <" + issueUri +">" +
//				"DESCRIBE ?issue " +
//				"WHERE {" +
//				"	?issue rdf:about <" + issueUri + "> . " +
//				"} " +
				"";	


//			System.out.println(describeIssue);



			Query issueQuery = QueryFactory.create(describeIssue);

			model.enterCriticalSection(Model.READ);
			try{

				QueryExecution issueQueryqueryEx = QueryExecutionFactory.create(issueQuery, model);			
				Model issueModel = issueQueryqueryEx.execDescribe();

				try{
					OutputStream out = new ByteArrayOutputStream();
					BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
					issueModel.write(bufferedOut, "N3");
					issuen3 = out.toString();	

				} finally {
					issueQueryqueryEx.close();
					issueModel.close();
				} 

			} finally {

				model.leaveCriticalSection();

			}

		} finally {

			maker.close();
			model.close();

		}

		return issuen3;

	}

	/**
	 * Updates the cache
	 * @param issues
	 * @param project
	 */
	private static void updateCache(Issue[] issues, Project project){


		/**
		 * Assumes that all other issues are dealt with..
		 * E.g. if no contact with the issue tracker is established - then this method should not be executed
		 * This could cause a lot of data to be lost
		 */
		if(issues != null && project != null && project.getName() != null){

			String modelName = MODELPREFIX + project.getName();

			try {

				ModelMaker maker = JenaHandler.getModelMaker();				

				//Start with a fresh model
				Model model = maker.createModel(modelName);

				//WRITE Lock on
				model.enterCriticalSection(Model.WRITE);

				try {

					model.removeAll();

					//Add all the data
					for (Issue issue : issues) {

						String n3 = issue.toN3();
						ByteArrayInputStream inputStream = new ByteArrayInputStream(n3.getBytes());
						model.read(inputStream, "", "N3");

					}

					OutputStream out = new ByteArrayOutputStream();
					BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
					model.write(bufferedOut, "N3");
					String n3 = out.toString();	

					System.out.println("Model " + modelName + " before commit: " + n3);

					model.commit();	

				} finally {

					model.leaveCriticalSection();
					model.close();
					maker.close();

				}


				/**
				 * No side effects should be visible to an end user? An admin should probably be notified?
				 */
			} catch (InternalServerErrorException e) {
				// TODO Auto-generated catch block

				e.printStackTrace();
			}


		}


	}


}
