package no.miles.mpl.semant.app.endpoint.datahandlers;


import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import no.miles.mpl.wrapper.BugTrackerWrapperInterface;
import no.miles.mpl.wrapper.JiraWrapper;
import no.miles.mpl.wrapper.datastructures.Issue;
import no.miles.mpl.wrapper.datastructures.IssueTrackers;
import no.miles.mpl.wrapper.datastructures.Project;
import no.miles.mpl.wrapper.TracWrapper;
import no.miles.mpl.wrapper.exception.BadRequestException;
import no.miles.mpl.wrapper.exception.InternalServerErrorException;
import no.miles.mpl.wrapper.exception.NotFoundException;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import com.hp.hpl.jena.db.DBConnection;
import com.hp.hpl.jena.db.IDBConnection;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.ModelMaker;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.shared.Lock;
import com.sun.swing.internal.plaf.basic.resources.basic_zh_CN;
import com.sun.syndication.feed.atom.Content;
import com.sun.syndication.feed.atom.Entry;

public class Projects {

	/**
	 * Return all project from config..
	 * @return
	 * @throws InternalServerErrorException 
	 */
	public static ArrayList<Project> getProjects(String username, String password) throws InternalServerErrorException
	{
		ArrayList<Project> projects = new ArrayList<Project>();

//		Connection conn = getConnection();
		ModelMaker maker = JenaHandler.getModelMaker();

//		System.out.println("Fetching projects..");

		Model model = maker.openModel("projects");

		//Convert to string
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		BufferedOutputStream bufferedOut = null;
//		new BufferedOutputStream(out);
//		model.write(bufferedOut, "N3");
		String n3 = null;
//		out.toString();	

//		System.out.println("Model: ");
//		System.out.println(n3);

		String projectNameQueryString = 		
			"PREFIX rdf: <" + Issue.RDFNS + ">" +
			"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" +
			"PREFIX owl:  <http://www.w3.org/2002/07/owl#>" +
			"PREFIX mpl: <" + Issue.MPLNS + ">" +
			"PREFIX foaf: <" + Issue.FOAFNS +">" +
			"PREFIX doap: <" + Issue.DOAPNS +">" +
			"PREFIX sioc: <" + Issue.SIOCNS +">" +
			"PREFIX wf: <" + Issue.WFNS +">" +
			"SELECT ?project ?projectName " +
			"WHERE {" +
			"      ?project rdf:type doap:Project . " +	
			"      ?project doap:name ?projectName . " +	
			"      }" +
			"";	

		Query query = QueryFactory.create(projectNameQueryString);

		//Append READ lock
		model.enterCriticalSection(Lock.READ) ;

		try {

			QueryExecution queryEx = QueryExecutionFactory.create(query, model);	


			ResultSet rs = queryEx.execSelect();

			try {

				while(rs.hasNext()){

					QuerySolution qs = rs.nextSolution();
//					System.out.println(qs.toString());

					String projectUri = qs.get("project").toString();
					String projectName = qs.get("projectName").toString();

					String projectQueryString = 		
						"PREFIX rdf: <" + Issue.RDFNS + ">" +
						"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" +
						"PREFIX owl:  <http://www.w3.org/2002/07/owl#>" +
						"PREFIX mpl: <" + Issue.MPLNS + ">" +
						"PREFIX foaf: <" + Issue.FOAFNS +">" +
						"PREFIX doap: <" + Issue.DOAPNS +">" +
						"PREFIX sioc: <" + Issue.SIOCNS +">" +
						"PREFIX wf: <" + Issue.WFNS +">" +
						"DESCRIBE ?project ?bugdb " +
						"WHERE {" +
						" {" +
						"    ?project doap:name \"" + projectName + "\" . " +	
						" 	 ?project doap:bug-database ?bugdb . " +
						" }" +
						" UNION " +
						" { " +
						"	?project rdf:about <" + projectUri + "> . " +
						"   ?project doap:bug-database ?bugdb . " +
						" }" +
						"} " +
						"";	

//					System.out.println(projectQueryString);

					Query projectQuery = QueryFactory.create(projectQueryString);
					QueryExecution projectQueryqueryEx = QueryExecutionFactory.create(projectQuery, model);			


					Model projectModel = projectQueryqueryEx.execDescribe();

//					StmtIterator stmtIter = projectModel.listStatements();
//					while(stmtIter.hasNext()){

//					model.remove(stmtIter.nextStatement());

//					}


					//Convert to string
					out = new ByteArrayOutputStream();
					bufferedOut = new BufferedOutputStream(out);
					projectModel.write(bufferedOut, "N3");
					n3 = out.toString();	

					Project project = Project.parse(n3);

					projectModel.close();

//					System.out.println("Trying to parse project from : " + n3);

//					System.out.println("Project : " + project);

//					System.out.println("The project's name is: " + project.getName());


					projects.add(project);

				}		

			} finally {

				queryEx.close();

			}


		} finally {

			//Remove READ lock
			model.leaveCriticalSection();			
			model.close();			
			maker.close();

		}

//		System.out.println("Model for project " + projectName);
//		System.out.println(n3);
		//Convert to string
//		out = new ByteArrayOutputStream();
//		bufferedOut = new BufferedOutputStream(out);
//		model.write(bufferedOut, "N3");
//		n3 = out.toString();	

//		System.out.println("Model: ");
//		System.out.println(n3);	

//		System.out.println("Returning projects..");

		return projects;
	}

	/**
	 * Gets a project
	 * @param projectKey
	 * @return
	 * @throws InternalServerErrorException
	 */
	public static Project getProject(String baseUri, String projectKey, String username, String password) throws InternalServerErrorException
	{
		Project project = null;

		ModelMaker maker = JenaHandler.getModelMaker();
		Model model = maker.getModel("projects");

		String projectUri = baseUri + "projects/" + projectKey;

		if(!projectExists(model, projectKey, projectUri)){
			return null;
		}

		String projectQueryString = 		
			"PREFIX rdf: <" + Issue.RDFNS + ">" +
			"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" +
			"PREFIX owl:  <http://www.w3.org/2002/07/owl#>" +
			"PREFIX mpl: <" + Issue.MPLNS + ">" +
			"PREFIX foaf: <" + Issue.FOAFNS +">" +
			"PREFIX doap: <" + Issue.DOAPNS +">" +
			"PREFIX sioc: <" + Issue.SIOCNS +">" +
			"PREFIX wf: <" + Issue.WFNS +">" +
			"DESCRIBE ?project ?bugdb " +
			"WHERE {" +
			" {" +
			" 	 ?project rdf:type doap:Project . " +
			"    ?project doap:name \"" + projectKey + "\" . " +
			"    ?project doap:bug-database ?bugdb . " +

			" }" +
			" UNION " +
			" { " +
			"	?project rdf:about <" + projectUri + "> . " +
			"   ?project doap:bug-database ?bugdb . " +
			" }" +
			"} " +
			"";	

		Query projectQuery = QueryFactory.create(projectQueryString);

		//Append READ lock
		model.enterCriticalSection(Lock.READ) ;

		try {

			QueryExecution projectQueryqueryEx = QueryExecutionFactory.create(projectQuery, model);			
			Model projectModel = projectQueryqueryEx.execDescribe();
			try {
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
				projectModel.write(bufferedOut, "N3");
				String n3 = out.toString();	

//				System.out.println("Model after get: ");
//				System.out.println(n3);

				project = Project.parse(n3);

			} finally {
				projectQueryqueryEx.close();
				projectModel.close();
			} 

		} finally {

			model.leaveCriticalSection();
			model.close();
			maker.close();

		}

//		System.out.println("Project model from getProject():");
//		System.out.println(project.toN3());

//		PreparedStatement st;
//		try {
//		st = conn.prepareStatement("SELECT * FROM projects WHERE name = ?");
//		st.setString(1, projectKey);
//		ResultSet rs = st.executeQuery();


//		while (rs.next()) {
//		String name = rs.getString("name");
//		String bugtracker = rs.getString("bugtracker");
//		String url = rs.getString("url");	

//		project = new Project(name, name);
//		project.setBugtrackerName(bugtracker);
//		project.setBugtrackerUrl(url);				
//		}
//		rs.close();
//		st.close();

		return project;

//		} catch (SQLException e) {
//		// TODO Auto-generated catch block
//		e.printStackTrace();
//		throw new InternalServerErrorException("Database connection error");
//		}



	}

	/**
	 * Insantiates the correct bugtrackerwrapper! (Trac or Jira, depending on config)
	 * @param projectKey
	 * @return
	 * @throws InternalServerErrorException 
	 */
	public static BugTrackerWrapperInterface getBugTracker(String baseUri, String projectKey, String username, String password) throws InternalServerErrorException
	{
		BugTrackerWrapperInterface bugTracker = null;
//		Connection conn = getConnection();

		Project project = getProject(baseUri, projectKey, username, password);

		String btType = project.getBugtrackertype();
		String btUrl = project.getBugtrackerurl();

		if(btType != null && btUrl != null){

			if(btType.equalsIgnoreCase(IssueTrackers.JIRA))
				bugTracker = new JiraWrapper(btUrl);

			if(btType.equalsIgnoreCase(IssueTrackers.TRAC))
				bugTracker = new TracWrapper(btUrl);
		}

//		PreparedStatement st;
//		try {
//		st = conn.prepareStatement("SELECT * FROM projects WHERE name = ?");
//		st.setString(1, projectKey);
//		ResultSet rs = st.executeQuery();
//		while (rs.next()) {
//		String name = rs.getString("name");
//		String bugtracker = rs.getString("bugtracker");
//		String url = rs.getString("url");

//		if(bugtracker.equals("jira")){
//		bugTracker = new JiraWrapper(url);
//		}
//		else if(bugtracker.equals("trac")){
//		bugTracker = new TracWrapper(url);
//		}				
////		System.out.println(name + " " + bugtracker + " " + url);
//		}
//		rs.close();
//		st.close();
//		} catch (SQLException e) {
//		// TODO Auto-generated catch block
//		e.printStackTrace();
//		return null;
//		}	

		return bugTracker;
	}



	/**
	 * Add a new project
	 * @param username
	 * @param password
	 * @param entry
	 * @return
	 * @throws BadRequestException
	 * @throws InternalServerErrorException
	 */
	public static Project addProject(String baseUri, String username, String password, Entry entry) throws BadRequestException, InternalServerErrorException
	{

//		System.out.println(entry.getXmlBase());
//		System.out.println(entry.get);

		if(entry.getContents() == null || entry.getContents().size() == 0)
			throw new BadRequestException("Content is empty!");

		Content content = (Content) entry.getContents().get(0);

//		System.out.println("type: " + content.getType());

//		/**
//		* If XML content - parse the issue..
//		*/
//		if(content != null && content.getValue() != null && (content.getType().equalsIgnoreCase("application/xml"))){

//		String projectKey = entry.getTitle();			
//		if(projectKey == null)
//		throw new BadRequestException("Bad title!");		

//		if(projectExists(projectKey))
//		throw new BadRequestException("Project already exists with key " + projectKey);

//		System.out.println("Trying to parse xml content");

//		String xmlString = content.getValue();

//		System.out.println(content.getValue().length());

//		System.out.println("xmlString " + xmlString + " end..");

//		InputStream in = new ByteArrayInputStream(xmlString.getBytes());

//		BufferedInputStream bufferedIn = new BufferedInputStream(in);

//		SAXBuilder builder = new SAXBuilder();		



//		try {

//		Document doc = builder.build(bufferedIn);			

//		Element root = doc.getRootElement();

//		Element bugtracker_name = root.getChild("bugtracker_name");
//		Element bugtracker_url = root.getChild("bugtracker_url");

//		String bt_name = bugtracker_name.getText();
//		String bt_url = bugtracker_url.getText();

//		if(bt_name == null || bt_url == null)
//		throw new BadRequestException("Missing bugtracker_name or bugtracker_url");

//		Project addedProject = new Project(projectKey);

//		addedProject.setBugtrackertype(bt_name);
//		addedProject.setBugtrackerurl(bt_url);

////		Connection conn = getConnection();
//		IDBConnection conn = getConnection();
////		PreparedStatement st;

////		try {
////		st = conn.prepareStatement("INSERT INTO projects (name, bugtracker, url) VALUES " +
////		"('" + addedProject.getKey() + "', '" + addedProject.getBugtrackerName() + "','" + addedProject.getBugtrackerUrl() + "')");

////		int result = st.executeUpdate();

////		System.out.println("DB insert result " + result);

////		} catch (SQLException e) {
////		// TODO Auto-generated catch block
////		e.printStackTrace();
////		throw new InternalServerErrorException("Error in writing to database");
////		}

//		return addedProject;			

//		} catch (JDOMException e) {
//		// TODO Auto-generated catch block
//		e.printStackTrace();
//		throw new BadRequestException("Couldn't parse content");
//		} catch (IOException e) {
//		// TODO Auto-generated catch block
//		e.printStackTrace();
//		throw new BadRequestException("Couldn't parse content");
//		}
//		} else 
		if(content != null && content.getValue() != null && (content.getType().equalsIgnoreCase("text/rdf+n3"))){

//			System.out.println("N3 in content!");

			String contentN3 = content.getValue();			

			Project project = Project.parse(contentN3);

			String projectName = project.getName();
			if(projectName == null)
				project.setName(entry.getTitle());

			project.setUrl(baseUri + project.getName());

			ModelMaker maker = JenaHandler.getModelMaker();

			Model model = maker.openModel("projects");

			if(model != null){				
				if(projectExists(model, project.getName(), project.getUrl())){
					throw new BadRequestException("Project already exists!");
				}
			}
			
			if(model == null){
//				System.out.println("Instantiating model for the first time..");
				model = maker.createModel("projects");
			}	

			//Set WRITE lock
			model.enterCriticalSection(Lock.WRITE);


			try{
				
		

				
				try{

					String n3 = project.toN3();
					ByteArrayInputStream inputStream = new ByteArrayInputStream(n3.getBytes());
					model.read(inputStream, "", "N3");			

					//Add the project
					model.commit();

				} finally {

				}

			} finally {

				//Remove WRITE lock
				model.leaveCriticalSection();

				model.close();
				maker.close();
			}

			return project;
		}

		throw new BadRequestException("Content is empty or not XML!");
	}

//	/**
//	* Methods that searches for an existing project in the model..
//	* @param model
//	* @param baseUri
//	* @param name
//	* @return
//	*/
//	private static Resource getProjectFromModel(Model model, String baseUri, String name){

//	Resource p = null;

//	ResIterator iterator = model.listSubjectsWithProperty(model.createProperty(Issue.RDFNS + "about"), baseUri + name);
//	while(iterator.hasNext()){
//	p = iterator.nextResource();

//	return p;

//	}


//	//Fetching the first match here..
//	iterator = model.listSubjectsWithProperty(model.createProperty(Issue.DOAPNS + "name"), name);
//	while(iterator.hasNext()){
//	p = iterator.nextResource();

//	System.out.println("Found match? " + p.toString());

//	StmtIterator stmtit = p.listProperties(model.createProperty(Issue.RDFNS + "type"));
//	while(stmtit.hasNext()){
//	Statement stmt = stmtit.nextStatement();

//	if(stmt.getObject().toString().equalsIgnoreCase(Issue.DOAPNS + "Project"))
//	return p;

//	}

//	}

//	return null;

//	}

	/**
	 * Add a new project
	 * @param username
	 * @param password
	 * @param entry
	 * @return
	 * @throws BadRequestException
	 * @throws InternalServerErrorException
	 */
	public static Project updateProject(String baseUri, String username, String password, String projectKey, Entry entry) throws BadRequestException, InternalServerErrorException
	{



//		System.out.println(entry.getXmlBase());
//		System.out.println(entry.get);

		if(entry.getContents() == null || entry.getContents().size() == 0)
			throw new BadRequestException("Content is empty!");

		Content content = (Content) entry.getContents().get(0);

//		System.out.println("type: " + content.getType());

		String projectUri = baseUri + projectKey;

		if(content != null && content.getValue() != null && (content.getType().equalsIgnoreCase("text/rdf+n3"))){

			ModelMaker maker = JenaHandler.getModelMaker();
			Model model = maker.getModel("projects");

			Project project = null;

			try {

				if(!projectExists(model, projectKey, projectUri)){
					throw new NotFoundException("Project doesn't exists with key " + projectKey);
				}

				String contentN3 = content.getValue();
				project = Project.parse(contentN3);			
				project.setUrl(projectUri);

			} finally {

				model.close();
				maker.close();

			}

			deleteProject(baseUri, username, password, projectKey);

			maker = JenaHandler.getModelMaker();
			model = maker.getModel("projects");

			model.enterCriticalSection(Lock.WRITE);
			try {

				try {

					String n3 = project.toN3();
					ByteArrayInputStream inputStream = new ByteArrayInputStream(n3.getBytes());
					model.read(inputStream, "", "N3");

//					System.out.println("Updated project with: ");
//					System.out.println(n3);

					model.commit();

				} finally {



				}

			} finally {

				model.leaveCriticalSection();

				model.close();
				maker.close();

			}

			return project;

		}

//		/**
//		* If XML content - parse the issue..
//		*/
//		if(content != null && content.getValue() != null && content.getType().equalsIgnoreCase("application/xml")){



////		String projectKey = entry.getTitle();			
//		if(projectKey == null || !(entry.getTitle().matches(projectKey))){

//		System.out.println("Bad title: " + entry.getTitle() + " should match: " + projectKey);

//		throw new BadRequestException("Bad title!");		
//		}

////		if(!projectExists(projectKey))
////		throw new NotFoundException("Project doesn't exists with key " + projectKey);

//		System.out.println("Trying to parse xml content");

//		String xmlString = content.getValue();

//		System.out.println(content.getValue().length());

//		System.out.println("xmlString " + xmlString + " end..");

//		InputStream in = new ByteArrayInputStream(xmlString.getBytes());

//		BufferedInputStream bufferedIn = new BufferedInputStream(in);

//		SAXBuilder builder = new SAXBuilder();		

//		try {

//		Document doc = builder.build(bufferedIn);				

//		Element root = doc.getRootElement();

//		Element bugtracker_name = root.getChild("bugtracker_name");
//		Element bugtracker_url = root.getChild("bugtracker_url");

//		String bt_name = bugtracker_name.getText();
//		String bt_url = bugtracker_url.getText();

//		if(bt_name == null || bt_url == null)
//		throw new BadRequestException("Missing bugtracker_name or bugtracker_url");

//		Project updatedProject = getProject(projectKey);

//		updatedProject.setBugtrackertype(bt_name);
//		updatedProject.setBugtrackerurl(bt_url);

////		Connection conn = getConnection();
//		IDBConnection conn = getConnection();
////		PreparedStatement st;

////		try {
////		st = conn.prepareStatement("UPDATE projects SET bugtracker = ?, url = ? WHERE name = ?");
////		st.setString(1, bt_name);
////		st.setString(2, bt_url);
////		st.setString(3, projectKey);

////		int result = st.executeUpdate();

////		System.out.println("DB insert result " + result);

////		} catch (SQLException e) {
////		// TODO Auto-generated catch block
////		e.printStackTrace();
////		throw new InternalServerErrorException("Error in writing to database");
////		}

//		return updatedProject;			

//		} catch (JDOMException e) {
//		// TODO Auto-generated catch block
//		e.printStackTrace();
//		throw new BadRequestException("Couldn't parse content");
//		} catch (IOException e) {
//		// TODO Auto-generated catch block
//		e.printStackTrace();
//		throw new BadRequestException("Couldn't parse content");
//		}
//		}	

		throw new BadRequestException("Content is empty or not XML!");
	}


	/**
	 * Returns true if a projec exists!
	 * @param model
	 * @param projectName
	 * @param projectUri
	 * @return
	 * @throws InternalServerErrorException
	 */
	public static boolean projectExists(Model model, String projectName, String projectUri) throws InternalServerErrorException{

//		Connection conn = getConnection();
//		IDBConnection conn = getConnection();


		ModelMaker maker = null;

		boolean exists = false;

		boolean cleanup = false;

		if(model == null){

			cleanup = true;

			maker = JenaHandler.getModelMaker();
//			System.out.println("Fetching projects..");
			model = maker.openModel("projects");

		}

//		System.out.println("Model is: ");
//		ByteArrayOutputStream out = new ByteArrayOutputStream();
//		BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
//		model.write(bufferedOut, "N3");
//		String n3 = out.toString();	
//		System.out.println(n3);

		String projectNameQueryString = 		
			"PREFIX rdf: <" + Issue.RDFNS + ">" +
			"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" +
			"PREFIX owl:  <http://www.w3.org/2002/07/owl#>" +
			"PREFIX mpl: <" + Issue.MPLNS + ">" +
			"PREFIX foaf: <" + Issue.FOAFNS +">" +
			"PREFIX doap: <" + Issue.DOAPNS +">" +
			"PREFIX sioc: <" + Issue.SIOCNS +">" +
			"PREFIX wf: <" + Issue.WFNS +">" +
			"ASK {" +
			" { " +
			"      ?project rdf:type doap:Project . " +	
			"      ?project doap:name \"" + projectName + "\" . " +	
			"} UNION { " +
			"      ?project rdf:about <" + projectUri + "> . " +	
			"      }" +
			"}" +
			"";	

//		System.out.println(projectNameQueryString);

		Query query = QueryFactory.create(projectNameQueryString);

		model.enterCriticalSection(Lock.READ);

		try {

			QueryExecution queryEx = QueryExecutionFactory.create(query, model);	

			try {

				exists = queryEx.execAsk();		

			} finally {

				queryEx.close();



//				model.close();

			}		

		} finally {

			model.leaveCriticalSection();

			if(cleanup){

				model.close();

				if(maker != null)
					maker.close();

			}

		}

//		System.out.println("Project existst? " + exists);

		return exists;

//		PreparedStatement st;
//		try {
//		st = conn.prepareStatement("SELECT name FROM projects WHERE name = ?");
//		st.setString(1, projectName);
//		ResultSet rs = st.executeQuery();
//		while (rs.next()) {
//		if(rs.getString("name").equals(projectName))
//		return true;
//		}
//		rs.close();
//		st.close();

//		} catch (SQLException e) {
//		// TODO Auto-generated catch block
//		e.printStackTrace();
//		}

	}



	/**
	 * Deletes a Project..
	 * @param username
	 * @param password
	 * @param entry
	 * @throws BadRequestException 
	 * @throws InternalServerErrorException 
	 */
	public static void deleteProject(String baseUri, String username, String password, String projectKey) throws BadRequestException, InternalServerErrorException{

		if(projectKey == null)
			throw new BadRequestException("Bad title/key!");

		ModelMaker maker = JenaHandler.getModelMaker();

		Model model = maker.getModel("projects");

		String projectUri = baseUri + projectKey;

		if(!projectExists(model, projectKey, projectUri))
			return;

//		//Convert to string
//		OutputStream out = new ByteArrayOutputStream();
//		BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
//		model.write(bufferedOut, "N3");
//		String n3 = out.toString();	

//		System.out.println("Model before delete");
//		System.out.println(n3);

		String projectQueryString = 		
			"PREFIX rdf: <" + Issue.RDFNS + ">" +
			"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" +
			"PREFIX owl:  <http://www.w3.org/2002/07/owl#>" +
			"PREFIX mpl: <" + Issue.MPLNS + ">" +
			"PREFIX foaf: <" + Issue.FOAFNS +">" +
			"PREFIX doap: <" + Issue.DOAPNS +">" +
			"PREFIX sioc: <" + Issue.SIOCNS +">" +
			"PREFIX wf: <" + Issue.WFNS +">" +
			"DESCRIBE ?project " +
			"WHERE {" +
			" {" +
			"    ?project doap:name \"" + projectKey + "\" . " +	
			" }" +
			" UNION " +
			" { " +
			"	?project rdf:about <" + projectUri + "> . " +
			" }" +
			"} " +
			"";	

//		System.out.println(projectQueryString);

		Query projectQuery = QueryFactory.create(projectQueryString);

		model.enterCriticalSection(Lock.WRITE);

		try{

			QueryExecution projectQueryqueryEx = QueryExecutionFactory.create(projectQuery, model);	

			try {

				Model projectModel = projectQueryqueryEx.execDescribe();

				StmtIterator stmtIter = projectModel.listStatements();
				while(stmtIter.hasNext()){

					model.remove(stmtIter.nextStatement());

				}

//				out = new ByteArrayOutputStream();
//				bufferedOut = new BufferedOutputStream(out);
//				model.write(bufferedOut, "N3");
//				n3 = out.toString();	

//				System.out.println("Model after delete");
//				System.out.println(n3);

				model.commit();

			} finally {

				projectQueryqueryEx.close();


			}	

		} finally {

			model.leaveCriticalSection();
			
			model.close();
			maker.close();

		}


//		maker = getModelMaker();

//		model = maker.getModel("projects");

//		ByteArrayOutputStream out = new ByteArrayOutputStream();
//		BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
//		model.write(bufferedOut, "N3");
//		String n3 = out.toString();	

//		System.out.println("Model after delete + commit: ");
//		System.out.println(n3);

//		if(!projectExists(projectKey))
//		return; //Already deleted..

//		Connection conn = getConnection();
//		PreparedStatement st;

//		try {
//		st = conn.prepareStatement("DELETE FROM projects WHERE name = ?");
//		st.setString(1, projectKey);

//		int result = st.executeUpdate();

//		System.out.println("DB insert result " + result);

//		} catch (SQLException e) {
//		// TODO Auto-generated catch block
//		e.printStackTrace();
//		throw new InternalServerErrorException("Error in deleting from Database");
//		}

	}





}
