package no.miles.mpl.semantic.app.endpoint;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.util.Enumeration;
import java.util.List;

import javax.servlet.Servlet;
import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;

import no.miles.mpl.semant.app.endpoint.datahandlers.Comments;
import no.miles.mpl.semant.app.endpoint.datahandlers.ConcurrencyManager;
import no.miles.mpl.semant.app.endpoint.datahandlers.Issues;
import no.miles.mpl.semant.app.endpoint.datahandlers.Projects;
import no.miles.mpl.semantic.app.endpoint.entries.IssueEntry;
import no.miles.mpl.semantic.app.endpoint.entries.ProjectEntry;
import no.miles.mpl.semantic.app.endpoint.exception.AtomConflictException;
import no.miles.mpl.semantic.app.endpoint.feeds.CachedIssueFeed;
import no.miles.mpl.semantic.app.endpoint.feeds.CommentFeed;
import no.miles.mpl.semantic.app.endpoint.feeds.ComponentFeed;
import no.miles.mpl.semantic.app.endpoint.feeds.EhCachedIssueFeed;
import no.miles.mpl.semantic.app.endpoint.feeds.EntryCachedFeed;
import no.miles.mpl.semantic.app.endpoint.feeds.IssueFeed;
import no.miles.mpl.semantic.app.endpoint.feeds.IssueSparqlResultFeed;
import no.miles.mpl.semantic.app.endpoint.feeds.PriorityFeed;
import no.miles.mpl.semantic.app.endpoint.feeds.ProjectFeed;
import no.miles.mpl.semantic.app.endpoint.feeds.StateFeed;
import no.miles.mpl.semantic.app.endpoint.feeds.SuperCachedIssueFeed;
import no.miles.mpl.semantic.app.endpoint.feeds.TypeFeed;
import no.miles.mpl.semantic.app.endpoint.feeds.VersionFeed;
import no.miles.mpl.wrapper.datastructures.Issue;
import no.miles.mpl.wrapper.datastructures.Project;
import no.miles.mpl.wrapper.exception.*;

import org.apache.commons.httpclient.HttpParser;
import org.apache.commons.lang.StringUtils;

import com.sun.syndication.feed.atom.Content;
import com.sun.syndication.feed.atom.Entry;
import com.sun.syndication.feed.atom.Feed;
import com.sun.syndication.io.impl.Base64;
import com.sun.syndication.propono.atom.common.AtomService;
import com.sun.syndication.propono.atom.common.Categories;
import com.sun.syndication.propono.atom.common.Collection;
import com.sun.syndication.propono.atom.common.Workspace;
import com.sun.syndication.propono.atom.server.AtomException;
import com.sun.syndication.propono.atom.server.AtomHandler;
import com.sun.syndication.propono.atom.server.AtomMediaResource;
import com.sun.syndication.propono.atom.server.AtomNotAuthorizedException;
import com.sun.syndication.propono.atom.server.AtomNotFoundException;
import com.sun.syndication.propono.atom.server.AtomRequest;
import com.sun.syndication.propono.atom.server.AtomServlet;

/**
 * This class handles all incoming requests and routes them to the correct Feeds/Entries and other operations..
 * @author erlingwl
 *
 */
public class MplAtomHandler implements AtomHandler{

	private HttpServletRequest httpReq;

	private static String PROJECTS = "projects";
	private static String ISSUES = "issues";
//	private static String BASE = "http://localhost:8070/no.miles.mpl.semantic.app.endpoint/";

	private String username = null;
	private String password = null;

	public MplAtomHandler(HttpServletRequest req){
		this.httpReq = req;	

		System.out.println("context? : " + req.getContextPath());
		System.out.println(req.getRequestURL());
		System.out.println(req.getLocalName());
		System.out.println(req.getServletPath());

//		System.out.println("context dir: " + AtomServlet.getContextDirPath());

//		this(req, AtomServlet.getContextDirPath());

//		System.out.println(req.getInputStream());

//		ServletInputStream is;
//		try {
//		is = req.getInputStream();
//		BufferedReader in = new BufferedReader(new InputStreamReader(is));

//		String line;
//		while ((line = in.readLine()) != null) {
//		System.out.println("Line: " + line);
//		}

//		} catch (IOException e) {
//		// TODO Auto-generated catch block
//		e.printStackTrace();
//		}

		/**
		 * Deals with basic authentication
		 */
		String auth = req.getHeader("Authorization");
		if(auth != null && auth.startsWith("Basic "))
		{
			auth = auth.substring(6);			
			String credentials = Base64.decode(auth);			
			int splitIndex = credentials.indexOf(":");
			username = credentials.substring(0, splitIndex);
			password = credentials.substring(splitIndex + 1);			
		}
		else if(auth == null){
			System.out.println("Authorization = null..");		
		}
	}	

	@Override
	public void deleteEntry(AtomRequest req) throws AtomException {
		// TODO Auto-generated method stub

		String editUri = req.getRequestURL().toString();

		String[] pathInfo = StringUtils.split(req.getPathInfo(),"/");	
		System.out.println("***** DeleteEntry *****");
		System.out.println("Uri: " + req.getRequestURL());
		System.out.println("getCollection " + req.toString());
		System.out.println("pathInfo: " +  req.getPathInfo());

		int pathEnd = req.getRequestURL().indexOf(req.getPathInfo());
		String baseUri = req.getRequestURL().substring(0, pathEnd + 1);


//		/projects/{projectkey}/ - it's a project
//		/projects/{projectkey}/issues/{issuekey} - it's an issue

		//We're dealing with a project
		if(pathInfo.length == 2 && pathInfo[0].equalsIgnoreCase("projects")){				

			System.out.println("return projectEntry");
			String projectKey = pathInfo[1];		

			try {
				Projects.deleteProject(baseUri + "projects/" ,username, password, projectKey);				
				return;

			} catch (BadRequestException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new AtomNotFoundException(e.getMessage());
			} catch (InternalServerErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new AtomException(e.getMessage());
			}

		}

		//Were dealing with an issue
		else if(pathInfo.length == 4 && pathInfo[0].equalsIgnoreCase("projects") && pathInfo[2].equalsIgnoreCase("issues")){

			System.out.println("return issueEntry");

			String projectKey = pathInfo[1];
			String issueKeyAndVersion = pathInfo[3];		



			String issueKey = issueKeyAndVersion.substring(0, issueKeyAndVersion.indexOf(ConcurrencyManager.VERSION_PREFIX) - 1);

			String entryId = editUri.substring(0, editUri.indexOf(ConcurrencyManager.VERSION_PREFIX));

			System.out.println("Issuekey " + issueKey);
			System.out.println("Edituri " + editUri);
			System.out.println("EntryId " + editUri);

			/**
			 * Using the same optimistic concurrency approach as GDATA
			 */
			if(!ConcurrencyManager.getEditUri(entryId).equals(editUri)){
				throw new AtomConflictException("Conflict! - Someone has updated this issue");	
			} 
			
			// Don't need this, it's deleted anyway:
//			else {
//
//				ConcurrencyManager.incrementEditUri(entryId);
//			}

			try {
				Issues.deleteIssue(username, password, projectKey, issueKey);
				return;
			} catch (BadRequestException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new AtomNotFoundException(e.getMessage());
			} catch (InternalServerErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new AtomException(e.getMessage());
			}			
		}

		throw new AtomNotFoundException();


	}


	@Override
	public AtomService getAtomService(AtomRequest req) throws AtomException {

		System.out.println("Uri: " + req.getRequestURL());
		System.out.println("getService..");

		Workspace workspace = new Workspace("WS","collections");		


		Collection projectsCol = new Collection(PROJECTS,"text/plain", req.getRequestURL().toString() + PROJECTS + "/");
		projectsCol.setAccept("entry");	

		List<Project> projects;
		try {
			projects = Projects.getProjects(username, password);

			for (Project project : projects) {
				Collection issueCol = new Collection(project.getName() + ISSUES,"text/plain", req.getRequestURL().toString() + PROJECTS + "/" + project.getName() +"/" + ISSUES + "/");
				issueCol.setAccept("entry");	
				workspace.addCollection(issueCol);
			}

			workspace.addCollection(projectsCol);
			AtomService service = new AtomService();
			service.addWorkspace(workspace);	

			return service;

		} catch (InternalServerErrorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new AtomException(e.getMessage());
		}

	}

	@Override
	public String getAuthenticatedUsername() {
//		System.out.println("getAuthUserName");
		return username;
	}

	@Override
	public Categories getCategories(AtomRequest req) throws AtomException {
		// TODO Auto-generated method stub

		String[] pathInfo = StringUtils.split(req.getPathInfo(),"/");	
		System.out.println("***** GetCategories *****");
		System.out.println("Uri: " + req.getRequestURL());
		System.out.println("getCollection " + req.toString());
		System.out.println("pathInfo: " +  pathInfo.toString());

		return null;
	}

	@Override
	public Feed getCollection(AtomRequest req) throws AtomException {

		String[] pathInfo = StringUtils.split(req.getPathInfo(),"/");	
		System.out.println("***** GetCollection *****");
		System.out.println("Uri: " + req.getRequestURL());
		System.out.println("getCollection " + req.toString());
		System.out.println("pathInfo: " +  req.getPathInfo());

		int pathEnd = req.getRequestURL().indexOf(req.getPathInfo());
		String baseUri = req.getRequestURL().substring(0, pathEnd + 1);
//		System.out.println("Base: "+ baseUri);

		Enumeration<String> paramNames = req.getParameterNames();
		while(paramNames.hasMoreElements()){

			String paramName = paramNames.nextElement();			
			String param = req.getParameter(paramName);

			System.out.println("Paramname " + paramName + " param: " + param);

		}

		//We're dealing with a project
		if(pathInfo[0].equals("projects")){	

			if(pathInfo.length == 1)
			{
				Feed feed;
				try {
					feed = new ProjectFeed(baseUri, username, password);
					return feed;
				} catch (InternalServerErrorException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					throw new AtomException(e.getMessage());
				} catch (Exception e) {
					// TODO: handle exception
					e.printStackTrace();
					throw new AtomException(e.getMessage());
				}

			}
			else if (pathInfo.length == 3)
			{
				String choice = pathInfo[2];

				String projectName = pathInfo[1];			
				System.out.println("Project: " + projectName);

				try {					

					//The project exists!
					if(Projects.projectExists(null, projectName, baseUri + "projects/" + projectName)){	

						if(choice.equalsIgnoreCase("issues")){

							Feed feed;

							String sparqlQuery = req.getParameter("sparql");

							if(sparqlQuery == null || sparqlQuery.length() == 0){			

//								feed = new IssueFeed(username, password, projectName, baseUri);	
								feed = new EntryCachedFeed(username, password, projectName, baseUri + "projects/");

							} else {								
								//Return results from sparqlquery
//								System.out.println("sparqlQuery: " + sparqlQuery);
								feed = new IssueSparqlResultFeed(username, password, projectName, baseUri, sparqlQuery);							
															
							}

							return feed;
						}

						else if(choice.equalsIgnoreCase("versions")){		

							return new VersionFeed(username, password, projectName, baseUri);

						} else if(choice.equalsIgnoreCase("entrycachedissues")){	

							return new EntryCachedFeed(username, password, projectName, baseUri + "projects/");

						} else if(choice.equalsIgnoreCase("cachedissues")){	

							return new CachedIssueFeed(username, password, projectName, baseUri);

						} else if(choice.equalsIgnoreCase("supercachedissues")){	

							return new SuperCachedIssueFeed(username, password, projectName, baseUri);

						} else if(choice.equalsIgnoreCase("ehcachedissues")){	

							return new EhCachedIssueFeed(username, password, projectName, baseUri);

						} else if(choice.equalsIgnoreCase("components")){							

							return new ComponentFeed(username, password, projectName, baseUri);

						} else if(choice.equalsIgnoreCase("priorities")){		

							return new PriorityFeed(username, password, projectName, baseUri);

						} else if(choice.equalsIgnoreCase("types")){	

							return new TypeFeed(username, password, projectName, baseUri);

						}

					} else {

						throw new AtomNotFoundException("Project doesn't exist");

					}


				} catch (InternalServerErrorException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					throw new AtomException(e.getMessage());

				} catch (BadRequestException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();

					if (e instanceof UnAuthorizedException) {
						throw new AtomNotAuthorizedException(e.getMessage());
					}
					else if(e instanceof ForbiddenException) {
						throw new AtomNotFoundException("Forbidden: " + e.getMessage());
					}
					else {
						throw new AtomException(e.getMessage());					
					}
				} catch (Exception e) {
					// TODO: handle exception
					e.printStackTrace();
					throw new AtomException(e.getMessage());
				}


				// /projects/{projectname/issues/{issue}/{*}
			} else if(pathInfo.length == 5 && pathInfo[2].equalsIgnoreCase("issues")){

				String property = pathInfo[4];

				String projectKey = pathInfo[1];

				String issueKey = pathInfo[3];

				//{*} = states
				if(property.equalsIgnoreCase("states")){

					try {
						return new StateFeed(username, password, projectKey, issueKey, baseUri);
					} catch (BadRequestException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						if(e instanceof ForbiddenException){
							throw new AtomNotAuthorizedException(e.getMessage());
						} else if(e instanceof UnAuthorizedException){
							throw new AtomNotAuthorizedException(e.getMessage());
						}

						throw new AtomNotFoundException(e.getMessage());
					} catch (InternalServerErrorException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						throw new AtomException(e.getMessage());
					}

				}


				//{*} = attachments
				if(property.equalsIgnoreCase("attachments")){
					return null;
				}


				//{*} = comments
				if(property.equalsIgnoreCase("comments")){

					try {
						return new CommentFeed(username, password, projectKey, issueKey, baseUri);
					} catch (BadRequestException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						if(e instanceof ForbiddenException){
							throw new AtomNotAuthorizedException(e.getMessage());
						} else if(e instanceof UnAuthorizedException){
							throw new AtomNotAuthorizedException(e.getMessage());
						}

						throw new AtomNotFoundException(e.getMessage());
					} catch (InternalServerErrorException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						throw new AtomException(e.getMessage());
					}

				}

			}
		}

		throw new AtomNotFoundException();

	}

	@Override
	public Entry getEntry(AtomRequest req) throws AtomException {
		// TODO Auto-generated method stub
		String[] pathInfo = StringUtils.split(req.getPathInfo(),"/");	
		System.out.println("***** GetEntry *****");
		System.out.println("URL: " + req.getRequestURL());
		System.out.println("pathInfo: " +  pathInfo.toString());
		System.out.println("pathInfoLength: " + pathInfo.length);

		int pathEnd = req.getRequestURL().indexOf(req.getPathInfo());
		String baseUri = req.getRequestURL().substring(0, pathEnd + 1);


		System.out.println("Querystring " + req.getQueryString());
		System.out.println("params.. "+ req.getParameterNames().toString());

		Enumeration<String> paramNames = req.getParameterNames();
		while(paramNames.hasMoreElements()){

			String paramName = paramNames.nextElement();			
			String param = req.getParameter(paramName);

			System.out.println("Paramname " + paramName + " param: " + param);

		}

		try {

//			/projects/{projectkey}/ - it's a project
//			/projects/{projectkey}/issues/{issuekey} - it's an issue

			//We're dealing with a project
			if(pathInfo.length == 2 && pathInfo[0].equalsIgnoreCase("projects")){				

				System.out.println("return projectEntry");

				String projectKey = pathInfo[1];			
				Project project;

				project = Projects.getProject(baseUri, projectKey, username, password);
				ProjectEntry projectEntry = new ProjectEntry(baseUri + "projects/", project);
				return projectEntry;

			}

			//Were dealing with an issue, version, priority, type or component
			else if(pathInfo.length == 4 && pathInfo[0].equalsIgnoreCase("projects")){

				String choice =  pathInfo[2];

				String projectKey = pathInfo[1];

				String choiceKey = pathInfo[3];	

				if(choice.equalsIgnoreCase("issues")){

					String issueKey = choiceKey;					
					Issue issue = Issues.getIssue(username, password, projectKey, issueKey);
					IssueEntry issueEntry = new IssueEntry(baseUri + "projects/" + projectKey + "/issues/", issue);

					if(issueEntry == null)
						throw new AtomNotFoundException();

					return issueEntry;						

				} else if (choice.equals("versions")){

					VersionFeed versionFeed = new VersionFeed(username, password, projectKey, baseUri);
					int id = Integer.parseInt(choiceKey);					
					return versionFeed.getEntry(id);

				} else if (choice.equals("priorities")){

					PriorityFeed priorityFeed = new PriorityFeed(username, password, projectKey, baseUri);
					int id = Integer.parseInt(choiceKey);					
					return priorityFeed.getEntry(id);

				} else if (choice.equals("types")){

					TypeFeed typeFeed = new TypeFeed(username, password, projectKey, baseUri);
					int id = Integer.parseInt(choiceKey);					
					return typeFeed.getEntry(id);

				} else if (choice.equals("components")){

					ComponentFeed componentFeed = new ComponentFeed(username, password, projectKey, baseUri);
					int id = Integer.parseInt(choiceKey);					
					return componentFeed.getEntry(id);

				}
			}



			// /projects/{p}/issues/{i}/{choice}/{c}/
			//Were dealing with a state, comment or (attachment)
			else if(pathInfo.length == 6 && pathInfo[0].equalsIgnoreCase("projects") && pathInfo[2].equalsIgnoreCase("issues")){

				String projectKey = pathInfo[1];
				String issueKey = pathInfo[3];

				String choice =  pathInfo[4];

				String choiceKey = pathInfo[5];	
				int id = Integer.parseInt(choiceKey);		

				if(choice.equalsIgnoreCase("states")){

					StateFeed stateFeed = new StateFeed(username, password, projectKey, issueKey, baseUri);
					return stateFeed.getEntry(id);

				} else if (choice.equals("comments")){

					CommentFeed commentFeed = new CommentFeed(username, password, projectKey, issueKey, baseUri);
					return commentFeed.getEntry(id);

				} else if (choice.equals("attachments")){					

					return null;
					//Not yet implemented
				}
			}

		} catch (BadRequestException e) {
			// TODO Auto-generated catch block
			if (e instanceof UnAuthorizedException) {
				throw new AtomNotAuthorizedException(e.getMessage());
			}
			else if(e instanceof ForbiddenException) {
				throw new AtomNotFoundException("Forbidden: " + e.getMessage());
			}
			else {
				throw new AtomException(e.getMessage());					
			}
//			e.printStackTrace();
		} catch (InternalServerErrorException e) {
			// TODO Auto-generated catch block
//			e.printStackTrace();
			throw new AtomException(e.getMessage());
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new AtomException(e.getMessage());
		}			

		throw new AtomNotFoundException();
	}

	@Override
	public AtomMediaResource getMediaResource(AtomRequest req)
	throws AtomException {
		// TODO Auto-generated method stub

		String[] pathInfo = StringUtils.split(req.getPathInfo(),"/");	
		System.out.println("***** GetMediaResource *****");
		System.out.println("Uri: " + req.getRequestURL());
		System.out.println("getCollection " + req.toString());
		System.out.println("pathInfo: " +  pathInfo.toString());

//		AtomMediaResource r = new AtomMediaResource(new File());

		throw new AtomNotFoundException();
	}

	@Override
	public boolean isAtomServiceURI(AtomRequest req) {
		// TODO Auto-generated method stub		

		String[] pathInfo = StringUtils.split(req.getPathInfo(),"/");
//		System.out.println("***** isAtomService *****");
//		System.out.println("Uri: " + req.getRequestURL());
//		System.out.println("getCollection " + req.toString());
//		System.out.println("pathInfo: " +  pathInfo.toString());

		if (pathInfo.length == 0)
			return true;

		return false;
	}

	@Override
	public boolean isCategoriesURI(AtomRequest req) {
		// TODO Auto-generated method stub
		String[] pathInfo = StringUtils.split(req.getPathInfo(),"/");	
//		System.out.println("***** isCategories *****");
//		System.out.println("Uri: " + req.getRequestURL());
//		System.out.println("pathInfo: " +  pathInfo.toString());

		return false;
	}

	@Override
	public boolean isCollectionURI(AtomRequest req) {
		// TODO Auto-generated method stub

		String[] pathInfo = StringUtils.split(req.getPathInfo(),"/");	
//		System.out.println("***** isCollection *****");
//		System.out.println("Uri: " + req.getRequestURL());
//		System.out.println("getCollection " + req.toString());
//		System.out.println("pathInfo: " +  pathInfo.toString());



		// /projects/{projectname}/{*}/
		if(pathInfo.length == 3 && pathInfo[0].equalsIgnoreCase("projects")){	

			String star = pathInfo[2];

			//{*} = issues?
			if(star.equalsIgnoreCase("issues"))
				return true;

			//{*} = cachedissues?
			if(star.equalsIgnoreCase("cachedissues"))
				return true;

			//{*} = cachedissues?
			if(star.equalsIgnoreCase("supercachedissues"))
				return true;

			//{*} = ehcachedissues?
			if(star.equalsIgnoreCase("ehcachedissues"))
				return true;

			//{*} = ehcachedissues?
			if(star.equalsIgnoreCase("entrycachedissues"))
				return true;


			//{*} = versions?
			if(star.equalsIgnoreCase("versions"))
				return true;

			//{*} = components?
			if(star.equalsIgnoreCase("components"))
				return true;

			//{*} = types?
			if(star.equalsIgnoreCase("types"))
				return true;

			//{*} = priorities?
			if(star.equalsIgnoreCase("priorities"))
				return true;




//			String projectName = pathInfo[1];			
//			System.out.println("Project: " + projectName);	

			//The project exists!
//			if(Projects.projectExists(projectName)){	

//			System.out.println("IssueFeed!!");

			// {*} = versions
//			if(property.equalsIgnoreCase("versions")

			return true;
//			}			
		}

		// /projects/{projectname/issues/{issue}/{*}
		else if(pathInfo.length == 5 && pathInfo[0].equalsIgnoreCase("projects") && pathInfo[2].equalsIgnoreCase("issues")){

			String property = pathInfo[4];

			//{*} = states
			if(property.equalsIgnoreCase("states"))
				return true;

			//{*} = attachments
			if(property.equalsIgnoreCase("attachments"))
				return true;

			//{*} = comments
			if(property.equalsIgnoreCase("comments"))
				return true;			
		}

		// /projects/
		else if(pathInfo.length == 1 && pathInfo[0].equalsIgnoreCase("projects"))
		{
//			System.out.println("ProjectFeed!!");
			return true;
		}

//		if (pathInfo.length == 3 && pathInfo[2].equals("issues")) return true;

		return false;
	}

	@Override
	public boolean isEntryURI(AtomRequest req) {
		// TODO Auto-generated method stub

		String[] pathInfo = StringUtils.split(req.getPathInfo(),"/");	
//		System.out.println("***** isEntryUri *****");
//		System.out.println("Uri: " + req.getRequestURL());
//		System.out.println("pathInfo: " +  pathInfo.toString());

		// TODO: Check that project or issue exists??

		//We're dealing with a project
		if(pathInfo.length == 2 && pathInfo[0].equalsIgnoreCase("projects")){

//			System.out.println("Is project entry");

			return true;
		}

		//Were dealing with an issue, version, priority, type or component
		else if(pathInfo.length == 4 && pathInfo[0].equalsIgnoreCase("projects")){

			String choice =  pathInfo[2];

			if(choice.equalsIgnoreCase("issues")){
				return true;
			} else if (choice.equals("versions")){
				return true;
			} else if (choice.equals("priorities")){
				return true;
			} else if (choice.equals("types")){
				return true;
			} else if (choice.equals("components")){
				return true;
			}
		}

		// /projects/{p}/issues/{i}/{choice}/{c}/
		//Were dealing with a state, comment or (attachment)
		else if(pathInfo.length == 6 && pathInfo[0].equalsIgnoreCase("projects") && pathInfo[2].equalsIgnoreCase("issues")){

			String choice =  pathInfo[4];

			if(choice.equalsIgnoreCase("states")){
				return true;
			} else if (choice.equals("comments")){
				return true;
			} else if (choice.equals("attachments")){
				return false;
			}
		}

		return false;
	}

	@Override
	public boolean isMediaEditURI(AtomRequest req) {
		// TODO Auto-generated method stub
		String[] pathInfo = StringUtils.split(req.getPathInfo(),"/");	
//		System.out.println("***** isMediaEditUri *****");
//		System.out.println("Uri: " + req.getRequestURL());
//		System.out.println("getCollection " + req.toString());
//		System.out.println("pathInfo: " +  pathInfo.toString());

		return false;
	}

	@Override
	public String postEntry(AtomRequest req, Entry entry) throws AtomException {
		// TODO Auto-generated method stub

		System.out.println("Postentry.." + entry.getXmlBase() + " " + entry.getForeignMarkup());


//		InputStream is;
//		try {
//		is = req.getInputStream();
//		BufferedReader in = new BufferedReader(new InputStreamReader(is));

//		String line;
//		while ((line = in.readLine()) != null) {
//		System.out.println("Line: " + line);
//		}

//		} catch (IOException e) {
//		// TODO Auto-generated catch block
//		e.printStackTrace();
//		}

		String[] pathInfo = StringUtils.split(req.getPathInfo(),"/");	
		System.out.println("***** postEntry *****");
		System.out.println("Uri: " + req.getRequestURL());
		System.out.println("getCollection " + req.toString());
		System.out.println("pathInfo: " +  pathInfo.toString());

		int pathEnd = req.getRequestURL().indexOf(req.getPathInfo());
		String baseUri = req.getRequestURL().substring(0, pathEnd + 1);

		Enumeration<String> paramNames = req.getParameterNames();
		while(paramNames.hasMoreElements()){

			String paramName = paramNames.nextElement();			
			String param = req.getParameter(paramName);

			System.out.println("Paramname " + paramName + " param: " + param);

		}

		//We're dealing with a project
		if(pathInfo.length == 1 && pathInfo[0].equalsIgnoreCase("projects")){


			System.out.println("Is project entry");

			try {

				System.out.println(((Content)entry.getContents().get(0)).getValue());

				System.out.println(((Content)entry.getContents().get(0)).getType());

//				System.out.println(((Content)entry.getContents().get(1)).getValue());


				System.out.println(req.getInputStream().toString());

			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}

			String s = "";

			try {
				Project addedProject = Projects.addProject(baseUri + "projects/", username, password, entry);	
				return baseUri + PROJECTS + "/" + addedProject.getName();			
			} catch (BadRequestException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new AtomException(e.getMessage());
			} catch (InternalServerErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new AtomException(e.getMessage());
			}


		}

		//Were dealing with an issue
		else if(pathInfo.length == 3 && pathInfo[0].equalsIgnoreCase("projects") && pathInfo[2].equalsIgnoreCase("issues")){

			System.out.println("Is issue entry");

			String projectKey = pathInfo[1];

			Issue addedIssue;
			try {

				System.out.println("trying to add an issue");

				System.out.println(entry.getTitle() + ":" + entry.getSummary().getValue());

				addedIssue = Issues.addIssue(username, password, projectKey, entry);

				System.out.println("added issue? :" + addedIssue.getKey());

				IssueEntry addedEntry = new IssueEntry(baseUri + "projects/" + projectKey + "/issues/", addedIssue);

				System.out.println("added entry?? :" + addedEntry.getId());

				return addedEntry.getId();

			} catch (BadRequestException e) {


				e.printStackTrace();

				// TODO Auto-generated catch block
				if (e instanceof UnAuthorizedException) {
					throw new AtomNotAuthorizedException(e.getMessage());
				}
				else if(e instanceof ForbiddenException) {
					throw new AtomNotFoundException("Forbidden: " + e.getMessage());
				}
				else {
					throw new AtomException(e.getMessage());					
				}
			} catch (InternalServerErrorException e) {
				// TODO Auto-generated catch block

				e.printStackTrace();

				throw new AtomException(e.getMessage());
			} catch (Exception e) {
				// TODO: handle exception
				e.printStackTrace();

				throw new AtomException(e.getMessage());
			}

		} 
		//Were dealing with a comment
		else if(pathInfo.length == 5 && pathInfo[0].equalsIgnoreCase("projects") && pathInfo[2].equalsIgnoreCase("issues") && pathInfo[4].equalsIgnoreCase("comments")){

			String projectKey = pathInfo[1];
			String issueKey = pathInfo[3];

			String id;
			try {
				id = Comments.addComment(username, password, projectKey, issueKey, entry);
				return id;
			} catch (InternalServerErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new AtomException(e.getMessage());
			} catch (BadRequestException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new AtomNotAuthorizedException(e.getMessage());
			}

		}

		throw new AtomException();
	}

	@Override
	public String postMedia(AtomRequest req, Entry entry) throws AtomException {
		// TODO Auto-generated method stub
		String[] pathInfo = StringUtils.split(req.getPathInfo(),"/");	
		System.out.println("***** postMedia *****");
		System.out.println("Uri: " + req.getRequestURL());
		System.out.println("getCollection " + req.toString());
		System.out.println("pathInfo: " +  pathInfo.toString());

//		InputStream is;
//		try {
//		is = req.getInputStream();
//		BufferedReader in = new BufferedReader(new InputStreamReader(is));

//		String line;
//		while ((line = in.readLine()) != null) {
//		System.out.println("Line: " + line);
//		}

//		} catch (IOException e) {
//		// TODO Auto-generated catch block
//		e.printStackTrace();
//		}

		throw new AtomException();
	}

	@Override
	public void putEntry(AtomRequest req, Entry entry) throws AtomException {
		// TODO Auto-generated method stub

		String editUri = req.getRequestURL().toString();

		String[] pathInfo = StringUtils.split(req.getPathInfo(),"/");	
		System.out.println("***** putEntry *****");
		System.out.println("Uri: " + editUri);
		System.out.println("getCollection " + req.toString());
//		System.out.println("pathInfo: " +  pathInfo.toString());

		int pathEnd = req.getRequestURL().indexOf(req.getPathInfo());
		String baseUri = req.getRequestURL().substring(0, pathEnd + 1);

		Enumeration<String> paramNames = req.getParameterNames();
		while(paramNames.hasMoreElements()){

			String paramName = paramNames.nextElement();			
			String param = req.getParameter(paramName);

			System.out.println("Paramname " + paramName + " param: " + param);			
		}

		//We're dealing with a project
		if(pathInfo.length == 2 && pathInfo[0].equalsIgnoreCase("projects")){
			System.out.println("Is project entry");

			String projectKey = pathInfo[1];

			try {
				Projects.updateProject(baseUri + "projects/", username, password, projectKey, entry);
			} catch (BadRequestException e) {

				e.printStackTrace();
				// TODO Auto-generated catch block
				if(e instanceof NotFoundException){
					throw new AtomNotFoundException(e.getMessage());
				} else {
					throw new AtomException(e.getMessage());
				}				

			} catch (InternalServerErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();				
				throw new AtomException(e.getMessage());
			}

		}



		//Were dealing with an issue
		else if(pathInfo.length == 4 && pathInfo[0].equalsIgnoreCase("projects") && pathInfo[2].equalsIgnoreCase("issues")){

			System.out.println("Is issue entry");

			String projectKey = pathInfo[1];

			String issueKeyAndVersion = pathInfo[3];			

			String issueKey = issueKeyAndVersion.substring(0, issueKeyAndVersion.indexOf(ConcurrencyManager.VERSION_PREFIX) - 1);

			String entryId = editUri.substring(0, editUri.indexOf(ConcurrencyManager.VERSION_PREFIX));

			System.out.println("Issuekey " + issueKey);
			System.out.println("Edituri " + editUri);
			System.out.println("EntryId " + editUri);

			
			/**
			 * Using the same optimistic concurrency approach as GDATA
			 */
			if(!ConcurrencyManager.getEditUri(entryId).equals(editUri)){

				throw new AtomException("Conflict! - Someone has updated this issue");	

			} else {

				ConcurrencyManager.incrementEditUri(entryId);
			}


			try {
				Issue updatedIssue = Issues.updateIssue(username, password, projectKey, entry);
			} catch (BadRequestException e) {
				// TODO Auto-generated catch block
				if (e instanceof UnAuthorizedException) {
					throw new AtomNotAuthorizedException(e.getMessage());
				}
				else if(e instanceof ForbiddenException) {
					throw new AtomNotFoundException("Forbidden: " + e.getMessage());
				}
				else {
					throw new AtomException(e.getMessage());					
				}
			} catch (InternalServerErrorException e) {
				// TODO Auto-generated catch block
				throw new AtomException(e.getMessage());
			}

//			Issue addedIssue = Projects.addIssue(projectKey, entry);

//			IssueEntry addedEntry = new IssueEntry(BASE + projectKey + "/issues/", addedIssue);

		}


	}



	@Override
	public void putMedia(AtomRequest req) throws AtomException {
		// TODO Auto-generated method stub
		String[] pathInfo = StringUtils.split(req.getPathInfo(),"/");	
		System.out.println("***** putMedia *****");
		System.out.println("Uri: " + req.getRequestURL());
		System.out.println("getCollection " + req.toString());
		System.out.println("pathInfo: " +  pathInfo.toString());


//		InputStream is;
//		try {
//		is = req.getInputStream();
//		BufferedReader in = new BufferedReader(new InputStreamReader(is));

//		String line;
//		while ((line = in.readLine()) != null) {
//		System.out.println("Line: " + line);
//		}

//		} catch (IOException e) {
//		// TODO Auto-generated catch block
//		e.printStackTrace();
//		}

	}

}
