package no.miles.mpl.semantic.app.endpoint.entries;


import java.util.ArrayList;
import java.util.Date;

import no.miles.mpl.semant.app.endpoint.datahandlers.ConcurrencyManager;
import no.miles.mpl.wrapper.datastructures.Issue;

import com.sun.syndication.feed.atom.Content;
import com.sun.syndication.feed.atom.Entry;
import com.sun.syndication.feed.atom.Link;
import com.sun.syndication.feed.atom.Person;

public class IssueEntry extends Entry {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2401154565972446112L;

	private Issue issue;

	/**
	 * 
	 * @param baseURI
	 * @param issue
	 * @param contentType
	 */
	public IssueEntry(String baseURI, Issue issue, String contentType){

		super();

		createEntry(baseURI, issue, contentType);


	}

	/**
	 * Default constructor - returns N3 in content
	 * @param baseURI
	 * @param issue
	 */
	public IssueEntry(String baseURI, Issue issue) {

		super();

		createEntry(baseURI, issue, "text/rdf+n3");	



	}



	private void createEntry(String baseURI, Issue issue, String contentType) {

		this.issue = issue;

		this.setId(baseURI + issue.getKey());			
		this.setTitle(issue.getSummary());	

		Date updated = new Date();

		if(issue.getUpdated() != null)
		{
			updated = issue.getUpdated();
		}

		this.setUpdated(updated);

		Link permaLink = new Link();
		permaLink.setRel("alternate");
		permaLink.setHref(issue.getViewissueurl());

		Link xmlLink = new Link();
		xmlLink.setRel("alternate");
		xmlLink.setType("application/xml");
		xmlLink.setHref(baseURI + issue.getKey() + "?contentformat=application/xml");

		//Add alternate links
		ArrayList<Link> alternateLinkList = new ArrayList<Link>();
		alternateLinkList.add(permaLink);
		alternateLinkList.add(xmlLink);
		this.setAlternateLinks(alternateLinkList);


		Link about = new Link();
		about.setRel("about");
		about.setHref("http://xmlns.com/baetle/#Issue");

		
		/**
		 * Deals with concurrency
		 */
		Link edit = new Link();
		edit.setRel("edit");		
		String editUri = ConcurrencyManager.getEditUri(this.getId());		
		edit.setHref(editUri);

		//Add the otherlinks
		ArrayList<Link> otherLinkList = new ArrayList<Link>();		
		otherLinkList.add(about);
		otherLinkList.add(edit);
		this.setOtherLinks(otherLinkList);

		Content summary = new Content();
		summary.setValue(issue.getDescription());
		summary.setType("text");
		this.setSummary(summary);

		Person reporter = new Person();
		reporter.setName(issue.getReported_by());
		ArrayList<Person> authorList = new ArrayList<Person>();
		authorList.add(reporter);	
		this.setAuthors(authorList);

		if(contentType == null || contentType.equalsIgnoreCase("text/rdf+n3")){
			//Add n3 representation to Content		
			Content content = new Content();		
			String n3 = issue.toN3();		
			content.setValue(n3);
			content.setType("text/rdf+n3");		
			ArrayList<Content> contents = new ArrayList<Content>();
			contents.add(content);		
			this.setContents(contents);
		} else if(contentType.equalsIgnoreCase("application/xml")){
			Content content = new Content();		
			String xml = issue.toXml();		
			content.setValue(xml);
			content.setType("application/xml");		
			ArrayList<Content> contents = new ArrayList<Content>();
			contents.add(content);		
			this.setContents(contents);
			
		}
	}









}
