package no.miles.mpl.semantic.app.endpoint.feeds;

import java.util.ArrayList;
import java.util.Date;

import no.miles.mpl.semant.app.endpoint.datahandlers.Issues;
import no.miles.mpl.semant.app.endpoint.datahandlers.Projects;
import no.miles.mpl.semantic.app.endpoint.entries.IssueEntry;
import no.miles.mpl.wrapper.datastructures.Issue;
import no.miles.mpl.wrapper.datastructures.Project;
import no.miles.mpl.wrapper.exception.BadRequestException;
import no.miles.mpl.wrapper.exception.InternalServerErrorException;

import com.sun.syndication.feed.atom.Entry;
import com.sun.syndication.feed.atom.Feed;

public class CachedIssueFeed extends Feed {

	private String username;
	private String password;

	public CachedIssueFeed(String username, String password, String projectName, String baseUrl) throws BadRequestException, InternalServerErrorException{
		super();

		this.username = username;
		this.password = password;

		this.setId(baseUrl + "projects/" + projectName + "/cachedissues/");
		this.setTitle(projectName + "cached issues");
		this.setUpdated(new Date());		

		try{
			Project project = Projects.getProject(baseUrl + "projects/", projectName, username, password);
			
			this.setEntries(getIssueEntries(project));
		} catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
			throw new InternalServerErrorException(e.getMessage());
		}
	}

	private ArrayList<Entry> getIssueEntries(Project project) throws BadRequestException, InternalServerErrorException{

		ArrayList<Entry> entries = new ArrayList<Entry>();

//		ArrayList<Issue> issues = project.getIssues();

		Issue[] issues = Issues.getCachedIssues(username, password, project);
		
		if(issues == null) {
			throw new BadRequestException("No issues found for this project");
		}

		long fetchedIssuesTime = System.currentTimeMillis();

//		System.out.println("Fetched " + issues.length + " issues at time: " + fetchedIssuesTime);

//		System.out.println("Fetched " + issues.length +" issues.." + new Date().toString());		

		try{

			for (Issue issue : issues) {
				Entry entry = new IssueEntry(getId(), issue);
				entries.add(entry);
			}
			
		}catch (Exception e) {
			e.printStackTrace();
			// TODO: handle exception
		}

//		for (Issue issue : issues) {

//		Entry entry = new IssueEntry(getId(), issue);			
//		entries.add(entry);
//		}

		long convertedIssuesTime = System.currentTimeMillis();

		System.out.println("Converted the issues in " + (convertedIssuesTime - fetchedIssuesTime) + " millis");

//		System.out.println("Converted issues to entries.." + new Date().toString());

		return entries;
	}
	
}
