package no.miles.mpl.semantic.app.endpoint.feeds;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import no.miles.mpl.semant.app.endpoint.datahandlers.Issues;
import no.miles.mpl.semant.app.endpoint.datahandlers.JenaHandler;
import no.miles.mpl.semant.app.endpoint.datahandlers.Projects;
import no.miles.mpl.semantic.app.endpoint.entries.SparqlResultEntry;
import no.miles.mpl.wrapper.datastructures.Issue;
import no.miles.mpl.wrapper.datastructures.Project;
import no.miles.mpl.wrapper.exception.BadRequestException;
import no.miles.mpl.wrapper.exception.InternalServerErrorException;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.shared.Lock;
import com.hp.hpl.jena.sparql.engine.binding.Binding;
import com.hp.hpl.jena.util.FileManager;
import com.sun.syndication.feed.atom.Entry;
import com.sun.syndication.feed.atom.Feed;

public class IssueSparqlResultFeed extends Feed {

	public String username;
	public String password;


	public static String BAETLENS = "http://xmlns.com/baetle/#";
	public static String RDFNS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	public static String WFNS = "http://www.w3.org/2005/01/wf/flow#";
	public static String FOAFNS = "http://xmlns.com/foaf/0.1/";
	public static String SIOCNS = "http://rdfs.org/sioc/ns#";
	public static String DOAPNS = "http://usefulinc.com/ns/doap#";

	public IssueSparqlResultFeed(String username, String password, String projectName, String baseUrl, String sparqlQuery) throws BadRequestException, InternalServerErrorException{
		super();

		this.username = username;
		this.password = password;

//		this.setId(baseUrl + "projects/" + projectName + "/issues/?sparql=" + sparqlQuery);
		this.setId(baseUrl + "projects/" + projectName + "/issues/?sparql=" + "");

		this.setTitle(projectName + " issues");
		this.setUpdated(new Date());		

		Project project = Projects.getProject(baseUrl + "projects/", projectName, username, password);	
		this.setEntries(getSparqlResult(project, sparqlQuery));
	}

	private ArrayList<Entry> getSparqlResult(Project project, String sparqlQuery) throws BadRequestException, InternalServerErrorException{

		ArrayList<Entry> entries = new ArrayList<Entry>();		
		Issue[] issues = Issues.getIssues(username, password, project);

		long fetchedIssuesTime = System.currentTimeMillis();

		System.out.println("Fetched " + issues.length + " issues at time: " + fetchedIssuesTime);

		//Load the ontology
		OntModel model = ModelFactory.createOntologyModel();
		loadN3(model, JenaHandler.BAETLEN3);

		try{
			for (Issue issue : issues) {					
				String n3Content = issue.toN3();				
				ByteArrayInputStream inputStream = new ByteArrayInputStream(n3Content.getBytes());
				model.read(inputStream, "", "N3");
			}
		}catch (Exception e) {
			e.printStackTrace();
			// TODO: handle exception
		}

		//Convert to string
//		ByteArrayOutputStream out = new ByteArrayOutputStream();
//		BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
//		model.write(bufferedOut, "N3");
//		String ontology = out.toString();		
//		System.out.println(ontology);

		Query query = QueryFactory.create(sparqlQuery);

		model.enterCriticalSection(Lock.READ);

		try{

			// Execute the query and obtain results
			QueryExecution qe = QueryExecutionFactory.create(query, model);

			try { 

				ResultSet rs = qe.execSelect();

				String xml = ResultSetFormatter.asXMLString(rs);

//				Resource rdf = ResultSetFormatter.asRDF(model, rs);

//				System.out.println(rdf.toString());

//				System.out.println(xml);

				InputStream in = new ByteArrayInputStream(xml.getBytes());

				BufferedInputStream bufferedIn = new BufferedInputStream(in);

				SAXBuilder builder = new SAXBuilder();		

				int i = 1;

				try {
					Document doc = builder.build(bufferedIn);
					Element root = doc.getRootElement();

//					System.out.println("root " + root.getName());

					Element resultsElement = root.getChild("results", null);

//					List<Element> children = root.getChildren();
//					for (Element element2 : children) {
//					if(element2.getName().equalsIgnoreCase("results"))
//					resultsElement = element2;
//					}

					if(resultsElement != null){
//						System.out.println("results exist..");
						List<Element> results = resultsElement.getChildren("result", null);
						for (Element element : results) {
							if(element.getName().equalsIgnoreCase("result")){
								Entry entry = new SparqlResultEntry(this.getId() + "&result=" + 1, element);				
								i++;		
								entries.add(entry);
//								System.out.println("added entry..");
							}
						}

					}

				} finally {

					qe.close();

				}

			} catch (JDOMException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();			
			} 


		} finally {

			model.leaveCriticalSection();
			model.close();

		}



//		String result = "";
//		while (rs.hasNext())
//		{


//		QuerySolution qs = rs.nextSolution();

//		Binding binding = rs.nextBinding();



//		System.out.println(qs.toString());

//		Iterator<String> varNames = qs.varNames();

//		while(varNames.hasNext()){

//		String qResult = "";



//		qResult += qs.get(varNames.next()) + " ";	


//		result += qResult + "\n";

//		}

//		}

//		System.out.println(result);


//		long convertedIssuesTime = System.currentTimeMillis();

		return entries;
	}

	/**
	 * Load the ontology..
	 * @param model
	 * @param fileName
	 */
	private static void loadN3(OntModel model, String fileName)
	{		
		// use the FileManager to find the input file
		InputStream in = FileManager.get().open( fileName );
		if (in == null) {
			throw new IllegalArgumentException(
					"File: " + fileName + " not found");
		}
		model.read(in, "", "N3");
	}

}
