package no.miles.mpl.semantic.app.endpoint.feeds;

import java.util.ArrayList;
import java.util.Date;

import no.miles.mpl.semant.app.endpoint.datahandlers.Projects;
import no.miles.mpl.semantic.app.endpoint.entries.PriorityEntry;
import no.miles.mpl.wrapper.BugTrackerWrapperInterface;
import no.miles.mpl.wrapper.exception.BadRequestException;
import no.miles.mpl.wrapper.exception.InternalServerErrorException;

import com.sun.syndication.feed.atom.Entry;
import com.sun.syndication.feed.atom.Feed;

public class PriorityFeed extends Feed {

	private String username;
	private String password;
	private String projectKey;
	private String baseUri;
	
	/**
	 * Constructor - instantiates the feed..
	 * @param username
	 * @param password
	 * @param projectKey
	 * @param baseUri
	 * @throws InternalServerErrorException 
	 * @throws BadRequestException 
	 */
	public PriorityFeed(String username, String password, String projectKey, String baseUri) throws BadRequestException, InternalServerErrorException{
		
		this.username = username;
		this.password = password;
		this.baseUri = baseUri;
		this.projectKey = projectKey;
		
		this.setId(baseUri + "/projects/" + projectKey + "/priorities/");
		this.setTitle(projectKey + " priorities");
		this.setUpdated(new Date());
		
		this.updateEntries();
	}
	
	/**
	 * Update the entries..
	 * @throws BadRequestException
	 * @throws InternalServerErrorException
	 */
	private void updateEntries() throws BadRequestException, InternalServerErrorException{
		
		BugTrackerWrapperInterface bugTracker = Projects.getBugTracker(baseUri + "/projects/", projectKey, username, password);
		
		int i = 0;
		
		ArrayList<Entry> entries = new ArrayList<Entry>();
		
		String[] components = bugTracker.getPriorities(username, password, projectKey);
		for (String component : components) {
			
			Entry entry = new PriorityEntry(baseUri, i, component);	
			entries.add(entry);
			i++;
		}
		
		this.setEntries(entries);
		
		
	}
	
	/**
	 * Get at entry with the specified
	 * @return
	 */
	public Entry getEntry(int id){
		
		return (Entry) this.getEntries().get(id);
		
	}
	
}
