package no.miles.mpl.semantic.app.endpoint.feeds;

import java.util.ArrayList;
import java.util.Date;

import no.miles.mpl.semant.app.endpoint.datahandlers.Projects;
import no.miles.mpl.semantic.app.endpoint.entries.StateEntry;
import no.miles.mpl.wrapper.BugTrackerWrapperInterface;
import no.miles.mpl.wrapper.datastructures.Comment;
import no.miles.mpl.wrapper.exception.BadRequestException;
import no.miles.mpl.wrapper.exception.InternalServerErrorException;

import com.sun.syndication.feed.atom.Entry;
import com.sun.syndication.feed.atom.Feed;

public class StateFeed extends Feed {
	
	private String username;
	private String password;
	private String projectKey;
	private String issueKey;
	private String baseUrl;
	
	/**
	 * Constructor that initializes the 
	 * @param username
	 * @param password
	 * @param projectKey
	 * @param issueKey
	 * @param baseUrl
	 * @throws InternalServerErrorException 
	 * @throws BadRequestException 
	 */
	public StateFeed(String username, String password, String projectKey, String issueKey, String baseUrl) throws BadRequestException, InternalServerErrorException{
		
		this.username = username;
		this.password = password;
		this.projectKey = projectKey;
		this.issueKey = issueKey;
		this.baseUrl = baseUrl;
		
		this.setId(baseUrl + "projects/" + projectKey + "/issues/" + issueKey + "/states/");
		this.setTitle(projectKey + " : " + issueKey + " states");
		this.setUpdated(new Date());
		
		this.updateEntries();
	}

	/**
	 * Update the entries..
	 * @throws InternalServerErrorException 
	 * @throws BadRequestException 
	 */
	private void updateEntries() throws BadRequestException, InternalServerErrorException{
		
		BugTrackerWrapperInterface bugTracker = Projects.getBugTracker(baseUrl + "/projects", projectKey, username, password);
		
		ArrayList<Entry> entries = new ArrayList<Entry>();
		
		String[] states = bugTracker.getStates(username, password, projectKey, issueKey);
				
		int i = 0;
		
		for (String state : states) {
			
			Entry entry = new StateEntry(this.getId(), i, state);
			entries.add(entry);
			
			i++;
		}
		
		this.setEntries(entries);
	}
	
	/**
	 * Get at entry with the specified
	 * @return
	 */
	public Entry getEntry(int id){
		
		return (Entry) this.getEntries().get(id);
		
	}

}
