import java.util.ArrayList;

import no.miles.mpl.semant.app.endpoint.datahandlers.Projects;
import no.miles.mpl.wrapper.datastructures.IssueTrackers;
import no.miles.mpl.wrapper.datastructures.Project;
import no.miles.mpl.wrapper.exception.BadRequestException;
import no.miles.mpl.wrapper.exception.InternalServerErrorException;

import antlr.collections.List;

import com.sun.syndication.feed.atom.Content;
import com.sun.syndication.feed.atom.Entry;

import junit.framework.TestCase;



public class TestProjects extends TestCase {

	private static String username = "test";
	private static String password = "test";
	private static String baseUri = "http://localhost/projects/";

	private static boolean runIntegrationTests = false;

	private Project createDefaultProject(){

		Project p = new Project("jihaa");
		p.setBugtrackertype(IssueTrackers.JIRA);
		p.setBugtrackerurl("http://localhost/jira");
		p.setUrl("http://localhost/projects/jihaa");
		return p;

	}

	private Project createDefaultUpdatedProject(){

		Project p = new Project("jihaa");
		p.setBugtrackertype(IssueTrackers.TRAC);
		p.setBugtrackerurl("http://localhost/trac");
		p.setUrl("http://localhost/projects/jihaa");
		return p;

	}

	private Entry createDefaultEntry(){

		Project p = createDefaultProject();
		Entry entry = new Entry();
		Content content = new Content();
		content.setType("text/rdf+n3");
		content.setValue(p.toN3());

		ArrayList contents = new ArrayList<Content>();
		contents.add(content);
		entry.setContents(contents);

		return entry;

	}

	private Entry createDefaultUpdatedEntry(){

		Project p = createDefaultUpdatedProject();

		Entry entry = new Entry();
		Content content = new Content();
		content.setType("text/rdf+n3");
		content.setValue(p.toN3());

		ArrayList contents = new ArrayList<Content>();
		contents.add(content);
		entry.setContents(contents);

		return entry;

	}

	public void testAddProject() throws Exception {

		if(runIntegrationTests){

			Entry entry = createDefaultEntry();

			try {
				Projects.addProject(baseUri, username, password, entry);
			} catch (BadRequestException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InternalServerErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}


	}

	public void testDeleteProject() throws Exception {


		if(runIntegrationTests){

			Entry entry = createDefaultEntry();

			Project project = createDefaultProject();

			//Make sure the project is added..
			try {
				project = Projects.addProject(baseUri, username, password, entry);
			} catch (BadRequestException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InternalServerErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			Projects.deleteProject(baseUri, username, password, project.getName());		

			Project deleted = Projects.getProject(baseUri, project.getName(), username, password);

			assertEquals(null, deleted);

		}

	}

	public void testUpdateProject() throws Exception {

		//Make sure project is added


		if(runIntegrationTests){

			try{
				Entry entry = createDefaultEntry();				
				Projects.addProject(baseUri, username, password, entry);
			} catch (Exception e) {
				// TODO: handle exception
				System.out.println("Project existed - ok");
			}

			Entry updatedEntry = createDefaultUpdatedEntry();

			Project project = createDefaultUpdatedProject();

			Projects.updateProject(baseUri, username, password, project.getName(), updatedEntry);

			Project updadetProject = Projects.getProject(baseUri, project.getName(), username, password);

			assertEquals(project.getBugtrackertype(), updadetProject.getBugtrackertype());
			assertEquals(project.getBugtrackerurl(), updadetProject.getBugtrackerurl());

		}

	}


	public void testGetProjects() throws Exception {


		if(runIntegrationTests){

			Projects.getProjects(username, password);

		}

	}

}
