package no.miles.mpl.wrapper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import no.miles.mpl.wrapper.datastructures.IssueStates;
import no.miles.mpl.wrapper.datastructures.IssueTypes;
import no.miles.mpl.wrapper.datastructures.PriorityTypes;

public class JiraMappings {

	private static TracMappings instance = null;

	private static Map<String, String> jiraToMplMap;
	private static Map<String, String> mplToJiraMap;

	private static Map<String, String> jiraToMplPriorityMap;
	private static Map<String, String> mplToJiraPriorityMap;

	private static Map<String, String> jiraResolutionToMplStateMap;
	private static Map<String, String> mplStateToJiraResolutionMap;

	private static Map<String, String> jiraStatusToMplStateMap;
	private static Map<String, String> mplStateToJiraStatusMap;

	private static ArrayList<String> openStates;
	private static ArrayList<String> resolvedStates;

	static {

		/**
		 * Open + subclasses
		 */
		openStates = new ArrayList<String>();
		openStates.add(IssueStates.STATE_OPEN);
		openStates.add(IssueStates.STATE_NEW);
		openStates.add(IssueStates.STATE_UNCONFIRMED);
//		openStates.add(IssueStates.STATE_REOPENED);
//		openStates.add(IssueStates.STATE_STARTED);
		
		resolvedStates = new ArrayList<String>();
		resolvedStates.add(IssueStates.STATE_RESOLVED);
		resolvedStates.add(IssueStates.STATE_FIXED);
		resolvedStates.add(IssueStates.STATE_DUPLICATE);
		resolvedStates.add(IssueStates.STATE_INCOMPLETE);
		resolvedStates.add(IssueStates.STATE_LATER);
		resolvedStates.add(IssueStates.STATE_NOT_REPRODUCIBLE);
		resolvedStates.add(IssueStates.STATE_REMIND);
		resolvedStates.add(IssueStates.STATE_WONT_FIX);
		resolvedStates.add(IssueStates.STATE_WORKS_FOR_ME);

		jiraResolutionToMplStateMap = new HashMap<String, String>();
		jiraResolutionToMplStateMap.put("Fixed", IssueStates.STATE_FIXED);
		jiraResolutionToMplStateMap.put("Won't Fix", IssueStates.STATE_WONT_FIX);
		jiraResolutionToMplStateMap.put("Duplicate", IssueStates.STATE_DUPLICATE);
		jiraResolutionToMplStateMap.put("Incomplete", IssueStates.STATE_INCOMPLETE);
		jiraResolutionToMplStateMap.put("Cannot Reproduce", IssueStates.STATE_NOT_REPRODUCIBLE);

		mplStateToJiraResolutionMap = new HashMap<String, String>();
		mplStateToJiraResolutionMap.put(IssueStates.STATE_FIXED, "Fixed");
		mplStateToJiraResolutionMap.put(IssueStates.STATE_WONT_FIX, "Won't Fix");
		mplStateToJiraResolutionMap.put(IssueStates.STATE_DUPLICATE, "Duplicate");
		mplStateToJiraResolutionMap.put(IssueStates.STATE_INCOMPLETE, "Incomplete");
		mplStateToJiraResolutionMap.put(IssueStates.STATE_NOT_REPRODUCIBLE, "Cannot Reproduce");

		jiraStatusToMplStateMap = new HashMap<String, String>();
		jiraStatusToMplStateMap.put("Open", IssueStates.STATE_OPEN);
		jiraStatusToMplStateMap.put("In Progress", IssueStates.STATE_STARTED);
		jiraStatusToMplStateMap.put("Reopened", IssueStates.STATE_REOPENED);
		jiraStatusToMplStateMap.put("Resolved", IssueStates.STATE_RESOLVED);
		jiraStatusToMplStateMap.put("Closed", IssueStates.STATE_CLOSED);

		mplStateToJiraStatusMap = new HashMap<String, String>();
		mplStateToJiraStatusMap.put(IssueStates.STATE_CLOSED, "Closed");
		mplStateToJiraStatusMap.put(IssueStates.STATE_DUPLICATE, "Resolved");
		mplStateToJiraStatusMap.put(IssueStates.STATE_FIXED, "Resolved");
		mplStateToJiraStatusMap.put(IssueStates.STATE_INCOMPLETE, "Resolved");
		mplStateToJiraStatusMap.put(IssueStates.STATE_LATER, "Resolved");
		mplStateToJiraStatusMap.put(IssueStates.STATE_NEW, "Open");		
		//Have to guess..:
		mplStateToJiraStatusMap.put(IssueStates.STATE_NON_TERMINAL_STATE, "Open");		
		mplStateToJiraStatusMap.put(IssueStates.STATE_NOT_REPRODUCIBLE, "Resolved");
		mplStateToJiraStatusMap.put(IssueStates.STATE_OPEN, "Open");
		mplStateToJiraStatusMap.put(IssueStates.STATE_REMIND, "Resolved");
		mplStateToJiraStatusMap.put(IssueStates.STATE_REOPENED, "Reopened");
		mplStateToJiraStatusMap.put(IssueStates.STATE_RESOLVED, "Resolved");
		mplStateToJiraStatusMap.put(IssueStates.STATE_STARTED, "In Progress");
		//Guess again:
		mplStateToJiraStatusMap.put(IssueStates.STATE_STATE, "Open");
		mplStateToJiraStatusMap.put(IssueStates.STATE_TERMINAL_STATE, "Closed");
		mplStateToJiraStatusMap.put(IssueStates.STATE_UNCONFIRMED, "Open");
		//Guess?
		mplStateToJiraStatusMap.put(IssueStates.STATE_VERIFIED, "Open");
		mplStateToJiraStatusMap.put(IssueStates.STATE_WONT_FIX, "Resolved");
		mplStateToJiraStatusMap.put(IssueStates.STATE_WORKS_FOR_ME, "Resolved");		

		jiraToMplMap = new HashMap<String, String>();
		jiraToMplMap.put("Bug", IssueTypes.ISSUE_TYPE_DEFECT);
		jiraToMplMap.put("New Feature", IssueTypes.ISSUE_TYPE_ENHANCEMENT);
		jiraToMplMap.put("Improvement", IssueTypes.ISSUE_TYPE_ENHANCEMENT);
		jiraToMplMap.put("Task", IssueTypes.ISSUE_TYPE_TASK);

		mplToJiraMap = new HashMap<String, String>();
		mplToJiraMap.put(IssueTypes.ISSUE_TYPE_DEFECT, "Bug");
		mplToJiraMap.put(IssueTypes.ISSUE_TYPE_ENHANCEMENT, "Improvement");
		mplToJiraMap.put(IssueTypes.ISSUE_TYPE_ENHANCEMENT, "New Feature");
		mplToJiraMap.put(IssueTypes.ISSUE_TYPE_TASK, "Task");

		jiraToMplPriorityMap = new HashMap<String, String>();
		jiraToMplPriorityMap.put("Critical", PriorityTypes.PRIORITY_CRITICAL);
		jiraToMplPriorityMap.put("Blocker", PriorityTypes.PRIORITY_BLOCKER);
		jiraToMplPriorityMap.put("Major", PriorityTypes.PRIORITY_MAJOR);
		jiraToMplPriorityMap.put("Minor", PriorityTypes.PRIORITY_MINOR);
		jiraToMplPriorityMap.put("Trivial", PriorityTypes.PRIORITY_TRIVIAL);

		mplToJiraPriorityMap = new HashMap<String, String>();
		mplToJiraPriorityMap.put(PriorityTypes.PRIORITY_CRITICAL, "Critical");
		mplToJiraPriorityMap.put(PriorityTypes.PRIORITY_BLOCKER, "Blocker");
		mplToJiraPriorityMap.put(PriorityTypes.PRIORITY_MAJOR, "Major");
		mplToJiraPriorityMap.put(PriorityTypes.PRIORITY_MINOR, "Minor");
		mplToJiraPriorityMap.put(PriorityTypes.PRIORITY_TRIVIAL, "Trivial");



	}

	/**
	 * Takes in a status and resolution and returns the best matching state
	 * @param status
	 * @param resolution
	 * @return
	 */
	public static String parseState(String status, String resolution){

//		System.out.println("Parsing.. Status: " + status + " resolution: " + resolution);
		
		String state = null;

		//Make sure Closed++ with resolution are not mapped to subclasses of Resolved..
		if(status.equalsIgnoreCase("Resolved")){

			state = jiraResolutionToMplStateMap.get(resolution);

		}

		//If no resolution is found, try to map the status
		if(state == null){
			state = jiraStatusToMplStateMap.get(status);
		}

		//If not - make it the most general state
		if(state == null)
			state = IssueStates.STATE_STATE;

//		System.out.println("Returning state: " + state);
		
		return  state;

	}

	/**
	 * Takes a state an returns the best matching status
	 * @param state
	 * @return
	 */
	public static String buildStatus(String state){
		
//		System.out.println("Building status from state: " + state);
		
		if(openStates.contains(state)){
			return "Open";
		} else if(state.equalsIgnoreCase(IssueStates.STATE_STARTED)){
			return "In Progress";
		} else if(state.equalsIgnoreCase(IssueStates.STATE_REOPENED)){
			return "Reopened";
		} else if(resolvedStates.contains(state)){
			return "Resolved";
		} else if(state.equalsIgnoreCase(IssueStates.STATE_CLOSED) || state.equalsIgnoreCase(IssueStates.STATE_TERMINAL_STATE)){
			return "Closed";
		}		

		//Guess that the status = open..
		return "Open";

	}

	/**
	 * Takse a state and returns the best matching resolution
	 * @param state
	 * @return
	 */
	public static String buildResolution(String state){

		//This should be trivial if the correct mappings are made in the Map
		String resolution = mplStateToJiraResolutionMap.get(state);

		return resolution;	


	}

	public static String parsePriority(String jiraPriority){

		String mplPriority = jiraToMplPriorityMap.get(jiraPriority);
		if(mplPriority != null)
			return mplPriority;

		return  PriorityTypes.PRIORITY;

	}

	public static String buildPriority(String mplPriority){

		String jiraPriority = mplToJiraPriorityMap.get(mplPriority);
		if(jiraPriority != null)
			return jiraPriority;

		//What to return if no defined mapping? i.e. baetle:Priority - gambling on Major :D
		return "Major";
	}

	public static String parseType(String jiraType){

		String mplType = jiraToMplMap.get(jiraType);
		if(mplType != null)
			return mplType;

		return IssueTypes.ISSUE_TYPE_ISSUE;

	}

	public static String buildType(String mplType){

		String jiraType = mplToJiraMap.get(mplType);
		if(jiraType != null)
			return jiraType;

		return "Task";

	}

}
