package no.miles.mpl.wrapper;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.SocketException;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.mail.Flags;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.URLName;
import javax.mail.Flags.Flag;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.SSLSocketFactory;
import javax.xml.rpc.ServiceException;

import org.apache.commons.net.SocketFactory;
import org.apache.commons.net.pop3.POP3Client;
import org.apache.commons.net.pop3.POP3MessageInfo;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;

import no.miles.mpl.wrapper.datastructures.Comment;
import no.miles.mpl.wrapper.datastructures.Issue;
import no.miles.mpl.wrapper.datastructures.IssueStates;
import no.miles.mpl.wrapper.datastructures.IssueTypes;
import no.miles.mpl.wrapper.datastructures.PriorityTypes;
import no.miles.mpl.wrapper.exception.BadRequestException;
import no.miles.mpl.wrapper.exception.ForbiddenException;
import no.miles.mpl.wrapper.exception.InternalServerErrorException;
import no.miles.mpl.wrapper.exception.NotFoundException;
import no.miles.mpl.wrapper.exception.UnAuthorizedException;

import com.atlassian.jira.rpc.soap.client.JiraSoapService;

import com.atlassian.jira.rpc.soap.client.JiraSoapServiceService;
import com.atlassian.jira.rpc.soap.client.JiraSoapServiceServiceLocator;
import com.atlassian.jira.rpc.soap.client.RemoteAttachment;
import com.atlassian.jira.rpc.soap.client.RemoteAuthenticationException;
import com.atlassian.jira.rpc.soap.client.RemoteComment;
import com.atlassian.jira.rpc.soap.client.RemoteComponent;
import com.atlassian.jira.rpc.soap.client.RemoteCustomFieldValue;
import com.atlassian.jira.rpc.soap.client.RemoteField;
import com.atlassian.jira.rpc.soap.client.RemoteFieldValue;
import com.atlassian.jira.rpc.soap.client.RemoteFilter;
import com.atlassian.jira.rpc.soap.client.RemoteIssue;
import com.atlassian.jira.rpc.soap.client.RemoteIssueType;
import com.atlassian.jira.rpc.soap.client.RemoteNamedObject;
import com.atlassian.jira.rpc.soap.client.RemotePermissionException;
import com.atlassian.jira.rpc.soap.client.RemotePriority;
import com.atlassian.jira.rpc.soap.client.RemoteResolution;
import com.atlassian.jira.rpc.soap.client.RemoteStatus;
import com.atlassian.jira.rpc.soap.client.RemoteValidationException;
import com.atlassian.jira.rpc.soap.client.RemoteVersion;
import com.atlassian.jira.rpc.soap.client.ext.JiraSoapServiceExtension;
import com.atlassian.jira.rpc.soap.client.ext.JiraSoapServiceExtensionService;
import com.atlassian.jira.rpc.soap.client.ext.JiraSoapServiceExtensionServiceLocator;
import com.atlassian.jira_soapclient.SOAPClient;
import com.sun.mail.pop3.POP3SSLStore;
import com.sun.net.ssl.internal.ssl.SSLServerSocketFactoryImpl;

public class JiraWrapper implements BugTrackerWrapperInterface {

//	private SOAPClient jiraSoapClient;

	private JiraSoapServiceService jiraSoapServiceGetter;
	private JiraSoapService jiraSoapService;
	private String token;

	private JiraSoapServiceExtensionService jiraSoapServiceExtensionGetter;
	private JiraSoapServiceExtension jiraSoapServiceExtension;

	private String extensionToken;

	private RemoteStatus[] remoteStatuses;

	private RemotePriority[] remotePriorities;

	private RemoteResolution[] remoteResolutions;

	private RemoteIssueType[] remoteIssueTypes;


	private static String ENDPOINT_SUFFIX = "rpc/soap/jirasoapservice-v2";
	
	private static String ENDPOINT_SUFFIX_EXT = "rpc/soap/jirasoapserviceext-v1";

	private String endpointadress;

	public JiraWrapper(String endpointadress)
	{	
		this.endpointadress = endpointadress;
	}	

	@Override
	public Issue getIssue(String username, String password, String projectKey,
			String issueKey) throws BadRequestException, InternalServerErrorException {

		initSoapClient(username, password);

		String key = buildIssueKey(projectKey, issueKey);

		RemoteIssue remoteIssue;
		try {
			remoteIssue = jiraSoapService.getIssue(token, key);
			Issue issue = parseIssue(remoteIssue);
			return issue;

		} catch (RemoteAuthenticationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new UnAuthorizedException(e.getFaultString());

		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();

			if(e instanceof RemotePermissionException){

				//TODO: When to know if forbidden / not found??

				RemotePermissionException rpe = (RemotePermissionException)e;			
				throw new NotFoundException(rpe.getFaultString());
			}

//			throw new InternalServerErrorException(e.getMessage());
		}
		return null;		

	}

	@Override
	public Issue[] getIssues(String username, String password,
			String projectKey) throws BadRequestException, InternalServerErrorException {

		initSoapClient(username, password);

//		ArrayList<Issue> issues = new ArrayList<Issue>();
		Issue[] issues = null;

		try {

			String[] projectKeys = new String[1];
			projectKeys[0] = projectKey;   


//			RemoteFilter[] filters = jiraSoapService.getSavedFilters(token);

//			jiraSoapService.get

//			System.out.println("SavedFilters: ");

//			for (RemoteFilter remoteFilter : filters) {

//			System.out.println("Filter: " + remoteFilter.getId() + " : " + remoteFilter.getName());

//			}

//			System.out.println("Calling soap " + new Date().toString());
			long startupTime = System.currentTimeMillis();

			RemoteIssue[] remoteIssues = jiraSoapService.getIssuesFromTextSearchWithProject(token, projectKeys, null, 99999);

			long fetchedIssuesTime = System.currentTimeMillis();

//			System.out.println("Got " + remoteIssues.length + " from Soap " + new Date().toString());

			if(remoteIssues != null && remoteIssues.length > 0)
			{
				issues = new Issue[remoteIssues.length];

				for(int i = 0; i < remoteIssues.length; i++)
				{
					RemoteIssue remoteIssue = remoteIssues[i];				
					issues[i] = parseIssue(remoteIssue);
//					issues.add(issue);

				}
				long convertedIssuesTime = System.currentTimeMillis();

				System.out.println("Spent " + (fetchedIssuesTime - startupTime) + " fetching issues from SOAP");
				System.out.println("Spent " + (convertedIssuesTime - fetchedIssuesTime) + " converting to Issues");
				System.out.println("Time is: " + convertedIssuesTime);

				return issues;
			}	

		} catch (RemoteAuthenticationException e) {

			throw new UnAuthorizedException(e.getFaultString());

		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;

	}

	@Override
	public Issue addIssue(String username, String password, String projectKey,
			Issue issue) throws BadRequestException, InternalServerErrorException {
		// TODO Auto-generated method stub
		issue.setProjectkey(projectKey);

		initSoapClient(username, password);

		try {
			RemoteIssue remoteIssue = jiraSoapService.createIssue(token, buildIssue(issue));
			return parseIssue(remoteIssue);

		} catch (RemotePermissionException e) {
			// TODO Auto-generated catch block
			throw new ForbiddenException(e.getFaultString());
//			e.printStackTrace();
		} catch (RemoteValidationException e) {
			// TODO Auto-generated catch block
//			e.printStackTrace();
			throw new BadRequestException(e.getFaultString());
		} catch (RemoteAuthenticationException e) {
			// TODO Auto-generated catch block
			throw new UnAuthorizedException(e.getFaultString());
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
//			e.printStackTrace();
			throw new InternalServerErrorException(e.getMessage());
		}


	}

	/**
	 * Updates an issue
	 * @throws UnAuthorizedException 
	 * @throws BadRequestException 
	 * @throws InternalServerErrorException 
	 */
	@Override
	public Issue updateIssue(String username, String password,
			String projectKey, Issue issue) throws BadRequestException, InternalServerErrorException {
		// TODO Auto-generated method stub

		issue.setProjectkey(projectKey);

		initSoapClient(username, password);

		try {

			String key = buildIssueKey(issue.getProjectkey(), issue.getKey());

			RemoteIssue existingIssue = jiraSoapService.getIssue(token, key);

			RemoteIssue newIssue = buildIssue(issue);
			RemoteFieldValue[] updatedFields = getUpdateValues(existingIssue, newIssue);

			RemoteIssue updatedIssue = jiraSoapService.updateIssue(token, 
					buildIssueKey(issue.getProjectkey(), issue.getKey()), updatedFields);

			//Change status + resolution if necessary
			RemoteIssue fullyUpdatedIssue = updateWorkFlow(updatedIssue, newIssue);

			return parseIssue(fullyUpdatedIssue);

		} catch (RemotePermissionException e) {
			// TODO Auto-generated catch block
			throw new ForbiddenException(e.getFaultString());
		} catch (RemoteAuthenticationException e) {
			// TODO Auto-generated catch block
			throw new UnAuthorizedException(e.getFaultString());
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InternalServerErrorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new InternalServerErrorException(e.getMessage());
		}


		return null;
	}


	private RemoteIssue updateWorkFlow(RemoteIssue existingIssue, RemoteIssue newIssue) throws com.atlassian.jira.rpc.soap.client.RemoteException, RemoteException
	{

//		1 : Open
//		3 : In Progress
//		4 : Reopened
//		5 : Resolved
//		6 : Closed

		if(newIssue.getStatus() != null && (!newIssue.getStatus().equals(existingIssue.getStatus()) || (!newIssue.getResolution().equals(existingIssue.getResolution()))))
		{

			String actionId = null;
			RemoteFieldValue[] params = new RemoteFieldValue[0];

			//Fetch available solutions
			ArrayList<String> resolutionIds = new ArrayList<String>();
			RemoteResolution[] resolutions = jiraSoapService.getResolutions(token);
			for (RemoteResolution remoteResolution : resolutions) {
//				System.out.println("Resolution " + remoteResolution.getId() + ":" + remoteResolution.getName());
				resolutionIds.add(remoteResolution.getId());
			}

			RemoteNamedObject[] availableActions = jiraSoapService.getAvailableActions(token, existingIssue.getKey());

			//Fetch available actions
			ArrayList<String> availableActionList = new ArrayList<String>();
			ArrayList<String> actionFieldList = new ArrayList<String>();
			//System.out.println("###################################");
			for (RemoteNamedObject remoteNamedObject : availableActions) {
//				System.out.println(remoteNamedObject.getId() + ":" + remoteNamedObject.getName());
				availableActionList.add(remoteNamedObject.getId());


				RemoteField[] actionFields = jiraSoapService.getFieldsForAction(token, existingIssue.getKey(), remoteNamedObject.getId());
				for (RemoteField remoteField : actionFields) {
//					System.out.println("  - " + remoteField.getId() + ":" + remoteField.getName());
					actionFieldList.add(remoteField.getId());
				}			

			}

//			System.out.println(newIssue.getResolution() + " " + newIssue.getAssignee());
			//Todo check that user exists?
			if(newIssue.getResolution() != null && newIssue.getAssignee() != null && actionFieldList.contains("resolution") && actionFieldList.contains("assignee") && resolutionIds.contains(newIssue.getResolution())){
				params = new RemoteFieldValue[3];
				params[0] = buildRemoteField("resolution", newIssue.getResolution());

				if(newIssue.getFixVersions() != null)
				{
					params[1] = buildRemoteField("fixVersions", getFixVersionIds(newIssue));
				}
				else{
					params[1] = new RemoteFieldValue("fixVersions", new String[0]);
				}				
				params[2] = buildRemoteField("assignee", newIssue.getAssignee());
			}	

//			for(int i = 0; i < availableActionList.size(); i ++)
//			{
//			System.out.println(".. " + availableActionList.get(i) + " ..");
//			}

			//Close Issue
			if(newIssue.getStatus().equalsIgnoreCase("6") && availableActionList.contains("2"))
			{
				actionId = "2";					
			}
			//Start progress
			else if(newIssue.getStatus().equalsIgnoreCase("3") && availableActionList.contains("4")){
				actionId = "4";			
			}
			//Resolve Issue
			else if(newIssue.getStatus().equalsIgnoreCase("5") && availableActionList.contains("5")){
				actionId = "5";
			}

			//Execute the workflow change..
			if(actionId != null)
			{	
//				System.out.println("progressing workflow " + actionId + " " + params.length);
				jiraSoapService.progressWorkflowAction(token, existingIssue.getKey(), actionId, params);
			}			
		}		

		return jiraSoapService.getIssue(token, existingIssue.getKey());
	}

	/**
	 * Returns the ids of the fixVersions..
	 * @param remoteIssue
	 * @return
	 */
	private ArrayList<String> getFixVersionIds(RemoteIssue remoteIssue){

		ArrayList<String> fixVersionIds = new ArrayList<String>();

		RemoteVersion[] fixVersions = remoteIssue.getFixVersions();
		for (RemoteVersion remoteVersion : fixVersions) {
			fixVersionIds.add(remoteVersion.getId());
		}


		return fixVersionIds;

	}


	/**
	 * Returns an array of remotefields that have changed and could be updated
	 * @param remoteIssue
	 * @param issue
	 * @return
	 * @throws RemoteException 
	 * @throws com.atlassian.jira.rpc.soap.client.RemoteException 
	 */
	private RemoteFieldValue[] getUpdateValues(RemoteIssue existingIssue, RemoteIssue newIssue) throws com.atlassian.jira.rpc.soap.client.RemoteException, RemoteException
	{
		ArrayList<RemoteFieldValue> updatedValuesList = new ArrayList<RemoteFieldValue>();

		//Fetch the fields that could be edited
		RemoteField[] remoteFields = jiraSoapService.getFieldsForEdit(token, existingIssue.getKey());

		ArrayList<String> editableFields = new ArrayList<String>();

//		//System.out.println("###################################");
		for (RemoteField remoteField : remoteFields) {
			editableFields.add(remoteField.getId());			
//			//System.out.println(remoteField.getId() + ":" + remoteField.getName());
		}
//		//System.out.println("###################################");


		//summary
		if(editableFields.contains("summary") && newIssue.getSummary() != null && !existingIssue.getSummary().equals(newIssue.getSummary()))		
			updatedValuesList.add(buildRemoteField("summary", newIssue.getSummary()));

		//issuetype
		if(editableFields.contains("issuetype") && newIssue.getType() != null && existingIssue.getType() != newIssue.getType())
			updatedValuesList.add(buildRemoteField("issuetype", newIssue.getType()));


		//priority
		if(editableFields.contains("priority") && newIssue.getPriority() != null && existingIssue.getPriority() != newIssue.getPriority())
			updatedValuesList.add(buildRemoteField("priority", newIssue.getPriority()));


		//duedate
		if(editableFields.contains("duedate") && newIssue.getDuedate() != null && existingIssue.getDuedate() != newIssue.getDuedate())
			updatedValuesList.add(buildRemoteField("duedate", newIssue.getDuedate().getTime().toString()));

		//components
		if(editableFields.contains("components") && newIssue.getComponents() != null && existingIssue.getComponents() != newIssue.getComponents()){
			ArrayList<String> changedComponents = new ArrayList<String>();
			RemoteComponent[] remoteComponents = newIssue.getComponents();
			for (RemoteComponent remoteComponent : remoteComponents) {
				changedComponents.add(remoteComponent.getId());
			}
			updatedValuesList.add(buildRemoteField("components", changedComponents));
		}		

		//versions
		if(editableFields.contains("versions") && newIssue.getAffectsVersions() != null && existingIssue.getAffectsVersions() != newIssue.getAffectsVersions()){
			RemoteVersion[] remoteVersions = newIssue.getAffectsVersions();		
			ArrayList<String> changedVersions = new ArrayList<String>();		
			for (RemoteVersion remoteVersion : remoteVersions) {
				changedVersions.add(remoteVersion.getId());
			}
			updatedValuesList.add(buildRemoteField("versions", changedVersions));
		}

		//fixVersions
		if(editableFields.contains("fixVersions") && newIssue.getFixVersions() != null && existingIssue.getFixVersions() != newIssue.getFixVersions()){
			RemoteVersion[] remoteVersions = newIssue.getFixVersions();		
			ArrayList<String> changedVersions = new ArrayList<String>();		
			for (RemoteVersion remoteVersion : remoteVersions) {
				changedVersions.add(remoteVersion.getId());
			}
			updatedValuesList.add(buildRemoteField("fixVersions", changedVersions));
		}

		//assignee
		if(editableFields.contains("assignee") && newIssue.getAssignee() != null && existingIssue.getAssignee() != newIssue.getAssignee())
			updatedValuesList.add(buildRemoteField("assignee", newIssue.getAssignee()));

		//reporter
		if(editableFields.contains("reporter") && newIssue.getReporter() != null && existingIssue.getReporter() != newIssue.getReporter())
			updatedValuesList.add(buildRemoteField("reporter", newIssue.getReporter()));

		//environment
		if(editableFields.contains("environment") && newIssue.getEnvironment() != null && existingIssue.getEnvironment() != newIssue.getEnvironment())
			updatedValuesList.add(buildRemoteField("environement", newIssue.getEnvironment()));

		//description
		if(editableFields.contains("description") && newIssue.getDescription() != null && existingIssue.getDescription() != newIssue.getDescription())
			updatedValuesList.add(buildRemoteField("description", newIssue.getDescription()));

		//Convert to array
		RemoteFieldValue[] updatedValues = new RemoteFieldValue[updatedValuesList.size()];
		for(int i = 0; i < updatedValuesList.size(); i++){
			updatedValues[i] = updatedValuesList.get(i);
		}

		return updatedValues;
	}


	/**
	 * Instantiate a remote field
	 * @param id
	 * @param value
	 * @return
	 */
	private RemoteFieldValue buildRemoteField(String id, String value)
	{		
		String[] values = new String[1];
		values[0] = value;

		RemoteFieldValue remoteFieldValue = new RemoteFieldValue(id, values);

		return remoteFieldValue;
	}

	/**
	 * Instantiate a remote field
	 * @param id
	 * @param value
	 * @return
	 */
	private RemoteFieldValue buildRemoteField(String id, ArrayList<String> valueList)
	{		
		String[] values = new String[valueList.size()];	

		for(int i = 0; i < valueList.size(); i++){
			values[i] = valueList.get(i);
		}		

		RemoteFieldValue remoteFieldValue = new RemoteFieldValue(id, values);

		return remoteFieldValue;
	}


	/**
	 * Init the soap client
	 * @param username
	 * @param password
	 * @throws UnAuthorizedException 
	 */
	private void initSoapClient(String username, String password) throws UnAuthorizedException{

		try {

			URL url = new URL(endpointadress + ENDPOINT_SUFFIX);
			jiraSoapServiceGetter = new JiraSoapServiceServiceLocator(); 
			jiraSoapService = jiraSoapServiceGetter.getJirasoapserviceV2(url);
			token = jiraSoapService.login(username, password);

			URL extUrl = new URL(endpointadress + ENDPOINT_SUFFIX_EXT);
			jiraSoapServiceExtensionGetter = new JiraSoapServiceExtensionServiceLocator();
			jiraSoapServiceExtension = jiraSoapServiceExtensionGetter.getJirasoapserviceextV1(extUrl);
			extensionToken = jiraSoapServiceExtension.login(username, password);

			remoteStatuses = jiraSoapService.getStatuses(token);
			remotePriorities = jiraSoapService.getPriorities(token);			
			remoteIssueTypes = jiraSoapService.getIssueTypes(token);
			remoteResolutions = jiraSoapService.getResolutions(token);


		} catch (RemoteAuthenticationException e) {			
			throw new UnAuthorizedException(e.getFaultString());
			// TODO: handle exception			
		} catch (RemoteException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (ServiceException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			//System.out.println("Could not parse the endpointadress: " + endpointadress);
//			e.printStackTrace();
		}
	}

	private String getIssueKey(String issueKey)
	{			
		String regex = "\\d+";
		Pattern pattern = Pattern.compile(regex);

		Matcher matcher = pattern.matcher(issueKey);

		matcher.find();
		issueKey = matcher.group();
		return issueKey;
	}


	private String buildIssueKey(String projectKey, String issueKey)
	{
		String key = projectKey + "-" + issueKey;
		return key;
	}


	private RemoteIssue buildIssue(Issue issue) throws RemotePermissionException, RemoteAuthenticationException, com.atlassian.jira.rpc.soap.client.RemoteException, RemoteException
	{
		RemoteIssue remoteIssue = new RemoteIssue();


		if(issue.getKey() != null)
		{
			String key = buildIssueKey(issue.getProjectkey(), issue.getKey());
			remoteIssue.setKey(key);

			RemoteIssue existingIssue = jiraSoapService.getIssue(token, key);

			if(existingIssue != null)
				remoteIssue = existingIssue;
		}


		RemoteVersion[] affectsVersions = getVersions(issue);

		String[] attachmentNames = new String[0]; //Missing

		RemoteComponent[] remoteComponents = getComponents(issue);

		Calendar created = new GregorianCalendar();
		if(issue.getCreated() != null)
			created.setTime(issue.getCreated());

		RemoteCustomFieldValue[] customFieldValues = new RemoteCustomFieldValue[0]; //Missing

		if(issue.getDueDate() != null){
			Calendar duedate = new GregorianCalendar(); 
			duedate.setTime(issue.getDueDate());
			remoteIssue.setDuedate(duedate);
		}

		String environment = null; //Missing

		RemoteVersion[] fixVersions = new RemoteVersion[0]; //Missing

		String priority = getPriorityKey(issue);		



		String resolution = getResolutionKey(issue);
		if(resolution != null)
			remoteIssue.setResolution(resolution);

		String status = getStatusKey(issue);
		remoteIssue.setStatus(status);

		//System.out.println("Status returned " + status);

		String type = getTypeKey(issue);

		Calendar updated = new GregorianCalendar(); //Missing

		Long votes = new Long(0); //Missing

		remoteIssue.setAffectsVersions(affectsVersions);
		remoteIssue.setAssignee(issue.getAssigned_to());
//		remoteIssue.setAttachmentNames(attachmentNames);
		remoteIssue.setComponents(remoteComponents);
		remoteIssue.setCreated(created);
//		remoteIssue.setCustomFieldValues(customFieldValues);
		remoteIssue.setDescription(issue.getDescription());

//		remoteIssue.setEnvironment(environment);
//		remoteIssue.setFixVersions(fixVersions);
//		remoteIssue.setId(id);
		remoteIssue.setPriority(priority);
		remoteIssue.setProject(issue.getProjectkey());
		remoteIssue.setReporter(issue.getReported_by());
//		remoteIssue.setResolution(resolution); //Handled elsewhere..
//		remoteIssue.setStatus(status); //Handled elsewhere..
		remoteIssue.setSummary(issue.getSummary());
		remoteIssue.setType(type);
//		remoteIssue.setUpdated(updated); //Automatically?
//		remoteIssue.setVotes(votes);	

		//System.out.println("remoteStatus " + remoteIssue.getStatus());
		//System.out.println("remotePriority " + remoteIssue.getPriority());

		return remoteIssue;
	}

	/**
	 * Get the type key (instead of name)
	 * @param issue
	 * @return
	 * @throws RemotePermissionException
	 * @throws RemoteAuthenticationException
	 * @throws RemoteException
	 */
	private String getTypeKey(Issue issue) throws RemotePermissionException, RemoteAuthenticationException, RemoteException
	{
		String typeKey = null;

		String jiraType = issue.getType();		

		if(jiraType != null){			

			jiraType = JiraMappings.buildType(jiraType);

//			RemoteIssueType[] remoteTypes = jiraSoapService.getIssueTypes(token);	
			for (RemoteIssueType remoteIssueType : remoteIssueTypes) {
				if(remoteIssueType.getName().equalsIgnoreCase(jiraType))
				{
					return remoteIssueType.getId();
				}
			}
		}

		return typeKey;
	}

	/**
	 * Get the status key (instead of name)
	 * @param issue
	 * @return
	 * @throws RemotePermissionException
	 * @throws RemoteAuthenticationException
	 * @throws RemoteException
	 */
	private String getStatusKey(Issue issue) {

		String state = issue.getState();

		String status = null;

		if(state !=null)
			status = JiraMappings.buildStatus(state);	

//		System.out.println("Finding status key for state: " + state + " status: " + status);

		if(status != null){

			for (RemoteStatus remoteStatus : remoteStatuses) {
				if(remoteStatus.getName().equalsIgnoreCase(status)) {
					return remoteStatus.getId();
				}					
			}	
		}

		return null;
	}

	/**
	 * Get the resolution key
	 * @param issue
	 * @return
	 * @throws RemotePermissionException
	 * @throws RemoteAuthenticationException
	 * @throws RemoteException
	 */
	private String getResolutionKey(Issue issue) throws RemotePermissionException, RemoteAuthenticationException, RemoteException{

		String state = issue.getState();

		if(state != null){

			String resolution = JiraMappings.buildResolution(state);
			if(resolution == null)
				return null;

//			RemoteResolution[] remoteResolutions = jiraSoapService.getResolutions(token);
			for (RemoteResolution remoteResolution : remoteResolutions) {
				if(remoteResolution.getName().equalsIgnoreCase(resolution)){					
					return remoteResolution.getId();					
				}

			}

		}
		return null;
	}

	/**
	 * Get the priority key (instead of name)
	 * @param issue
	 * @return
	 * @throws RemotePermissionException
	 * @throws RemoteAuthenticationException
	 * @throws RemoteException
	 */
	private String getPriorityKey(Issue issue) throws RemotePermissionException, RemoteAuthenticationException, RemoteException
	{
		//System.out.println("issuePriority " + issue.getPriority());

		String priorityKey = null;

		String priorityName = issue.getPriority();

		if(priorityName != null){

			priorityName = JiraMappings.buildPriority(priorityName);

//			RemotePriority[] remotePriorities = jiraSoapService.getPriorities(token);			
			for (RemotePriority remotePriority : remotePriorities) {

				//System.out.println("remotePriority " + remotePriority.getId() + ":" + remotePriority.getName());

				if(remotePriority.getName().equalsIgnoreCase(priorityName)){
					return remotePriority.getId();
				}
			}
		}

		return priorityKey;
	}

	/**
	 * Returns an array of RemoteComponents confirmed for the supplied issue
	 * @param issue
	 * @return
	 * @throws RemotePermissionException
	 * @throws RemoteAuthenticationException
	 * @throws com.atlassian.jira.rpc.soap.client.RemoteException
	 * @throws RemoteException
	 */
	private RemoteComponent[] getComponents(Issue issue) throws RemotePermissionException, RemoteAuthenticationException, com.atlassian.jira.rpc.soap.client.RemoteException, RemoteException{

		RemoteComponent[] remoteComponents = new RemoteComponent[0];

		ArrayList<RemoteComponent> remoteComponentList = new ArrayList<RemoteComponent>();

		RemoteComponent[] projectComponents = jiraSoapService.getComponents(token, issue.getProjectkey());

		if(issue.getVersions() != null)
		{			
			for (RemoteComponent remoteComponent : projectComponents) {
				if(issue.getComponents().contains(remoteComponent.getName())){
					remoteComponentList.add(remoteComponent);
//					System.out.println(remoteComponent.getId() + ":" + remoteComponent.getName());
				}
			}
//			remoteComponents = (RemoteComponent[]) remoteComponentList.toArray();

			remoteComponents = new RemoteComponent[remoteComponentList.size()];

			for(int i = 0; i < remoteComponentList.size(); i++)
			{
				remoteComponents[i] = remoteComponentList.get(i);
			}

		}

		return remoteComponents;
	}

	/**
	 * Returns an array of the remoteversion confirmed for the issue
	 * @param issue
	 * @return
	 * @throws RemotePermissionException
	 * @throws RemoteAuthenticationException
	 * @throws com.atlassian.jira.rpc.soap.client.RemoteException
	 * @throws RemoteException
	 */
	private RemoteVersion[] getVersions(Issue issue) throws RemotePermissionException, RemoteAuthenticationException, com.atlassian.jira.rpc.soap.client.RemoteException, RemoteException{
		RemoteVersion[] remoteVersions = new RemoteVersion[0];

		ArrayList<RemoteVersion> remoteVersionList = new ArrayList<RemoteVersion>();

		RemoteVersion[] projectVersions = jiraSoapService.getVersions(token, issue.getProjectkey());

		if(issue.getVersions() != null)
		{
			for (RemoteVersion remoteVersion : projectVersions) {

				if(issue.getVersions().contains(remoteVersion.getName())){
					remoteVersionList.add(remoteVersion);					
				}
			}			
//			remoteVersions = (RemoteVersion[]) remoteVersionList.toArray();

			remoteVersions = new RemoteVersion[remoteVersionList.size()];

			for (int i = 0; i < remoteVersionList.size(); i++)
			{
				remoteVersions[i] = remoteVersionList.get(i);				
			}

		}

		return remoteVersions;
	}

	/**
	 * Returns an Issue based on the provided RemoteIssue
	 * @param key
	 * @param summary
	 * @param remoteIssue
	 * @return
	 * @throws RemoteException 
	 * @throws RemoteAuthenticationException 
	 * @throws RemotePermissionException 
	 * @throws BadRequestException 
	 * @throws InternalServerErrorException 
	 */
	private Issue parseIssue(RemoteIssue remoteIssue) throws RemotePermissionException, RemoteAuthenticationException, RemoteException, BadRequestException, InternalServerErrorException
	{		
		Issue issue = new Issue();

//		System.out.println("Parsing issue.."  + new Date().toString());

		//System.out.println("remotekey " + remoteIssue.getKey());

		issue.setKey(getIssueKey(remoteIssue.getKey()));

//		System.out.println("Got key "  + new Date().toString());

		issue.setSummary(remoteIssue.getSummary());

//		System.out.println("Set summary "  + new Date().toString());

		issue.setAssigned_to(remoteIssue.getAssignee());

//		System.out.println("Set assignee "  + new Date().toString());

		RemoteComponent[] components = remoteIssue.getComponents();
		for (RemoteComponent remoteComponent : components) {
			issue.getComponents().add(remoteComponent.getName());
		}

//		System.out.println("Set components "  + new Date().toString());

		issue.setProjectkey(remoteIssue.getProject());

//		System.out.println("Set projectkey "  + new Date().toString());

		//System.out.println("project: " + remoteIssue.getProject());

		issue.setCreated(remoteIssue.getCreated().getTime());

		if(remoteIssue.getDuedate() != null)
			issue.setDueDate(remoteIssue.getDuedate().getTime());

//		System.out.println("Set created "  + new Date().toString());

		issue.setDescription(remoteIssue.getDescription());

//		System.out.println("Set description "  + new Date().toString());

		String priority = getPriorityName(remoteIssue.getPriority());
		try {
			issue.setPriority(priority);
		} catch (Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
			throw new BadRequestException(e1.getMessage());
		}


//		if(priority.equalsIgnoreCase("trivial"))
//		priority = PriorityTypes.PRIORITY_TRIVIAL;
//		else if(priority.equalsIgnoreCase("major"))
//		priority = PriorityTypes.PRIORITY_MAJOR;
//		else if(priority.equalsIgnoreCase(anotherString))

		//System.out.println("priority.. " + priority);



//		System.out.println("Set priority "  + new Date().toString());

		//System.out.println("issue pr " + issue.getPriority());

		issue.setReported_by(remoteIssue.getReporter());
//		System.out.println("Set reported "  + new Date().toString());

		String status = getStatusName(remoteIssue.getStatus());
		String resolution = getResolutionName(remoteIssue.getResolution());	

		String state = JiraMappings.parseState(status, resolution);
		try {
			issue.setState(state);
		} catch (Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
			throw new InternalServerErrorException(e1.getMessage());
		}

		String type = getTypeName(remoteIssue.getType());
		try {
			issue.setType(type);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new BadRequestException(e.getMessage());
		}

		issue.setViewissueurl(endpointadress + "browse/" + remoteIssue.getKey());

//		System.out.println("Set type "  + new Date().toString());

		issue.setUpdated(remoteIssue.getUpdated().getTime());
//		System.out.println("updated: " + issue.getUpdated());

//		System.out.println("Set updated "  + new Date().toString());

		RemoteVersion[] versions = remoteIssue.getAffectsVersions();
		for (RemoteVersion remoteVersion : versions) {
			issue.getVersions().add(remoteVersion.getName());
		}




//		System.out.println("Set versions "  + new Date().toString());

		return issue;
	}

	private String getStatusName(String statusKey) throws RemotePermissionException, RemoteAuthenticationException, RemoteException
	{
		String statusName = null;

		for (RemoteStatus remoteStatus : remoteStatuses) {
			if(remoteStatus.getId().equalsIgnoreCase(statusKey))
				return remoteStatus.getName();
		}	

		return statusName;		
	}

	/**
	 * Gets the name of the type from Jira and then the correct mapping
	 * @param typeKey
	 * @return
	 * @throws RemotePermissionException
	 * @throws RemoteAuthenticationException
	 * @throws RemoteException
	 */
	private String getTypeName(String typeKey) throws RemotePermissionException, RemoteAuthenticationException, RemoteException{

		for (RemoteIssueType remoteIssueType : remoteIssueTypes) {
			if(remoteIssueType.getId().equalsIgnoreCase(typeKey)){

				String typeName = remoteIssueType.getName();
				typeName = JiraMappings.parseType(typeName);

				return typeName;
			}
		}

		return null;
	}

	private String getResolutionName(String resolutionKey) throws RemotePermissionException, RemoteAuthenticationException, RemoteException{

		for (RemoteResolution remoteResolution : remoteResolutions) {
			if(remoteResolution.getId().equalsIgnoreCase(resolutionKey)){

				String resolutionName = remoteResolution.getName();

				//TODO: Parse / map this..
//				resolutionName = JiraMappings.parseType(typeName);

				return resolutionName;
			}
		}

		return null;
	}

	/**
	 * Gets the name of the priority from Jira and then the correct mapping
	 * @param priorityKey
	 * @return
	 * @throws RemotePermissionException
	 * @throws RemoteAuthenticationException
	 * @throws RemoteException
	 */
	private String getPriorityName (String priorityKey) throws RemotePermissionException, RemoteAuthenticationException, RemoteException{

		for (RemotePriority remotePriority : remotePriorities) {
			if(remotePriority.getId().equalsIgnoreCase(priorityKey)){

				String priorityName = remotePriority.getName();
				priorityName = JiraMappings.parsePriority(priorityName);

				return priorityName;

			}

		}

		//Undefined or == null..
		return PriorityTypes.PRIORITY;
	}


	@Override
	public void deleteIssue(String username, String password,
			String projectKey, String issueKey) throws BadRequestException {
		// TODO Auto-generated method stub

		initSoapClient(username, password);		

		try {
			jiraSoapService.deleteIssue(token, buildIssueKey(projectKey, issueKey));
		} catch (RemotePermissionException e) {
			// TODO Auto-generated catch block
			throw new ForbiddenException(e.getFaultString());
		} catch (RemoteAuthenticationException e) {
			// TODO Auto-generated catch block
			throw new UnAuthorizedException(e.getFaultString());
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@Override
	public void addComment(String username, String password, String projectKey,
			String issueKey, Comment comment) throws BadRequestException, InternalServerErrorException {
		// TODO Auto-generated method stub

		try {
			initSoapClient(username, password);

			String key = buildIssueKey(projectKey, issueKey);

			RemoteComment remoteComment = new RemoteComment();
			remoteComment.setAuthor(comment.getAuthor());

			Calendar created = new GregorianCalendar();
			if(comment.getCreated() != null){
				created.setTime(comment.getCreated());
			}
			remoteComment.setCreated(created);

			remoteComment.setBody(comment.getContent());


			jiraSoapService.addComment(token, key, remoteComment);

		} catch (UnAuthorizedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new UnAuthorizedException(e.getMessage());
		} catch (RemotePermissionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new UnAuthorizedException(e.getMessage());
		} catch (RemoteAuthenticationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new UnAuthorizedException(e.getMessage());
		} catch (com.atlassian.jira.rpc.soap.client.RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();


		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new InternalServerErrorException(e.getMessage());
		}


	}

	@Override
	public Comment[] getComments(String username, String password,
			String projectKey, String issueKey) throws BadRequestException, InternalServerErrorException {
		try {

			ArrayList<Comment> comments = new ArrayList<Comment>();

			initSoapClient(username, password);
			String key = buildIssueKey(projectKey, issueKey);	


			RemoteComment[] remoteComments = jiraSoapService.getComments(token, key);

			for(int i=0; i < remoteComments.length; i++){
				RemoteComment remoteComment = remoteComments[i];

				String author = remoteComment.getAuthor();
				String content = remoteComment.getBody();

				Comment comment = null;

				Calendar createdCal = remoteComment.getCreated();
				if(createdCal != null){				
					Date created = remoteComment.getCreated().getTime();
					comment = new Comment(author, content, created);		
				} else {
					comment = new Comment(author, content);
				}

				comments.add(comment);				
			}

			Comment[] commentsArray = new Comment[comments.size()];
			for(int i = 0; i < comments.size(); i++){
				commentsArray[i] = comments.get(i);
			}
			return commentsArray;

		} catch (UnAuthorizedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new UnAuthorizedException(e.getMessage());
		} catch (RemotePermissionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new ForbiddenException(e.getMessage());
		} catch (RemoteAuthenticationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new ForbiddenException(e.getMessage());
		} catch (com.atlassian.jira.rpc.soap.client.RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new InternalServerErrorException(e.getFaultString());
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new InternalServerErrorException(e.getMessage());
		}		


	}

	@Override
	public String[] getComponentNames(String username, String password,
			String projectKey) throws BadRequestException, InternalServerErrorException {
		// TODO Auto-generated method stub
		try {
			initSoapClient(username, password);
			RemoteComponent[] components = jiraSoapService.getComponents(token, projectKey);
			String[] componentNames = new String[components.length];
			for(int i=0; i < components.length; i++){
				componentNames[i] = components[i].getName();
			}
			return componentNames;

		} catch (UnAuthorizedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new UnAuthorizedException(e.getMessage());
		} catch (RemotePermissionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new ForbiddenException(e.getMessage());
		} catch (RemoteAuthenticationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new ForbiddenException(e.getMessage());
		} catch (com.atlassian.jira.rpc.soap.client.RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new InternalServerErrorException(e.getFaultString());
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new InternalServerErrorException(e.getMessage());
		}
	}

	@Override
	public String[] getVersions(String username, String password,
			String projectKey) throws BadRequestException, InternalServerErrorException {
		// TODO Auto-generated method stub
		try {
			initSoapClient(username, password);
			RemoteVersion[] versions = jiraSoapService.getVersions(token, projectKey);
			String[] versionNames = new String[versions.length];
			for(int i=0; i < versions.length; i++){
				versionNames[i] = versions[i].getName();				
			}
			return versionNames;

		} catch (UnAuthorizedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new UnAuthorizedException(e.getMessage());
		} catch (RemotePermissionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new ForbiddenException(e.getMessage());
		} catch (RemoteAuthenticationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new ForbiddenException(e.getMessage());
		} catch (com.atlassian.jira.rpc.soap.client.RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new InternalServerErrorException(e.getFaultString());
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new InternalServerErrorException(e.getMessage());
		}
	}

	@Override
	public String[] getIssueTypes(String username, String password,
			String projectKey) {
		// TODO Auto-generated method stub
		String[] issueTypes = (String[]) IssueTypes.getInstance().getArray();
		return issueTypes;
	}

	@Override
	public String[] getPriorities(String username, String password,
			String projectKey) {
		// TODO Auto-generated method stub
		String[] priorities = (String[]) PriorityTypes.getInstance().getArray();
		return priorities;
	}

	@Override
	public String[] getStates(String username, String password,
			String projectKey, String issueKey) {
		// TODO Auto-generated method stub
		try {			
			//TODO: Do something to find out which actions are really available..!!
			initSoapClient(username, password);			
//			jiraSoapService.getAvailableActions(arg0, arg1)

			String[] states = (String[]) IssueStates.getInstance().getArray();
			return states;

		} catch (UnAuthorizedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * Add an attachement
	 * @param username
	 * @param password
	 * @param projectKey
	 * @param issueKey
	 */
	private void addAttachement(String username, String password, String projectKey, String issueKey, String fileName){

		try {

			initSoapClient(username, password);
			String key = buildIssueKey(projectKey, issueKey);	
			String[] fileNames = new String[0];
			byte[][] attachements = new byte[0][0];

			jiraSoapService.addAttachmentsToIssue(token, key, fileNames, attachements);

		} catch (UnAuthorizedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemotePermissionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemoteValidationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemoteAuthenticationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (com.atlassian.jira.rpc.soap.client.RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	/**
	 * Get the attachements belonging to an issue..
	 * @param username
	 * @param password
	 * @param projectKey
	 * @param issueKey
	 */
	private void getAttachements(String username, String password, String projectKey, String issueKey){

		try {
			initSoapClient(username, password);

			String key = buildIssueKey(projectKey, issueKey);			
			RemoteAttachment[] attachements = jiraSoapService.getAttachmentsFromIssue(token, key);
			for (RemoteAttachment remoteAttachment : attachements) {

				remoteAttachment.getAuthor();
				remoteAttachment.getCreated();
				remoteAttachment.getFilename();
				remoteAttachment.getFilesize();
				remoteAttachment.getId();
				remoteAttachment.getMimetype();


			}



		} catch (UnAuthorizedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemotePermissionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemoteValidationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemoteAuthenticationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (com.atlassian.jira.rpc.soap.client.RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}


	public ArrayList<String> getUpdatedIssueKeysFromEmail(String username, String password,
			String projectKey) {
		// TODO Auto-generated method stub

		ArrayList<String> keys = new ArrayList<String>();	

		String SSL_FACTORY = "javax.net.ssl.SSLSocketFactory";

		Properties pop3Props = new Properties();

		pop3Props.setProperty("mail.pop3.socketFactory.class", SSL_FACTORY);
		pop3Props.setProperty("mail.pop3.socketFactory.fallback", "false");
		pop3Props.setProperty("mail.pop3.port",  "995");
		pop3Props.setProperty("mail.pop3.socketFactory.port", "995");

		URLName url = new URLName("pop3", "pop.gmail.com", 995, "",
				"milesjiraupdates", "workaround");

		Session session = Session.getInstance(pop3Props, null);
		Store store = new POP3SSLStore(session, url);
		try {

			System.out.println("Connecting..");
			store.connect();

			System.out.println("Connected");

			Folder folder = store.getDefaultFolder();

			folder = folder.getFolder("INBOX");

			folder.open(Folder.READ_WRITE);

			Message[] messages = folder.getMessages();

			Pattern pattern = Pattern.compile("\\(" + projectKey + "-(\\d+)\\)");

			ArrayList<Message> usedMessages = new ArrayList<Message>();

			for (Message message : messages) {

				String subject = message.getSubject();					

//				System.out.println(subject);

				Matcher matcher = pattern.matcher(subject);

				while(matcher.find()){

					String match = matcher.group(1);

					keys.add(match);

					usedMessages.add(message);

					System.out.println(match);

				}


			}

			Message[] deleteMessages = new Message[usedMessages.size()];

			for(int i = 0; i< usedMessages.size(); i++){

				deleteMessages[i] = usedMessages.get(i);

			}					

			Flags flags = new Flags(Flag.DELETED);

			folder.setFlags(deleteMessages, flags, true);				

			folder.close(true);

			store.close();



		} catch (MessagingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return keys;

	}

	@Override
	public ArrayList<String> getUpdatedIssueKeys(String username,
			String password, String projectKey, Date date) throws UnAuthorizedException, InternalServerErrorException {
		// TODO Auto-generated method stub

		ArrayList<String> updatedIssueKeys = new ArrayList<String>();

		try {


			initSoapClient(username, password);


			String[] projectKeys = new String[1];
			projectKeys[0] = projectKey;   

			Calendar cal = new GregorianCalendar();
			
			if(date != null){			
				cal.setTime(date);
			} else {
				cal = null;
			}

			String[] issueKeys = jiraSoapServiceExtension.getUpdatedIssueKeysWithProject(extensionToken, projectKeys, cal);

			for (String issueKey : issueKeys) {

				String parsedKey = getIssueKey(issueKey);

				updatedIssueKeys.add(parsedKey);

			}


		} catch (UnAuthorizedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new UnAuthorizedException(e.getMessage());
		} catch (com.atlassian.jira.rpc.soap.client.ext.RemotePermissionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new UnAuthorizedException(e.getMessage());
		} catch (com.atlassian.jira.rpc.soap.client.ext.RemoteAuthenticationException e) {
			// TODO Auto-generated catch block

			e.printStackTrace();
			throw new UnAuthorizedException(e.getMessage());
		} catch (com.atlassian.jira.rpc.soap.client.ext.RemoteException e) {
			// TODO Auto-generated catch block

			e.printStackTrace();
			throw new InternalServerErrorException(e.getMessage());
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new InternalServerErrorException(e.getMessage());
		}

		return updatedIssueKeys;
	}

	@Override
	public ArrayList<String> getIssueKeys(String username, String password,
			String projectKey) throws UnAuthorizedException,
			InternalServerErrorException {

		ArrayList<String> allIssueKeys = new ArrayList<String>();

		try {


			initSoapClient(username, password);


			String[] projectKeys = new String[1];
			projectKeys[0] = projectKey;   

			String[] issueKeys = jiraSoapServiceExtension.getIssueKeysWithProject(extensionToken, projectKeys);

			for (String issueKey : issueKeys) {

				String parsedKey = getIssueKey(issueKey);

				allIssueKeys.add(parsedKey);

			}


		} catch (UnAuthorizedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new UnAuthorizedException(e.getMessage());
		} catch (com.atlassian.jira.rpc.soap.client.ext.RemotePermissionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new UnAuthorizedException(e.getMessage());
		} catch (com.atlassian.jira.rpc.soap.client.ext.RemoteAuthenticationException e) {
			// TODO Auto-generated catch block

			e.printStackTrace();
			throw new UnAuthorizedException(e.getMessage());
		} catch (com.atlassian.jira.rpc.soap.client.ext.RemoteException e) {
			// TODO Auto-generated catch block

			e.printStackTrace();
			throw new InternalServerErrorException(e.getMessage());
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new InternalServerErrorException(e.getMessage());
		}

		return allIssueKeys;

	}





}
