package no.miles.mpl.wrapper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import no.miles.mpl.wrapper.datastructures.IssueStates;
import no.miles.mpl.wrapper.datastructures.IssueTypes;
import no.miles.mpl.wrapper.datastructures.PriorityTypes;

public class TracMappings {

	private static TracMappings instance = null;

	private static Map<String, String> tracToMplMap;
	private static Map<String, String> mplToTracMap;

	private static Map<String, String> tracToMplPriorityMap;
	private static Map<String, String> mplToTracPriorityMap;

	private static ArrayList<String> newStates;
	private static ArrayList<String> closedStates;

	private static Map<String, String> tracResolutionToMplStateMap;
	private static Map<String, String> mplStateToTracResolutionMap;

	private static Map<String, String> tracStatusToMplStateMap;	

	static {

		mplStateToTracResolutionMap = new HashMap<String, String>();
		mplStateToTracResolutionMap.put(IssueStates.STATE_FIXED, "fixed");
		mplStateToTracResolutionMap.put(IssueStates.STATE_WONT_FIX, "wontfix");
		mplStateToTracResolutionMap.put(IssueStates.STATE_WORKS_FOR_ME, "worksforme");
		mplStateToTracResolutionMap.put(IssueStates.STATE_DUPLICATE, "duplicate");
		mplStateToTracResolutionMap.put(IssueStates.STATE_INCOMPLETE, "invalid");

		newStates = new ArrayList<String>();
		newStates.add(IssueStates.STATE_OPEN);
		newStates.add(IssueStates.STATE_NEW);
		newStates.add(IssueStates.STATE_UNCONFIRMED);

		tracResolutionToMplStateMap = new HashMap<String, String>();
		tracResolutionToMplStateMap.put("fixed", IssueStates.STATE_FIXED);
		tracResolutionToMplStateMap.put("wontfix", IssueStates.STATE_WONT_FIX);
		tracResolutionToMplStateMap.put("duplicate", IssueStates.STATE_DUPLICATE);
		tracResolutionToMplStateMap.put("invalid", IssueStates.STATE_INCOMPLETE);
		tracResolutionToMplStateMap.put("worksforme", IssueStates.STATE_WORKS_FOR_ME);

		tracStatusToMplStateMap = new HashMap<String, String>();
		tracStatusToMplStateMap.put("new", IssueStates.STATE_NEW);
		//I'm a bit unsure of this:
		tracStatusToMplStateMap.put("closed", IssueStates.STATE_RESOLVED);
		tracStatusToMplStateMap.put("assigned", IssueStates.STATE_STARTED);
		tracStatusToMplStateMap.put("reopened", IssueStates.STATE_REOPENED);

		closedStates = new ArrayList<String>();
		closedStates.add(IssueStates.STATE_RESOLVED);
		closedStates.add(IssueStates.STATE_FIXED);
		closedStates.add(IssueStates.STATE_DUPLICATE);
		closedStates.add(IssueStates.STATE_INCOMPLETE);
		closedStates.add(IssueStates.STATE_LATER);
		closedStates.add(IssueStates.STATE_NOT_REPRODUCIBLE);
		closedStates.add(IssueStates.STATE_REMIND);
		closedStates.add(IssueStates.STATE_WONT_FIX);
		closedStates.add(IssueStates.STATE_WORKS_FOR_ME);
		closedStates.add(IssueStates.STATE_CLOSED);
		closedStates.add(IssueStates.STATE_TERMINAL_STATE);

		tracToMplMap = new HashMap<String, String>();
		tracToMplMap.put("defect", IssueTypes.ISSUE_TYPE_DEFECT);
		tracToMplMap.put("enhancement", IssueTypes.ISSUE_TYPE_ENHANCEMENT);
		tracToMplMap.put("task", IssueTypes.ISSUE_TYPE_TASK);		

		mplToTracMap = new HashMap<String, String>();
		mplToTracMap.put(IssueTypes.ISSUE_TYPE_DEFECT, "defect");
		mplToTracMap.put(IssueTypes.ISSUE_TYPE_ENHANCEMENT, "enhancement");
		mplToTracMap.put(IssueTypes.ISSUE_TYPE_TASK, "task");

		tracToMplPriorityMap = new HashMap<String, String>();
		tracToMplPriorityMap.put("critical", PriorityTypes.PRIORITY_CRITICAL);
		tracToMplPriorityMap.put("blocker", PriorityTypes.PRIORITY_BLOCKER);
		tracToMplPriorityMap.put("major", PriorityTypes.PRIORITY_MAJOR);
		tracToMplPriorityMap.put("minor", PriorityTypes.PRIORITY_MINOR);
		tracToMplPriorityMap.put("trivial", PriorityTypes.PRIORITY_TRIVIAL);

		mplToTracPriorityMap = new HashMap<String, String>();
		mplToTracPriorityMap.put(PriorityTypes.PRIORITY_CRITICAL, "critical");
		mplToTracPriorityMap.put(PriorityTypes.PRIORITY_BLOCKER, "blocker");
		mplToTracPriorityMap.put(PriorityTypes.PRIORITY_MAJOR, "major");
		mplToTracPriorityMap.put(PriorityTypes.PRIORITY_MINOR, "minor");
		mplToTracPriorityMap.put(PriorityTypes.PRIORITY_TRIVIAL, "trivial");

	}

	/**
	 * Returns the best matching state from the status + resolution
	 * @param status
	 * @param resolution
	 * @return
	 */
	public static String parseState(String status, String resolution){

		String state = null;

		//Make sure other Statuses with resolution are not mapped to subclasses of Resolved..
		if(status.equalsIgnoreCase("closed")){
			state = tracResolutionToMplStateMap.get(resolution);
		}

		//If no resolution is found, try to map the status
		if(state == null){
			state = tracStatusToMplStateMap.get(status);
		}

		//If not - make it the most general state
		if(state == null)
			state = IssueStates.STATE_STATE;

		return  state;

	}

	public static String buildStatus(String state){

		if(state != null){

			if(newStates.contains(state)){
				return "new";
			} else if(state.equalsIgnoreCase(IssueStates.STATE_STARTED)){
				return "assigned";
			} else if(state.equalsIgnoreCase(IssueStates.STATE_REOPENED)){
				return "reopened";
			} else if(closedStates.contains(state)){
				return "closed";
			} 	
		}

		//Guess that the status = open..
		return "new";		
	}

	public static String buildResolution(String state){
		
		//This should be trivial if the correct mappings are made in the Map
		String resolution = mplStateToTracResolutionMap.get(state);

		return resolution;
	}

	public static String parsePriority(String tracPriority){

		String mplPriority = tracToMplPriorityMap.get(tracPriority);
		if(mplPriority != null)
			return mplPriority;

		return  PriorityTypes.PRIORITY;

	}

	public static String buildPriority(String mplPriority){

		String tracPriority = mplToTracPriorityMap.get(mplPriority);
		if(tracPriority != null)
			return tracPriority;

		//What to return if no defined mapping? i.e. baetle:Priority - gambling on major :D
		return "major";
	}

	public static String parseType(String tracType){

		String mplType = tracToMplMap.get(tracType);
		if(mplType != null)
			return mplType;

		return IssueTypes.ISSUE_TYPE_ISSUE;

	}

	public static String buildType(String mplType){

		String tracType = mplToTracMap.get(mplType);
		if(tracType != null)
			return tracType;

		//What to return if no defined mapping? i.e. baetle:Issue - gambling on task :D
		return "task";

	}

}
