package no.miles.mpl.wrapper;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.apache.log4j.helpers.ISO8601DateFormat;
import org.apache.xmlrpc.XmlRpcException;
import org.apache.xmlrpc.client.XmlRpcClientException;
import org.apache.xmlrpc.common.XmlRpcNotAuthorizedException;


import sun.reflect.generics.reflectiveObjects.NotImplementedException;

import no.miles.mpl.tracapi.DynamicProxy;
import no.miles.mpl.tracapi.Ticket;
import no.miles.mpl.tracapi.TracApi;
import no.miles.mpl.tracapi.TracSystem;
import no.miles.mpl.wrapper.datastructures.Comment;
import no.miles.mpl.wrapper.datastructures.Issue;
import no.miles.mpl.wrapper.datastructures.IssueStates;
import no.miles.mpl.wrapper.datastructures.IssueTypes;
import no.miles.mpl.wrapper.datastructures.PriorityTypes;
import no.miles.mpl.wrapper.exception.BadRequestException;
import no.miles.mpl.wrapper.exception.InternalServerErrorException;
import no.miles.mpl.wrapper.exception.UnAuthorizedException;

public class TracWrapper implements BugTrackerWrapperInterface {

	private static String XML_SUFFIX = "login/xmlrpc";
//	private Properties projects;
	private String projectRoot;

	public TracWrapper(String projectRoot)
	{
		this.projectRoot = projectRoot;
	}

	private DynamicProxy connect(String username, String password, String projectKey) {

		//System.out.println("connect " + projectKey);
		URL projectUrl = getProjectUrl(projectKey);		
		TracApi tracApi = new TracApi();
		DynamicProxy proxy = tracApi.connect(projectUrl, username, password);
		return proxy;
	}

	/**
	 * Get the correct project URL
	 * @param projectKey
	 * @return
	 */
	private URL getProjectUrl(String projectKey){

		URL projectURL;

		String sep = projectRoot.endsWith( "/" ) ? "" : "/";	

		try {
			projectURL = new URL(projectRoot + sep + XML_SUFFIX);
			return projectURL;
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}


	}

	@Override
	public Issue getIssue(String username, String password, String projectKey, String issueKey) throws BadRequestException, InternalServerErrorException{

//		System.out.println("getIssue " + projectKey);

		DynamicProxy proxy = connect(username, password, projectKey);

		Ticket ticket = (Ticket) proxy.newInstance(Ticket.class);


		int intKey = Integer.parseInt(issueKey);

		try{
			//Get ticket array
			Object[] ticketObjectList = ticket.get(intKey);

			return parseObjectList(ticketObjectList);
		}
		catch (XmlRpcClientException e) {
			// TODO: handle exception
			throw new BadRequestException(e.getMessage());
		}
		catch (XmlRpcException e){			
			return null;
		}		
		catch (Exception e) {
			e.printStackTrace();
			// TODO: handle exception
			throw new InternalServerErrorException(e.getMessage());
		}
	}





	@Override
	public Issue[] getIssues(String username, String password, String projectKey) throws BadRequestException, InternalServerErrorException {

		System.out.println("Get issues from Trac..");

//		System.out.println("getIssue " + projectKey);
		DynamicProxy proxy = connect(username, password, projectKey);
		Ticket ticket = (Ticket) proxy.newInstance( Ticket.class );

		TracSystem tracSystem = (TracSystem) proxy.newInstance( TracSystem.class );

		System.out.println("Success??");

		//TODO: change query!!

		long startTime = System.currentTimeMillis();
		long fetchedIssuesTime = 0;
		long parsedIssuesTime = 0;

		try{

//			System.out.println("Fetching ticket index.." + new Date().toString());
			Object[] ticketList = (Object[]) ticket.query("status=closed|new|assigned|reopened");
//			System.out.println("Fetched ticket index of (" + ticketList.length + " tickets)" + new Date().toString());


			Issue[] issues = null;

			if(ticketList != null && ticketList.length > 0)
			{
//				ArrayList<Issue> issues = new ArrayList<Issue>();
//				issues = new Issue[ticketList.length];

				Object[] signatures = new Object[ticketList.length];

				System.out.println("Fetching one by one.." + new Date().toString());
				for(int i = 0; i < ticketList.length; i++)
				{				
					HashMap<String, Object> signature = new HashMap<String, Object>();
					String issueKey = ticketList[i].toString();				
					Integer ticketKey = Integer.parseInt(issueKey);					

					Object[] params = new Object[1];
					params[0] = ticketKey;	

					signature.put("methodName", "ticket.get");
					signature.put("params", params);

					signatures[i] = signature;

//					Object[] ticketObjectList = ticket.get(ticketKey);
//					issues[i] = parseObjectList(issueKey, ticketObjectList);

//					issues[i] = getIssue(username, password, projectKey, ticketKey);
//					issues.add(issue);				
				}
//				Object object


				Object[] results = tracSystem.multicall(signatures);
//				System.out.println("Results: " + results.length);

				fetchedIssuesTime = System.currentTimeMillis();

				issues = new Issue[results.length];

				int i = 0;

				for (Object object : results) {

					Object[] resultArray = (Object[]) object;

					Object[] resultArray2 = (Object[]) resultArray[0];

					Issue issue = parseObjectList(resultArray2);
					issues[i] = issue;
					i++;

//					System.out.println("Issue: " + issue.getKey() + " " + issue.getSummary());

//					System.out.println(object.toString());
				}

				parsedIssuesTime = System.currentTimeMillis();





				System.out.println("Spent " + (fetchedIssuesTime - startTime) + " millis fetching issues. Time is " + fetchedIssuesTime);

				System.out.println("Spent " + (parsedIssuesTime - fetchedIssuesTime) + " converting to Issues");

//				System.out.println("Fetced one by one.." + new Date().toString());

				return issues;
			}

			else
			{
				return null;
			}


		} catch (XmlRpcClientException e) {
			// TODO: handle exception
			throw new BadRequestException(e.getMessage());
		}

		catch (Exception e) {
			throw new InternalServerErrorException(e.getMessage());
			// TODO: handle exception
		}
	}

	@Override
	public Issue addIssue(String username, String password, String projectKey,
			Issue issue) throws BadRequestException, InternalServerErrorException {
		// TODO Auto-generated method stub
		DynamicProxy proxy = connect(username, password, projectKey);
		Ticket ticket = (Ticket) proxy.newInstance( Ticket.class );

		String summary = issue.getSummary();
		String description = issue.getDescription();				


		Integer issueKey;
		try {
			issueKey = ticket.create(summary == null ? "" : summary, description == null ? "" : description, getAttributeStruct(issue)); 
			return getIssue(username, password, projectKey, issueKey.toString());		
		} catch (XmlRpcClientException e) {
			// TODO: handle exception
			e.printStackTrace();
			throw new BadRequestException(e.getMessage());
		} 		
		catch (Exception e) {			
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new InternalServerErrorException(e.getMessage());
		}


	}

	@Override
	public Issue updateIssue(String username, String password,
			String projectKey, Issue issue) throws BadRequestException, InternalServerErrorException {
		// TODO Auto-generated method stub

		DynamicProxy proxy = connect(username, password, projectKey);
		Ticket ticket = (Ticket) proxy.newInstance( Ticket.class );

		//TODO add a comment
		String comment = "Updated through remote API";

		try{
			Object[] ticketObjectList = ticket.update(Integer.parseInt(issue.getKey()), comment, getAttributeStruct(issue));

			return parseObjectList(ticketObjectList);

		} catch (XmlRpcClientException e) {
			// TODO: handle exception
			throw new BadRequestException(e.getMessage());
		} catch (NumberFormatException e) {
			// TODO Auto-generated catch block
			throw new BadRequestException(e.getMessage());
		} catch (Exception e) {
			throw new InternalServerErrorException(e.getMessage());
			// TODO Auto-generated catch block
		}
	}

	/**
	 * Creates a map of the issues attributes
	 * @param issue
	 * @return
	 */
	private Map<String, String> getAttributeStruct (Issue issue) {

		Map<String, String> ticketStruct = new HashMap<String, String>();		

		ticketStruct.put("summary", issue.getSummary() == null ? "" : issue.getSummary());

		String state = issue.getState();

		String status = TracMappings.buildStatus(state);
		ticketStruct.put("status", status == null ? "" : status);

		String resolution = TracMappings.buildResolution(state);
		if(resolution != null){
			ticketStruct.put("resolution", resolution);
		}		

		//Convert the type..
		if(issue.getType() != null){			
			String type = issue.getType();
			type = TracMappings.buildType(type);
			ticketStruct.put("type", type);
		}

		String priority = issue.getPriority();
		priority = TracMappings.buildPriority(priority);

		ticketStruct.put("priority", priority == null ? "" : priority);

		String componentName = "";

		if(issue.getComponents() != null && issue.getComponents().size() > 0)
		{
			componentName = issue.getComponents().get(0);
			ticketStruct.put("component", componentName == null ? "" : componentName);
		}



		String version = "";

		if(issue.getVersions() != null && issue.getVersions().size() > 0)
		{
			version = issue.getVersions().get(0);
			ticketStruct.put("version", version == null ? "" : version);
		}		



		ticketStruct.put("owner", issue.getAssigned_to() == null ? "" : issue.getAssigned_to());
		ticketStruct.put("reporter", issue.getReported_by() == null ? "" : issue.getReported_by());		
		ticketStruct.put("description", issue.getDescription() == null ? "" : issue.getDescription());

		return ticketStruct;
	}

	/**
	 * Parse an objectlist to an issue
	 * @param issueKey
	 * @param ticketObjectList
	 * @return
	 * @throws BadRequestException 
	 */
	private Issue parseObjectList(Object[] ticketObjectList) throws BadRequestException{

		Issue issue = null;

		if(ticketObjectList != null && ticketObjectList.length > 0)
		{
			Integer id = (Integer) ticketObjectList[0];

			//Third element = struct{...}
			HashMap<String, String> ticketStruct = (HashMap<String, String>) ticketObjectList[3];				

//			Set<String> keys = ticketStruct.keySet();
//			for (String string : keys) {
//			System.out.println("key: " + string);
//			System.out.println("value: " + ticketStruct.get(string));
//			}			

			String summary = ticketStruct.get("summary");
			issue = new Issue(id.toString(), summary);

			try{

//				Object obj = ticketObjectList[1];
//				System.out.println(obj.toString());

//				Integer epoc = (Integer) ticketObjectList[1];
//				Date created = new Date(epoc);
			
				DateFormat dateFormat = new ISO8601DateFormat();
				
				Date created = dateFormat.parse(ticketObjectList[1].toString());
				
				issue.setCreated(created);	
				
				

//				System.out.println(System.currentTimeMillis());

//				System.out.println("Created " + epoc.longValue() + " " + created.toString());

//				Integer modified = (Integer) ticketObjectList[2];
//				Date updated = new Date(modified.longValue());
				
				

				
				Date updated = dateFormat.parse(ticketObjectList[2].toString());
				
		
//
////				System.out.println("Modified " + modified.longValue() + " " + updated.toString());
//
				issue.setUpdated(updated);

			}
			catch(Exception e)
			{
				e.printStackTrace();
			}

			String status = ticketStruct.get("status");
			String resolution = ticketStruct.get("resolution");

			String state = TracMappings.parseState(status, resolution);

			try {
				issue.setState(state);
			} catch (Exception e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
				throw new BadRequestException(e1.getMessage());
			}

			//Convert the type
			String type = ticketStruct.get("type");
			type = TracMappings.parseType(type);			
			try {
				issue.setType(type);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new BadRequestException(e.getMessage());
			}

			String priority = ticketStruct.get("priority");
			priority = TracMappings.parsePriority(priority);			
			try {
				issue.setPriority(priority);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new BadRequestException(e.getMessage());
			}			

			issue.getComponents().add(ticketStruct.get("component"));
			issue.getVersions().add(ticketStruct.get("version"));
			issue.setAssigned_to(ticketStruct.get("owner"));
			issue.setReported_by(ticketStruct.get("reporter"));
			issue.setDescription(ticketStruct.get("description"));

			issue.setViewissueurl(projectRoot + "ticket/" + id.toString());

		}
		return issue;
	}

	@Override
	public void deleteIssue(String username, String password,
			String projectKey, String issueKey) throws UnAuthorizedException {
		// TODO Auto-generated method stub
		DynamicProxy proxy = connect(username, password, projectKey);
		Ticket ticket = (Ticket) proxy.newInstance( Ticket.class );
		try{
			int result = ticket.delete(Integer.parseInt(issueKey));
		}
		catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
		}
	}

	@Override
	public void addComment(String username, String password, String projectKey,
			String issueKey, Comment comment) throws InternalServerErrorException {
		// TODO Auto-generated method stub
		DynamicProxy proxy = connect(username, password, projectKey);

		Ticket ticket = (Ticket) proxy.newInstance(Ticket.class);

		int intKey = Integer.parseInt(issueKey);

		try {
			Object[] array = ticket.update(intKey, comment.getContent(), null);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new InternalServerErrorException(e.getMessage());
		}

	}

	@Override
	public Comment[] getComments(String username, String password,
			String projectKey, String issueKey) throws InternalServerErrorException {
		// TODO Auto-generated method stub
		DynamicProxy proxy = connect(username, password, projectKey);

		Ticket ticket = (Ticket) proxy.newInstance(Ticket.class);

		int intKey = Integer.parseInt(issueKey);

//		System.out.println("intkey " + intKey);

		try{

			ArrayList<Comment> comments = new ArrayList<Comment>();

			Object[] changeLog = ticket.changeLog(intKey, 0);
			for (Object object : changeLog) {
//				System.out.println(object);
//				System.out.println(object.getClass().getName());

				Object[] change =  (Object[]) object;

				Integer date = (Integer) change[0];
				String author = (String) change[1];
				String field = (String) change[2];
				String oldvalue = (String) change[3];
				String newvalue = (String) change[4];
				Integer permanent = (Integer) change[5];

				if(field.equalsIgnoreCase("comment")){

					if(newvalue != null && newvalue.length() > 0){

						Date created = new Date(date);					
						Comment comment = new Comment(author, newvalue, created);					
						comments.add(comment);

					}

				}

//				for (Object object2 : change) {
//				System.out.println(object2);
//				System.out.println(object2.getClass().getName());

			}

			Comment[] commentsArray = new Comment[comments.size()];
			for(int i = 0; i < comments.size(); i++){
				commentsArray[i] = comments.get(i);
			}
			return commentsArray;

		} catch (Throwable e) {
			e.printStackTrace();
			// TODO: handle exception
			throw new InternalServerErrorException(e.getMessage());
		}

	}

	@Override
	public String[] getComponentNames(String username, String password,
			String projectKey) {
		// TODO Auto-generated method stub
		DynamicProxy proxy = connect(username, password, projectKey);

		Ticket ticket = (Ticket) proxy.newInstance(Ticket.class);	

		String[] componentNames = (String[]) ticket.component_getAll();
		return componentNames;
	}

	@Override
	public String[] getVersions(String username, String password,
			String projectKey) throws InternalServerErrorException {
		// TODO Auto-generated method stub
		DynamicProxy proxy = connect(username, password, projectKey);

		Ticket ticket = (Ticket) proxy.newInstance(Ticket.class);	

		Object[] versionNames;
		try {
			versionNames = ticket.version_getAll();

			String[] versions = new String[versionNames.length];

			for (int i = 0; i < versionNames.length; i++) {

				versions[i] = (String)versionNames[i];
			}

			return versions;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new InternalServerErrorException(e.getMessage());
		}
	}

	@Override
	public String[] getIssueTypes(String username, String password,
			String projectKey) {
		// TODO Auto-generated method stub
		String[] issueTypes = (String[]) IssueTypes.getInstance().getArray();		
		return issueTypes;
	}

	@Override
	public String[] getPriorities(String username, String password,
			String projectKey) {
		// TODO Auto-generated method stub
		String[] priorities = (String[]) PriorityTypes.getInstance().getArray();
		return priorities;
	}

	@Override
	public String[] getStates(String username, String password,
			String projectKey, String issueKey) {

		//TODO: Do something to find out which actions are really available..!!
		String[] states = (String[]) IssueStates.getInstance().getArray();
		return states;
	}

	@Override
	public ArrayList<String> getUpdatedIssueKeys(String username, String password,
			String projectKey, Date date) throws InternalServerErrorException {
		// TODO Auto-generated method stub
		DynamicProxy proxy = connect(username, password, projectKey);
		Ticket ticket = (Ticket) proxy.newInstance( Ticket.class );

		ArrayList<String> issueKeys = new ArrayList<String>();

		try {

//			Long iso8601date = new Long(date.getTime()); 

			Object[] ticketList = (Object[]) ticket.getRecentChanges(date);

			if(ticketList != null && ticketList.length > 0)
			{


				System.out.println("Fetching one by one.." + new Date().toString());
				for(int i = 0; i < ticketList.length; i++)
				{				
					String issueKey = ticketList[i].toString();	

					issueKeys.add(issueKey);

				}

			}

		}	catch (Throwable t) {
			// TODO: handle exception
			t.printStackTrace();
			throw new InternalServerErrorException(t.getMessage());
		}	

		return issueKeys;

	}

	@Override
	public ArrayList<String> getIssueKeys(String username, String password,
			String projectKey) throws UnAuthorizedException,
			InternalServerErrorException {

		DynamicProxy proxy = connect(username, password, projectKey);
		Ticket ticket = (Ticket) proxy.newInstance( Ticket.class );

		ArrayList<String> issueKeys = new ArrayList<String>();

		try{

			Object[] ticketList = (Object[]) ticket.query("status=closed|new|assigned|reopened");


			if(ticketList != null && ticketList.length > 0)
			{


				System.out.println("Fetching one by one.." + new Date().toString());
				for(int i = 0; i < ticketList.length; i++)
				{				
					String issueKey = ticketList[i].toString();	

					issueKeys.add(issueKey);

				}

			}

		} catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
			throw new InternalServerErrorException(e.getMessage());
		}

		return issueKeys;

	}


}
