
import java.util.ArrayList;
import java.util.Date;

import junit.framework.TestCase;

import no.miles.mpl.wrapper.BugTrackerWrapperInterface;
import no.miles.mpl.wrapper.JiraWrapper;
import no.miles.mpl.wrapper.TracWrapper;
import no.miles.mpl.wrapper.datastructures.Issue;
import no.miles.mpl.wrapper.datastructures.IssueStates;
import no.miles.mpl.wrapper.datastructures.IssueTypes;
import no.miles.mpl.wrapper.datastructures.PriorityTypes;
import no.miles.mpl.wrapper.exception.BadRequestException;
import no.miles.mpl.wrapper.exception.NotFoundException;
import no.miles.mpl.wrapper.exception.UnAuthorizedException;

import org.junit.After;
import org.junit.Before;


public class JiraWrapperTest extends TestCase {

	private static String endpointAdress = "http://localhost:8080/jira/";

	

	private Issue createDefaultIssue() throws Exception{

		Issue issue = new Issue();
		issue.setSummary("JUnit test issue");
		issue.setCreated(new Date());
		issue.setReported_by("test");
		issue.setAssigned_to("test");
		ArrayList<String> components = new ArrayList<String>();
		components.add("esgs");

		issue.setComponents(components);
//		issue.setStatus("Closed");
		issue.setType(IssueTypes.ISSUE_TYPE_ENHANCEMENT);
//		issue.setResolution("1");		
		issue.setPriority(PriorityTypes.PRIORITY_CRITICAL);
//		issue.setResolution("Fixed");

		return issue;
	}

	@Before
	public void setUp() throws Exception {

	}

	@After
	public void tearDown() throws Exception {
	}

	public void testAddJiraIssue() throws Exception{

		if(AllTests.performIntegrationTest){
		
//		for(int i = 0; i < 1000; i++){

			BugTrackerWrapperInterface jiraWrapper = new JiraWrapper(endpointAdress);	

			Issue issue = createDefaultIssue();

			Issue addedIssue = jiraWrapper.addIssue("test", "test", "LC", issue);

			assertEquals(issue.getSummary(), addedIssue.getSummary());
			assertEquals(issue.getAssigned_to(), addedIssue.getAssigned_to());
			assertEquals(issue.getReported_by(), addedIssue.getReported_by());
			assertEquals(IssueStates.STATE_OPEN, addedIssue.getState());
			assertEquals(issue.getType(), addedIssue.getType());
			assertEquals(issue.getComponents().get(0), addedIssue.getComponents().get(0));
			assertEquals(issue.getPriority(), addedIssue.getPriority());

			jiraWrapper.deleteIssue("test", "test", "LC", addedIssue.getKey());
//		}
		}
	}

	public void testUpdateJiraIssue() throws Exception{

		if(AllTests.performIntegrationTest){
			BugTrackerWrapperInterface jiraWrapper = new JiraWrapper(endpointAdress);

			Issue issue = createDefaultIssue();

			Issue addedIssue = jiraWrapper.addIssue("test", "test", "LC", issue);

			addedIssue.setAssigned_to("test");
			addedIssue.setDescription("new description");
			addedIssue.setPriority(PriorityTypes.PRIORITY_MINOR);
			addedIssue.setReported_by("test");
			addedIssue.setSummary("New summary");
//			addedIssue.setStatus("Closed");
//			addedIssue.setResolution("Fixed");

			addedIssue.setState(IssueStates.STATE_FIXED);

			Issue updatedIssue = jiraWrapper.updateIssue("test", "test", "LC", addedIssue);	

			assertEquals(addedIssue.getPriority(), updatedIssue.getPriority());
			assertEquals(addedIssue.getDescription(), updatedIssue.getDescription());
			assertEquals(addedIssue.getReported_by(), updatedIssue.getReported_by());
			assertEquals(addedIssue.getState(), updatedIssue.getState());

			jiraWrapper.deleteIssue("test", "test", "LC", addedIssue.getKey());
		}

	}

	public void testGetJiraIssue() throws Exception{
		if(AllTests.performIntegrationTest){
			BugTrackerWrapperInterface jiraWrapper = new JiraWrapper(endpointAdress);

			Issue issue = createDefaultIssue();

			Issue addedIssue = jiraWrapper.addIssue("test", "test", "LC", issue);	

			Issue fetchedIssue = jiraWrapper.getIssue("test", "test", "LC", addedIssue.getKey());

			assertEquals(addedIssue.getKey(), fetchedIssue.getKey());

			jiraWrapper.deleteIssue("test", "test", "LC", addedIssue.getKey());
		}

	}

	public void testGetJiraIssues(){		

	}

	public void testDeleteJiraIssue() throws Exception{
		if(AllTests.performIntegrationTest){

			BugTrackerWrapperInterface jiraWrapper = new JiraWrapper(endpointAdress);

			Issue issue = createDefaultIssue();

			Issue addedIssue = jiraWrapper.addIssue("test", "test", "LC", issue);

			jiraWrapper.deleteIssue("test", "test", "LC", addedIssue.getKey());

			boolean found = true;

			try{
				Issue deletedIssue = jiraWrapper.getIssue("test", "test", "LC", addedIssue.getKey());

			} catch (NotFoundException e) {
				// TODO: handle exception
				found = false;
			}

			assertEquals(false, found);
		}



	}	

	
	public void testGetUpdatedIssueKeys() {		
		
//		BugTrackerWrapperInterface jiraWrapper = new JiraWrapper(endpointAdress);
		
//		ArrayList<String> keys = jiraWrapper.getUpdatedIssueKeys("test", "test", "LC");
		
	}
	
}
