
import java.util.ArrayList;
import java.util.Date;

import junit.framework.TestCase;

import no.miles.mpl.wrapper.BugTrackerWrapperInterface;
import no.miles.mpl.wrapper.TracWrapper;
import no.miles.mpl.wrapper.datastructures.Comment;
import no.miles.mpl.wrapper.datastructures.Issue;
import no.miles.mpl.wrapper.datastructures.IssueStates;
import no.miles.mpl.wrapper.datastructures.IssueTypes;
import no.miles.mpl.wrapper.datastructures.PriorityTypes;
import no.miles.mpl.wrapper.exception.BadRequestException;
import no.miles.mpl.wrapper.exception.InternalServerErrorException;
import no.miles.mpl.wrapper.exception.UnAuthorizedException;

import org.junit.After;
import org.junit.Before;


public class TracWrapperTest extends TestCase {

	private static String endpointAdress = "http://localhost:8090/trac/mpl/";

	private Issue createDefaultIssue() throws Exception{

		Issue issue = new Issue();
		issue.setSummary("JUnit test issue");
		issue.setCreated(new Date());
		issue.setReported_by("test");
		issue.setAssigned_to("test");
//		issue.setStatus("new");
		issue.setState(IssueStates.STATE_NEW);
		issue.setType(IssueTypes.ISSUE_TYPE_DEFECT);
		issue.setPriority(PriorityTypes.PRIORITY_CRITICAL);

		return issue;
	}

	@Before
	public void setUp() throws Exception {

	}

	@After
	public void tearDown() throws Exception {

	}

	public void testGetVersions(){

		if(AllTests.performIntegrationTest){
			BugTrackerWrapperInterface tracWrapper = new TracWrapper(endpointAdress);

			try {
				String[] versions = tracWrapper.getVersions("test", "test", "mpl");
				for (String version : versions) {
					System.out.println("Version: " + version);
				}

			} catch (BadRequestException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InternalServerErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

	}

	public void testAddTracIssue() throws Exception{

		if(AllTests.performIntegrationTest){
			BugTrackerWrapperInterface tracWrapper = new TracWrapper(endpointAdress);

			Issue issue = createDefaultIssue();

			Issue addedIssue = tracWrapper.addIssue("test", "test", "mpl", issue);

			assertEquals(issue.getSummary(), addedIssue.getSummary());
			assertEquals(issue.getAssigned_to(), addedIssue.getAssigned_to());
			assertEquals(issue.getReported_by(), addedIssue.getReported_by());
			assertEquals(issue.getState(), addedIssue.getState());
			assertEquals(issue.getType(), addedIssue.getType());

			tracWrapper.deleteIssue("test", "test", "mpl", addedIssue.getKey());
		}
	}

	public void testUpdateTracIssue() throws Exception{

		if(AllTests.performIntegrationTest){
			BugTrackerWrapperInterface tracWrapper = new TracWrapper(endpointAdress);

			Issue issue = createDefaultIssue();

			Issue addedIssue = tracWrapper.addIssue("test", "test", "mpl", issue);

			addedIssue.setSummary("Changed test issue");
//			addedIssue.setStatus("assigned");
			addedIssue.setState(IssueStates.STATE_STARTED);

			Issue updatedIssue = tracWrapper.updateIssue("test", "test", "mpl", addedIssue);

			assertEquals(addedIssue.getSummary(), updatedIssue.getSummary());
			assertEquals(addedIssue.getAssigned_to(), updatedIssue.getAssigned_to());
			assertEquals(addedIssue.getReported_by(), updatedIssue.getReported_by());
			assertEquals(addedIssue.getState(), updatedIssue.getState());
			assertEquals(addedIssue.getType(), updatedIssue.getType());

			tracWrapper.deleteIssue("test", "test", "mpl", addedIssue.getKey());
		}
	}

	public void testGetTracIssue() throws Exception{

		if(AllTests.performIntegrationTest){
			BugTrackerWrapperInterface tracWrapper = new TracWrapper(endpointAdress);

			Issue issue = createDefaultIssue();

			Issue addedIssue = tracWrapper.addIssue("test", "test", "mpl", issue);

			Issue fetchedIssue = tracWrapper.getIssue("test", "test", "mpl", addedIssue.getKey());

			assertEquals(addedIssue.getKey(), fetchedIssue.getKey());
			assertEquals(addedIssue.getSummary(), addedIssue.getSummary());

			tracWrapper.deleteIssue("test", "test", "mpl", addedIssue.getKey());
		}
	}


	public void testMockup() throws BadRequestException, InternalServerErrorException{

		if(AllTests.performIntegrationTest){
			BugTrackerWrapperInterface tracWrapper = new TracWrapper(endpointAdress);
			String issueKey = "5";

			System.out.println("Fetching ticket 5");
//			Issue issueFour = tracWrapper.getIssue("test", "test", "mpl", issueKey);		


			System.out.println("Fetching comments");
			Comment[] comments = tracWrapper.getComments("test", "test", "mpl", issueKey);
			for (Comment comment : comments) {
				System.out.println("Comment: " + comment.getContent());
			}
			System.out.println("done fetching comments");
		}

	}

//	public void testGetTracIssues(){		

//	BugTrackerWrapperInterface tracWrapper = new TracWrapper(endpointAdress);

//	try {
//	Issue[] issues = tracWrapper.getIssues("test", "test", "mpl");

//	for (Issue issue : issues) {
//	String ont = issue.toN3();				
////	System.out.println("ont: " + ont);

//	Issue.parseOntology(ont, "mpl");
//	}

//	} catch (BadRequestException e) {
//	// TODO Auto-generated catch block
//	e.printStackTrace();
//	} catch (InternalServerErrorException e) {
//	// TODO Auto-generated catch block
//	e.printStackTrace();
//	} catch (Exception e) {
//	// TODO Auto-generated catch block
//	e.printStackTrace();
//	}



//	}

	public void testDeleteTracIssue() throws Exception{
		
		if(AllTests.performIntegrationTest){
			BugTrackerWrapperInterface tracWrapper = new TracWrapper(endpointAdress);

			Issue issue = createDefaultIssue();

			Issue addedIssue = tracWrapper.addIssue("test", "test", "mpl", issue);

			tracWrapper.deleteIssue("test", "test", "mpl", addedIssue.getKey());

			Issue deletedIssue = tracWrapper.getIssue("test", "test", "mpl", addedIssue.getKey());

			assertEquals(null, deletedIssue);
		}

	}	

}
