package no.miles.mpl.wrapper.datastructures;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Resource;

/**
 * Holds an attachement (or at least information of it) belonging to an issue..)
 * @author erlingwl
 *
 */
public class Attachment {
	
	public static String DCNS = "http://purl.org/dc/elements/1.1/";
	public static String DCTNS = "http://purl.org/dc/terms/";


	private Date created;
	private String author;
	private String fileName;
	private String uri;
	
	public Date getCreated() {
		return created;
	}

	public void setCreated(Date created) {
		this.created = created;
	}

	public String getAuthor() {
		return author;
	}

	public void setAuthor(String author) {
		this.author = author;
	}

	public String getFileName() {
		return fileName;
	}

	public void setFileName(String fileName) {
		this.fileName = fileName;
	}

	public String getUri() {
		return uri;
	}

	public void setUri(String uri) {
		this.uri = uri;
	}

	/**
	 * Constructor used for an existing attachement
	 * @param author
	 * @param fileName
	 * @param Created
	 * @param uri
	 */
	public Attachment(String author, String fileName, Date created, String uri){
		
		this.author = author;
		this.fileName = fileName;
		this.created = created;
		this.uri = uri;
	}
	
	/**
	 * Constructor used for a new (uncommitted) attachement
	 * @param author
	 * @param fileName
	 */
	public Attachment(String author, String fileName){
		
		this.author = author;
		this.fileName = fileName;
		this.created = new Date();
	}
	
	/**
	 * Return the Attachement in n3 representation
	 * @return
	 */
	public String toN3(){
	
		String n3 = null;
		
		OntModel model = ModelFactory.createOntologyModel();
		model.setNsPrefix("", Issue.BAETLENS);
		model.setNsPrefix("sioc", Issue.SIOCNS);
		model.setNsPrefix("dc", DCNS);
		model.setNsPrefix("dct", DCTNS);
		model.setNsPrefix("foaf", Issue.FOAFNS);

//		Resource commentResource = model.createResource(Issue.SIOCNS + "Post");			
//		Individual commentIndividual = model.createIndividual(commentResource);
//
//
//		//xsd:dateTime
//		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-d'T'HH:mm:ss.S");
//
//		if(created != null){			
//			String xsdCreated = df.format(created);	
//			commentIndividual.addProperty(model.createDatatypeProperty(DCTNS + "created"), xsdCreated);
//		}
//
//		if(author != null){			
//			Resource authorResource = model.createResource(Issue.SIOCNS + "User");
//			Individual authorIndividual = model.createIndividual(authorResource);
//			authorIndividual.addProperty(model.createDatatypeProperty(Issue.FOAFNS + "accountName"), author);		
//			commentIndividual.addProperty(model.createObjectProperty(Issue.SIOCNS + "has_creator"), authorIndividual);
//		}
//
//		if(content != null){
//			commentIndividual.addProperty(model.createDatatypeProperty(Issue.SIOCNS + "content"), content);
//		}

		//Convert to string
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
		model.write(bufferedOut, "N3");

		n3 = out.toString();
		
		return n3;		
	}
	
	/**
	 * Parse an Attachement from n3
	 * @param n3
	 * @return
	 */
	public Attachment parseAttachement(String n3){
		
		Attachment attachement = null;
		
		OntModel model = ModelFactory.createOntologyModel();
		model.setNsPrefix("", Issue.BAETLENS);
		model.setNsPrefix("sioc", Issue.SIOCNS);
		model.setNsPrefix("dc", DCNS);
		model.setNsPrefix("dct", DCTNS);
		model.setNsPrefix("foaf", Issue.FOAFNS);

//		Resource commentResource = model.createResource(Issue.SIOCNS + "Post");			
//		Individual commentIndividual = model.createIndividual(commentResource);
//
//
//		//xsd:dateTime
//		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-d'T'HH:mm:ss.S");
//
//		if(created != null){			
//			String xsdCreated = df.format(created);	
//			commentIndividual.addProperty(model.createDatatypeProperty(DCTNS + "created"), xsdCreated);
//		}
//
//		if(author != null){			
//			Resource authorResource = model.createResource(Issue.SIOCNS + "User");
//			Individual authorIndividual = model.createIndividual(authorResource);
//			authorIndividual.addProperty(model.createDatatypeProperty(Issue.FOAFNS + "accountName"), author);		
//			commentIndividual.addProperty(model.createObjectProperty(Issue.SIOCNS + "has_creator"), authorIndividual);
//		}

//		if(content != null){
//			commentIndividual.addProperty(model.createDatatypeProperty(Issue.SIOCNS + "content"), content);
//		}

		//Convert to string
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
		model.write(bufferedOut, "N3");

//		n3 = out.toString();
		
		
		return attachement;
		
	}
	
}
