package no.miles.mpl.wrapper.datastructures;

import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;


import com.hp.hpl.jena.ontology.DatatypeProperty;
import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.ObjectProperty;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.RDFReader;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

public class Issue {

	public static String BAETLENS = "http://xmlns.com/baetle/#";
	public static String RDFNS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	public static String WFNS = "http://www.w3.org/2005/01/wf/flow#";
	public static String FOAFNS = "http://xmlns.com/foaf/0.1/";
	public static String SIOCNS = "http://rdfs.org/sioc/ns#";
	public static String DOAPNS = "http://usefulinc.com/ns/doap#";
	public static String MPLNS = "http://platform.miles.no/ont/#";

	private String key;
	private String summary;
	private String projectkey;
	private String type;
	private String priority;
	private String state;
	private ArrayList<String> components = new ArrayList<String>();
	private ArrayList<String> versions = new ArrayList<String>();
	private String assigned_to;
	private String reported_by;
	private String description;
//	private String status;
//	private String resolution;
	private Date created;
	private Date updated;
	private Date dueDate;
	private String viewissueurl;
//	private ArrayList<Comment> comments = new ArrayList<Comment>();

	//TODO: comments

	public String getViewissueurl() {
		return viewissueurl;
	}

	public void setViewissueurl(String viewissueurl) {
		this.viewissueurl = viewissueurl;
	}

	public Date getUpdated() {
		return updated;
	}

	public void setUpdated(Date updated) {
		this.updated = updated;
	}

	public String getProjectkey() {
		return projectkey;
	}

	public void setProjectkey(String projectkey) {
		this.projectkey = projectkey;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) throws Exception {

		if(IssueTypes.getInstance().contains(type)){
			this.type = type;
		} else {
			throw new Exception("Type " + type + " not supported");
		}
	}

	public String getPriority() {
		return priority;
	}

	public void setPriority(String priority) throws Exception {
		if(PriorityTypes.getInstance().contains(priority)){
			this.priority = priority;
		} else {
			throw new Exception("Priority " + priority + " not supported");
		}		
	}

	public String getState() {
		return state;
	}


	public void setState(String state) throws Exception {

		if(IssueStates.getInstance().contains(state)){			
			this.state = state;
		} else {
			throw new Exception("State " + state + " not supported");
		}

	}

	public String getAssigned_to() {
		return assigned_to;
	}

	public void setAssigned_to(String assigned_to) {
		this.assigned_to = assigned_to;
	}

	public String getReported_by() {
		return reported_by;
	}

	public void setReported_by(String reported_by) {
		this.reported_by = reported_by;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Date getCreated() {
		return created;
	}

	public void setCreated(Date created) {
		this.created = created;
	}

	public Issue()
	{

	}

	public Issue(String key)
	{
		this.key = key;		
	}

	public Issue(String key, String summary)
	{		
		this.key = key;
		this.summary = summary;
	}

	public String getKey() {
		return key;
	}

	public void setKey(String key) {
		this.key = key;
	}

	public String getSummary() {
		return summary;
	}

	public void setSummary(String summary) {
		this.summary = summary;
	}

//	public String getStatus() {
//	return status;
//	}

//	public void setStatus(String status) {
//	this.status = status;
//	}

//	public String getResolution() {
//	return resolution;
//	}

//	public void setResolution(String resolution) {
//	this.resolution = resolution;
//	}

	public ArrayList<String> getComponents() {
		return components;
	}

	public void setComponents(ArrayList<String> components) {
		this.components = components;
	}

	public ArrayList<String> getVersions() {
		return versions;
	}

	public void setVersions(ArrayList<String> versions) {
		this.versions = versions;
	}

	/**
	 * Can parse an issue when the key and url are unknown
	 * @param ontology
	 * @param projectKey
	 * @return
	 * @throws BadRequestException 
	 */
	public static Issue parseOntology(String ontology, String projectKey) throws Exception{

		return parseOntology(ontology, projectKey, null, null);

	}


//	public static Issue parseOntology(String ontology, String projectKey, String issueKey){

//	return parseOntology(ontology, projectKey, issueKey, null);

//	}

	/**
	 * @throws BadRequestException 
	 * 
	 */
	public static Issue parseOntology(String ontology, String projectKey, String issueKey, String url) throws Exception{

		Issue issue = new Issue();		

		/**
		 * Properties that can't directly be deduced from owl / rdf??
		 */
		issue.setProjectkey(projectKey);

		if(issueKey != null)
			issue.setKey(issueKey);		

		OntModel model = ModelFactory.createOntologyModel();
		ByteArrayInputStream inputStream = new ByteArrayInputStream(ontology.getBytes());
		model.read(inputStream, "", "N3");

		Resource resource = null;

		//TODO: try something else..
//		if(url != null){
//			resource = model.getResource(url);
////			issue.setViewissueurl(url); Not really important..
//		}

	
		
		if(resource == null){		

			ResIterator iterator; 			
			for (String issueType : IssueTypes.getInstance()) {				
				
				iterator = model.listSubjectsWithProperty(model.createProperty(RDFNS + "type"), model.createResource(issueType));
				
				while(iterator.hasNext()){
										
					resource = iterator.nextResource();			
					break;
				}

				if(resource != null)
					break;				
			}		
		}


		String rdfType = resource.getProperty(model.createProperty(RDFNS + "type")).getObject().toString();
		issue.setType(rdfType);	


		//Try to fetch state
		String state = null;
		Statement stateStatement = resource.getProperty(model.createObjectProperty(WFNS + "state"));
		if(stateStatement != null){		
			Statement stateTypeStatement = stateStatement.getResource().getProperty(model.createProperty(RDFNS + "type"));		
			if(stateTypeStatement != null)
				state = stateTypeStatement.getObject().toString();		
		}		
		if(state != null)
			issue.setState(state);

		//Fetch summary / title
		String summary = resource.getProperty(model.createDatatypeProperty(BAETLENS + "summary")).getObject().toString();				
		if(summary != null)
			issue.setSummary(summary);

		//Fetch priority
		String priority = null;

		Statement priorityStatement = resource.getProperty(model.createObjectProperty(BAETLENS + "priority"));
		if(priorityStatement != null){
			Statement priorityTypeStatement = priorityStatement.getResource().getProperty(model.createProperty(RDFNS + "type"));
			if(priorityTypeStatement != null)
				priority = priorityTypeStatement.getObject().toString();
		}
		if(priority != null)
			issue.setPriority(priority);

		//xsd:dateTime
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-d'T'HH:mm:ss.S");

		//Created
		String created = null;
		Statement createdStatement = resource.getProperty(model.createDatatypeProperty(BAETLENS + "created"));
		if(createdStatement != null){
			created = createdStatement.getObject().toString();
		}
		if(created != null)
			issue.setCreated(df.parse(created));

		//Updated
		String updated = null;
		Statement updatedStatement = resource.getProperty(model.createDatatypeProperty(BAETLENS + "updated"));
		if(updatedStatement != null){
			updated = createdStatement.getObject().toString();
		}
		if(updated != null)
			issue.setUpdated(df.parse(updated));

		//Due date
		String due_date = null;
		Statement due_dateStatement = resource.getProperty(model.createDatatypeProperty(BAETLENS + "due_date"));
		if(due_dateStatement != null){
			due_date = createdStatement.getObject().toString();
		}
		if(due_date != null)
			issue.setUpdated(df.parse(due_date));

		//Fetch description
		String description = null;
		Statement descriptionStatement = resource.getProperty(model.createDatatypeProperty(BAETLENS + "description"));
		if(descriptionStatement != null){
			description = descriptionStatement.getObject().toString();	
		}
		if(description != null)
			issue.setDescription(description);

		//reporter
		String reporter = null;
		Statement reporterStatement = resource.getProperty(model.createObjectProperty(BAETLENS + "reporter"));
		if(reporterStatement != null){
			Statement reporterNameStatement = reporterStatement.getResource().getProperty(model.createDatatypeProperty(FOAFNS + "accountName"));
			if(reporterNameStatement != null)
				reporter = reporterNameStatement.getObject().toString();
		}
		if(reporter != null)
			issue.setReported_by(reporter);

		//assinged to
		String assigned_to = null;
		Statement assigned_toStatement = resource.getProperty(model.createObjectProperty(BAETLENS + "assigned_to"));
		if(assigned_toStatement != null){
			Statement assigned_toNameStatement = assigned_toStatement.getResource().getProperty(model.createDatatypeProperty(FOAFNS + "accountName"));
			if(assigned_toNameStatement != null)
				assigned_to = assigned_toNameStatement.getObject().toString();
		}
		if(assigned_to != null)
			issue.setAssigned_to(assigned_to);

		//Components
		ArrayList<String> components = new ArrayList<String>();
		ArrayList<Statement> statements = new ArrayList<Statement>();
		StmtIterator aboutIterator = resource.listProperties(model.createObjectProperty(BAETLENS + "about"));
		while(aboutIterator.hasNext()){
			Statement aboutStatement = aboutIterator.nextStatement();
			statements.add(aboutStatement);
		}
		for (Statement aboutStatement : statements) {
			Statement aboutNameStatement = aboutStatement.getResource().getProperty(model.createDatatypeProperty(BAETLENS + "name"));
			if(aboutNameStatement != null){
				String component = aboutNameStatement.getObject().toString();
				if(component != null)
					components.add(component);
			}
		}		
		if(components.size() > 0)
			issue.setComponents(components);

		//Versions
		ArrayList<String> versions = new ArrayList<String>();
		statements = new ArrayList<Statement>();
		StmtIterator versionIterator = resource.listProperties(model.createObjectProperty(BAETLENS + "version"));
		while(versionIterator.hasNext()){
			Statement versionStatement = versionIterator.nextStatement();
			statements.add(versionStatement);
		}
		for (Statement versionStatement : statements) {
			Statement versionIdStatement = versionStatement.getResource().getProperty(model.createDatatypeProperty(BAETLENS + "id"));
			if(versionIdStatement != null){
				String version = versionIdStatement.getObject().toString();
				if(version != null)
					versions.add(version);
			}
		}
		if(versions.size() > 0)
			issue.setVersions(versions);

		return issue;
	}

//	public static Issue parseXml(String xml){
//
////		Issue issue = new Issue();		
//		return null;
//	}

	public String toN3(){

		String ontology = null;
		OntModel model = ModelFactory.createOntologyModel();
		model.setNsPrefix("", BAETLENS);
		model.setNsPrefix("wf", WFNS);
		model.setNsPrefix("foaf", FOAFNS);
		model.setNsPrefix("sioc", SIOCNS);
		model.setNsPrefix("doap", DOAPNS);

		Resource issueResource = model.createResource(type);			
		Individual issueIndividual = null;

		if(getViewissueurl() != null){
			issueIndividual = model.createIndividual(getViewissueurl(), issueResource);
		} else
		{
			//The issues view url is not specified / could be a new issue - not yet committed..
			issueIndividual = model.createIndividual(issueResource);
		}

		//Instantiate the correct priority type
		String priority = getPriority();		
		Resource priorityResource = model.createResource(priority);
		Individual priorityIndividual = model.createIndividual(priorityResource);
		issueIndividual.addProperty(model.createObjectProperty(BAETLENS + "priority"), priorityIndividual);

		//Instantiate the correct state type
		String state = getState();	
		if(state != null){

			Resource stateResource = model.createResource(state);
			Individual stateIndividual = model.createIndividual(stateResource);
			issueIndividual.addProperty(model.createObjectProperty(WFNS + "state"), stateIndividual);

		}

		//xsd:dateTime
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-d'T'HH:mm:ss.S");

		Date created = getCreated();
		if(created != null){			
			String xsdCreated = df.format(created);			
			issueIndividual.addProperty(model.createDatatypeProperty(BAETLENS + "created"), xsdCreated);
		}

		//xsd:updated
		Date updated = getUpdated();
		if(updated != null){
			String xsdUpdated = df.format(updated);	
			issueIndividual.addProperty(model.createDatatypeProperty(BAETLENS + "updated"), xsdUpdated);
		}

		//xsd:due_date
		Date dueDate = getDueDate();
		if(dueDate != null){
			String xsdDueDate = df.format(dueDate);	
			issueIndividual.addProperty(model.createDatatypeProperty(BAETLENS + "due_date"), xsdDueDate);
		}

		if(getDescription() != null)
			issueIndividual.addProperty(model.createDatatypeProperty(BAETLENS + "description"), getDescription());

		issueIndividual.addProperty(model.createDatatypeProperty(BAETLENS + "summary"), getSummary());

		//reporter
		if(getReported_by() != null){
			Resource reporterResource = model.createResource(SIOCNS + "User");
			Individual reporterIndividual = model.createIndividual(reporterResource);
			reporterIndividual.addProperty(model.createDatatypeProperty(FOAFNS + "accountName"), getReported_by());
			issueIndividual.addProperty(model.createObjectProperty(BAETLENS + "reporter"), reporterIndividual);
		}

		//assigned_to
		if(getAssigned_to() != null){
			Resource assigned_toResource = model.createResource(SIOCNS + "User");
			Individual assigned_toIndividual = model.createIndividual(assigned_toResource);
			assigned_toIndividual.addProperty(model.createDatatypeProperty(FOAFNS + "accountName"), getAssigned_to());
			issueIndividual.addProperty(model.createObjectProperty(BAETLENS + "assigned_to"), assigned_toIndividual);
		}

		//versions
		if(getVersions() != null){
			for (String version : getVersions()) {
				Resource versionResource = model.createResource(BAETLENS + "Version");
				Individual versionIndividual = model.createIndividual(versionResource);
				versionIndividual.addProperty(model.createDatatypeProperty(BAETLENS + "id"), version);
				issueIndividual.addProperty(model.createObjectProperty(BAETLENS + "version"), versionIndividual);
			}

		}

		//components
		if(getComponents() != null){
			for (String component : getComponents()) {
				Resource softwarePackageResource = model.createResource(BAETLENS + "SoftwarePackage");
				Individual softwarePackageIndividual = model.createIndividual(softwarePackageResource);
				softwarePackageIndividual.addProperty(model.createDatatypeProperty(BAETLENS + "name"), component);
				issueIndividual.addProperty(model.createObjectProperty(BAETLENS + "about"), softwarePackageIndividual);
			}
		}

		//project
		if(getProjectkey() != null){
			Resource projectResource = model.createResource(DOAPNS + "Project");
			Individual projectIndividual = model.createIndividual(projectResource);
			projectIndividual.addProperty(model.createDatatypeProperty(DOAPNS + "name"), getProjectkey());
			//TODO: Add more properties to the project? Or seeAlso link?
			issueIndividual.addProperty(model.createObjectProperty(BAETLENS + "project"), projectIndividual);
		}


		//Convert to string
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
		model.write(bufferedOut, "N3");

		ontology = out.toString();

		return ontology;
	}

	public String toXml(){

		return null;
	}

	public Date getDueDate() {
		return dueDate;
	}

	public void setDueDate(Date dueDate) {
		this.dueDate = dueDate;
	}




}
