package no.miles.mpl.wrapper.datastructures;

import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;


public class Project {

	private ArrayList<Issue> issues;
	private String name;
	private String issuetracker;
	private String issuetrackerurl;	
	private ArrayList<String> types;
	private ArrayList<String> components;
	private ArrayList<String> versions;
	private ArrayList<String> priorities;
	private ArrayList<String> statuses;
	private ArrayList<String> resolutions;

	private String url;

	public Project(String name)	{

		this.name = name;
		issues = new ArrayList<Issue>();
	}


	public String getBugtrackertype() {
		return issuetracker;
	}

	public void setBugtrackertype(String bugtrackertype) {
		this.issuetracker = bugtrackertype;
	}

	public String getBugtrackerurl() {
		return issuetrackerurl;
	}

	public void setBugtrackerurl(String bugtrackerurl) {
		this.issuetrackerurl = bugtrackerurl;
	}

	public String getUrl() {
		return url;
	}

	public void setUrl(String url) {
		this.url = url;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public ArrayList<Issue> getIssues() {
		return issues;
	}

	public void setIssues(ArrayList<Issue> issues) {
		this.issues = issues;
	}

	public ArrayList<String> getTypes() {
		return types;
	}

	public void setTypes(ArrayList<String> types) {
		this.types = types;
	}

	public ArrayList<String> getComponents() {
		return components;
	}

	public void setComponents(ArrayList<String> components) {
		this.components = components;
	}

	public ArrayList<String> getVersions() {
		return versions;
	}

	public void setVersions(ArrayList<String> versions) {
		this.versions = versions;
	}

	public ArrayList<String> getPriorities() {
		return priorities;
	}

	public void setPriorities(ArrayList<String> priorities) {
		this.priorities = priorities;
	}

	public ArrayList<String> getStatuses() {
		return statuses;
	}

	public void setStatuses(ArrayList<String> statuses) {
		this.statuses = statuses;
	}

	public ArrayList<String> getResolutions() {
		return resolutions;
	}

	public void setResolutions(ArrayList<String> resolutions) {
		this.resolutions = resolutions;
	}	

	public String toN3(){

		String n3 = null;

		OntModel model = ModelFactory.createOntologyModel();
		model.setNsPrefix("", Issue.BAETLENS);
		model.setNsPrefix("wf", Issue.WFNS);
		model.setNsPrefix("foaf", Issue.FOAFNS);
		model.setNsPrefix("sioc", Issue.SIOCNS);
		model.setNsPrefix("doap", Issue.DOAPNS);
		model.setNsPrefix("mpl", Issue.MPLNS);

		Resource projectResource = model.createResource(Issue.DOAPNS + "Project");			
		Individual projectIndividual = null;

		if(this.getUrl() != null){
			projectIndividual = model.createIndividual(this.getUrl(), projectResource);
		} else	{
			projectIndividual = model.createIndividual(projectResource);
		}

		projectIndividual.addProperty(model.createDatatypeProperty(Issue.DOAPNS + "name"), this.getName());

		Resource onlineAccount = model.createResource(Issue.SIOCNS + "OnlineAccount");	
		Resource issueTracker = model.createResource(this.getBugtrackertype());

		Individual issueTrackerIndividual = model.createIndividual(this.getBugtrackerurl(), onlineAccount);
		issueTrackerIndividual.addRDFType(issueTracker);

		projectIndividual.addProperty(model.createObjectProperty(Issue.DOAPNS + "bug-database"), issueTrackerIndividual);

		//Convert to string
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
		model.write(bufferedOut, "N3");
		n3 = out.toString();	

		return n3;

	}

	public static Project parse(String n3){

		Project project = null;

		OntModel model = ModelFactory.createOntologyModel();
		ByteArrayInputStream inputStream = new ByteArrayInputStream(n3.getBytes());
		model.read(inputStream, "", "N3");
		
//		System.out.println("so far so good..");
		
//		System.out.println(n3);

		String projectDataQueryString = 		
			"PREFIX rdf: <" + Issue.RDFNS + ">" +
			"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" +
			"PREFIX owl:  <http://www.w3.org/2002/07/owl#>" +
			"PREFIX mpl: <" + Issue.MPLNS + ">" +
			"PREFIX foaf: <" + Issue.FOAFNS +">" +
			"PREFIX doap: <" + Issue.DOAPNS +">" +
			"PREFIX sioc: <" + Issue.SIOCNS +">" +
			"PREFIX wf: <" + Issue.WFNS +">" +
			"SELECT ?project ?projectName ?bugdb ?bugdbtype " +
			"WHERE {" +
			"      ?project rdf:type doap:Project . " +	
			"      ?project doap:name ?projectName . " +
			"      ?project doap:bug-database ?bugdb . " +	
			"      ?bugdb rdf:type ?bugdbtype . " +	
			"      }" +
			"";	

		Query query = QueryFactory.create(projectDataQueryString);
		
//		System.out.println("query ok");

		QueryExecution queryEx = QueryExecutionFactory.create(query, model);			

		ResultSet rs = queryEx.execSelect();

		String name = null;
		String uri = null;
		String bugdb = null;
		String bugdbtype = null;

		while(rs.hasNext()){
			
//			System.out.println("sol..");

			QuerySolution qs = rs.nextSolution();
			
//			System.out.println(qs);
			
			if(name == null)
			name = qs.get("projectName").toString();
			
			if(uri == null)
			uri = qs.get("project").toString();
			
			if(bugdb == null)
			bugdb = qs.get("bugdb").toString();

			String possiblebugdbtype = qs.get("bugdbtype").toString();
			if(IssueTrackers.getInstance().contains(possiblebugdbtype))
				bugdbtype = possiblebugdbtype;			

//			System.out.println(qs.toString());
		}		

		if(project == null && name != null){	
			project = new Project(name);
		}

		if(project != null){

			project.setUrl(uri);
			project.setBugtrackerurl(bugdb);
			project.setBugtrackertype(bugdbtype);

		}


		return project;
	}


}
