//package no.miles.mpl.tracapi;
//
//import javax.annotation.Resource.AuthenticationType;
//
//public class TracXmlRpcClientV2 {
//
//	private class XmlRpcRequest {
//
//		private final String method;
//
//		private final Object[] parameters;
//
//		public XmlRpcRequest(String method, Object[] parameters) {
//			this.method = method;
//			this.parameters = parameters;
//		}
//
//		public Object execute() throws Exception {
//			try {
//				// first attempt
//				return executeCallInternal();
//			} catch (Exception e) {
//				if (accountMangerAuthenticationFailed) {
//					// do not try again if this has failed in the past since it
//					// is more likely that XML_RPC permissions have not been set
//					throw e;
//				}
//
//				AuthenticationCredentials credentials = location.getCredentials(AuthenticationType.REPOSITORY);
//				if (!credentialsValid(credentials)) {
//					throw e;
//				}
//
//				// try form-based authentication via AccountManagerPlugin as a
//				// fall-back
//				HostConfiguration hostConfiguration = WebClientUtil.createHostConfiguration(httpClient, USER_AGENT,
//						location, monitor);
//				try {
//					authenticateAccountManager(httpClient, hostConfiguration, credentials, monitor);
//				} catch (TracLoginException loginException) {
//					// caused by wrong user name or password
//					throw loginException;
//				} catch (IOException ignore) {
//					accountMangerAuthenticationFailed = true;
//					throw e;
//				}
//
//				try {
//					validateAuthenticationState(httpClient);
//				} catch (TracLoginException ignore) {
//					// most likely form based authentication is not supported by
//					// repository
//					accountMangerAuthenticationFailed = true;
//					throw e;
//				}
//
//				factory.setCookies(httpClient.getState().getCookies());
//			}
//
//			// second attempt
//			return executeCallInternal(monitor);
//		}
//
//		private Object executeCallInternal(IProgressMonitor monitor) throws TracException {
//			try {
//				TracXmlRpcClientRequest request = new TracXmlRpcClientRequest(xmlrpc.getClientConfig(), method,
//						parameters, monitor);
//				return xmlrpc.execute(request);
//			} catch (TracHttpException e) {
//				if (e.code == HttpStatus.SC_UNAUTHORIZED) {
//					throw new TracLoginException();
//				} else if (e.code == HttpStatus.SC_FORBIDDEN) {
//					throw new TracPermissionDeniedException();
//				} else if (e.code == HttpStatus.SC_PROXY_AUTHENTICATION_REQUIRED) {
//					throw new TracProxyAuthenticationException();
//				} else {
//					throw new TracException(e);
//				}
//			} catch (XmlRpcException e) {
//				if (e.code == NO_SUCH_METHOD_ERROR) {
//					throw new TracNoSuchMethodException(e);
//				} else {
//					throw new TracRemoteException(e);
//				}
//			} catch (Exception e) {
//				throw new TracException(e);
//			}
//		}
//
//	}
//
//	public static final String XMLRPC_URL = "/xmlrpc";
//
//	public static final String REQUIRED_REVISION = "1950";
//
//	public static final int REQUIRED_EPOCH = 0;
//
//	public static final int REQUIRED_MAJOR = 0;
//
//	public static final int REQUIRED_MINOR = 1;
//
//	private static final int NO_SUCH_METHOD_ERROR = 1;
//
//	private static final int LATEST_VERSION = -1;
//
//	public static final int REQUIRED_WIKI_RPC_VERSION = 2;
//
//	private XmlRpcClient xmlrpc;
//
//	private TracHttpClientTransportFactory factory;
//
//	private int majorAPIVersion = -1;
//
//	private int minorAPIVersion = -1;
//
//	private int epochAPIVersion = -1;
//
//	private boolean accountMangerAuthenticationFailed;
//
//	private XmlRpcClientConfigImpl config;
//
//	private final HttpClient httpClient;
//
//	public TracXmlRpcClient(AbstractWebLocation location, Version version) {
//		super(location, version);
//
//		httpClient = new HttpClient();
//		httpClient.setHttpConnectionManager(new MultiThreadedHttpConnectionManager());
//		httpClient.getParams().setCookiePolicy(CookiePolicy.RFC_2109);
//	}
//
//	public synchronized XmlRpcClient getClient() throws TracException {
//		if (xmlrpc == null) {
//			config = new XmlRpcClientConfigImpl();
//			config.setEncoding(ITracClient.CHARSET);
//			config.setTimeZone(TimeZone.getTimeZone(ITracClient.TIME_ZONE));
//			config.setContentLengthOptional(false);
//			config.setConnectionTimeout(WebClientUtil.CONNNECT_TIMEOUT);
//			config.setReplyTimeout(WebClientUtil.SOCKET_TIMEOUT);
//
//			xmlrpc = new XmlRpcClient();
//			xmlrpc.setConfig(config);
//
//			factory = new TracHttpClientTransportFactory(xmlrpc, httpClient);
//			factory.setLocation(location);
//			xmlrpc.setTransportFactory(factory);
//		}
//
//		// update configuration with latest values
//		AuthenticationCredentials credentials = location.getCredentials(AuthenticationType.REPOSITORY);
//		if (credentialsValid(credentials)) {
//			config.setBasicUserName(credentials.getUserName());
//			config.setBasicPassword(credentials.getPassword());
//		} else {
//			config.setBasicUserName(null);
//			config.setBasicPassword(null);
//		}
//		config.setServerURL(getXmlRpcUrl(credentials));
//
//		return xmlrpc;
//	}
//
//	private URL getXmlRpcUrl(AuthenticationCredentials credentials) throws TracException {
//		try {
//			String location = repositoryUrl.toString();
//			if (credentialsValid(credentials)) {
//				location += LOGIN_URL;
//			}
//			location += XMLRPC_URL;
//
//			return new URL(location);
//		} catch (Exception e) {
//			throw new TracException(e);
//		}
//	}
//
//	private Object call(String method, Object... parameters) throws TracException {
//		return call(null, method, parameters);
//	}
//
//	private Object call(IProgressMonitor monitor, String method, Object... parameters) throws TracException {
//		while (true) {
//			getClient();
//
//			try {
//				XmlRpcRequest request = new XmlRpcRequest(method, parameters);
//				return request.execute(monitor);
//			} catch (TracLoginException e) {
//				if (location.requestCredentials(AuthenticationType.REPOSITORY, null) == ResultType.NOT_SUPPORTED) {
//					throw e;
//				}
//			} catch (TracPermissionDeniedException e) {
//				if (location.requestCredentials(AuthenticationType.REPOSITORY, null) == ResultType.NOT_SUPPORTED) {
//					throw e;
//				}
//			} catch (TracProxyAuthenticationException e) {
//				if (location.requestCredentials(AuthenticationType.PROXY, null) == ResultType.NOT_SUPPORTED) {
//					throw e;
//				}
//			}
//		}
//	}
//
//	private Object[] multicall(Map<String, Object>... calls) throws TracException {
//		return multicall(null, calls);
//	}
//
//	private Object[] multicall(IProgressMonitor monitor, Map<String, Object>... calls) throws TracException {
//		Object[] result = (Object[]) call(monitor, "system.multicall", new Object[] { calls });
//		for (Object item : result) {
//			try {
//				checkForException(item);
//			} catch (XmlRpcException e) {
//				throw new TracRemoteException(e);
//			} catch (Exception e) {
//				throw new TracException(e);
//			}
//		}
//		return result;
//	}
//
//	private void checkForException(Object result) throws NumberFormatException, XmlRpcException {
//		if (result instanceof Map) {
//			Map<?, ?> exceptionData = (Map<?, ?>) result;
//			if (exceptionData.containsKey("faultCode") && exceptionData.containsKey("faultString")) {
//				throw new XmlRpcException(Integer.parseInt(exceptionData.get("faultCode").toString()),
//						(String) exceptionData.get("faultString"));
//			}
//		}
//	}
//
//	private Map<String, Object> createMultiCall(String methodName, Object... parameters) throws TracException {
//		Map<String, Object> table = new HashMap<String, Object>();
//		table.put("methodName", methodName);
//		table.put("params", parameters);
//		return table;
//	}
//
//	private Object getMultiCallResult(Object item) {
//		return ((Object[]) item)[0];
//	}
//
//}
