package no.miles.mpl.atomextension.issue;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;


import org.jdom.Element;
import org.jdom.Namespace;

import com.sun.syndication.feed.module.Module;
import com.sun.syndication.io.ModuleGenerator;

public class IssueModuleGenerator implements ModuleGenerator {

	private static final Namespace ISSUE_NS  = Namespace.getNamespace("mplissue", IssueModule.URI);

	@Override
	public void generate(Module module, Element element) {
		// TODO Auto-generated method stub

		Element root = element;
		while (root.getParent()!=null && root.getParent() instanceof Element) {
			root = (Element) element.getParent();
		}
		root.addNamespaceDeclaration(ISSUE_NS);

		IssueModule im = (IssueModule) module;

		if(im.getAssignedTo() != null){
			root.addContent(generateSimpleElement("assigned_to", im.getAssignedTo()));
		}

		if(im.getComponents() != null){        	
			ArrayList<String> components = im.getComponents();
			for (String string : components) {
				root.addContent(generateSimpleElement("component", string));
			}   	       	
		}

//		if(im.getCreated() != null){

//		}

		if(im.getDescription() != null){
			root.addContent(generateSimpleElement("description", im.getDescription()));
		}

		if(im.getPriority() != null){
			root.addContent(generateSimpleElement("priority", im.getPriority()));
		}

		if(im.getReportedBy() != null){
			root.addContent(generateSimpleElement("reported_by", im.getReportedBy()));
		}

		if(im.getResolution() != null){
			root.addContent(generateSimpleElement("resolution", im.getResolution()));
		}

		if(im.getStatus() != null){
			root.addContent(generateSimpleElement("status", im.getStatus()));
		}

		if(im.getType() != null){
			root.addContent(generateSimpleElement("type", im.getType()));
		}

		if(im.getVersions() != null){        	
			ArrayList<String> versions = im.getVersions();
			for (String string : versions) {
				root.addContent(generateSimpleElement("version", string));
			}   	       	
		}
	}

	private static final Set NAMESPACES;

	static {
		Set<Namespace> nss = new HashSet<Namespace>();
		nss.add(ISSUE_NS);
		NAMESPACES = Collections.unmodifiableSet(nss);
	}

	@Override
	public String getNamespaceUri() {
		// TODO Auto-generated method stub
		return IssueModule.URI;
	}

	@Override
	public Set getNamespaces() {
		// TODO Auto-generated method stub
		return NAMESPACES;
	}

	protected Element generateSimpleElement(String name, String value)  {
		Element element = new Element(name, ISSUE_NS);
		element.addContent(value);
		return element;
	}

}
