package no.miles.mpl.semantic.restfulendpoint;

import java.io.ByteArrayOutputStream;

import no.miles.mpl.semantic.AtomToBaetleConverter;
import no.miles.mpl.semantic.SparqlQuery;

import org.restlet.Context;
import org.restlet.data.Form;
import org.restlet.data.MediaType;
import org.restlet.data.Parameter;
import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.resource.Representation;
import org.restlet.resource.Resource;
import org.restlet.resource.StringRepresentation;
import org.restlet.resource.Variant;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.InfModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.reasoner.Reasoner;
import com.hp.hpl.jena.reasoner.ReasonerRegistry;

public class RestProject extends Resource {

	protected String projectKey;
	protected String uri;
	protected String username;
	protected String password;
	protected String root;
	protected OntModel model;
	protected String query;
	protected Reasoner reasoner;
	protected InfModel inf;

	protected String sparqlQuery;

	public RestProject(Context context, Request request, Response response) {
		super(context, request, response);

		this.projectKey = (String) request.getAttributes().get("project");

		getVariants().add(new Variant(MediaType.TEXT_PLAIN));

		this.uri = request.getResourceRef().toString();
		this.root = request.getRootRef().toString();

		this.query = request.getResourceRef().getQuery();

		Form qForm = request.getResourceRef().getQueryAsForm();

		sparqlQuery = null;

		if(qForm != null)
		{		
			Parameter sparqlParam = qForm.getFirst("sparql");
			
			if(sparqlParam != null)
			sparqlQuery = sparqlParam.getValue();
		}



		String url = "http://localhost:8070/no.miles.mpl.layers.app/projects/" + projectKey + "/issues/";
		String user = "test";
		String pass = "test";

//		System.out.println(url);

		model = AtomToBaetleConverter.convertFeed(url, user, pass);

		Reasoner reasoner = ReasonerRegistry.getOWLReasoner();

		inf = ModelFactory.createInfModel(reasoner, model);

//		model.write(System.out, "RDF/XML-ABBREV");	
	}

	@Override
	public Representation getRepresentation(Variant variant) {
		Representation result = null;

		if(query != null && query.toLowerCase().startsWith("sparql"))
		{
			result = new StringRepresentation(sparqlSelect());
		}
		else
		{
			Model describeModel = describe();
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			describeModel.write(out);
			result = new StringRepresentation(out.toString());
		}


		return result;
	}

	private Model describe()
	{
		String queryString = 		
			"PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>" +
			"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" +
			"PREFIX owl:  <http://www.w3.org/2002/07/owl#>" +
			"PREFIX mpl: <" + AtomToBaetleConverter.nsMpl + ">" +
			"PREFIX foaf: <" + AtomToBaetleConverter.nsFoaf +">" +
			"PREFIX doap: <" + AtomToBaetleConverter.nsDoap +">" +
			"PREFIX sioc: <" + AtomToBaetleConverter.nsSioc +">" +
			"PREFIX wf: <" + AtomToBaetleConverter.nsWorkFlow +">" +
			"DESCRIBE ?project " +
			"WHERE {" +
			"      ?project rdf:type doap:Project . " +	
			"      ?project foaf:name \"" + projectKey + "\" . " +	
			"      }";	

//		Model describeModel = SparqlQuery.describe(model, queryString);
		Model describeModel = SparqlQuery.describe(inf, queryString);

		return describeModel;
	}

	protected String sparqlSelect()
	{	

		String prefixes =
			"PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>" +
			"PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>" +
			"PREFIX owl:  <http://www.w3.org/2002/07/owl#>" +
			"PREFIX mpl: <" + AtomToBaetleConverter.nsMpl + ">" +
			"PREFIX foaf: <" + AtomToBaetleConverter.nsFoaf +">" +
			"PREFIX doap: <" + AtomToBaetleConverter.nsDoap +">" +
			"PREFIX sioc: <" + AtomToBaetleConverter.nsSioc +">" +
			"PREFIX wf: <" + AtomToBaetleConverter.nsWorkFlow +"> ";

		query = prefixes + sparqlQuery;

//		System.out.println("stripped query: " + query);

		String sparqlResult = SparqlQuery.performQuery(describe(), query);		
		return sparqlResult;
	}
}
