package no.miles.mpl.semantic;

import java.io.InputStream;
import java.util.Iterator;

import no.miles.mpl.atomextension.issue.IssueModule;

import com.ctc.wstx.sr.NsDefaultProvider;
import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.util.FileManager;
import com.sun.syndication.propono.atom.client.AtomClientFactory;
import com.sun.syndication.propono.atom.client.ClientCollection;
import com.sun.syndication.propono.atom.client.ClientEntry;

public class AtomConverter {

	private static String nsMpl = "http://platform.miles.no/ont/1.0/mpl.owl#";	

	/**
	 * Converts the atom feed containing Issues to a respective ontology..
	 * @param url
	 * @param user
	 * @param pass
	 * @return
	 */
	public static OntModel convertFeed(String url, String user, String pass)
	{
		OntModel model = null;

		try {
			ClientCollection atomClient = AtomClientFactory.getCollection(url, user, pass);
			
			//TODO: Remove this..
			String projectName = "mpl";

			Iterator<ClientEntry> entries = atomClient.getEntries();

			model =  ModelFactory.createOntologyModel();

//			model.setNsPrefix("mpl", nsMpl);
			

			

			String inputFileName = "mpl.owl";
			// use the FileManager to find the input file
			InputStream in = FileManager.get().open( inputFileName );
			if (in == null) {
				throw new IllegalArgumentException(
						"File: " + inputFileName + " not found");
			}
			model.read(in, "");
			
			//Create a project
			Resource projectResource = model.createResource(nsMpl + "Project");
			Individual projectInd = model.createIndividual(projectResource);
			projectInd.addProperty(model.createDatatypeProperty(nsMpl + "hasName"), projectName);

			while(entries.hasNext()){

				ClientEntry entry = entries.next();
				IssueModule module = (IssueModule) entry.getModule(IssueModule.URI);

				Resource resource = model.createResource(nsMpl + "Issue");
				Individual i = model.createIndividual(resource);
				i.addProperty(model.createDatatypeProperty(nsMpl + "hasSummary"), entry.getSummary().getValue());
				i.addProperty(model.createDatatypeProperty(nsMpl + "hasTitle"), entry.getTitle());
				i.addProperty(model.createDatatypeProperty(nsMpl + "hasUrl"), entry.getId());
				
				//Add project
				i.addProperty(model.createObjectProperty(nsMpl + "belongsToProject"), projectInd);
				
				if(module != null)
				{
					String priority = module.getPriority();
					addPriority(model, priority, i);
				}
			}			
			
//			model.write(System.out, "RDF/XML-ABBREV");	

		}

		catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
		}


		return model;

	}

	/**
	 * Instantiates the correct priority instance..
	 * @param model
	 * @param type
	 * @param ind
	 */
	private static void addPriority(OntModel model, String type, Individual ind)
	{
		//Nullcheck
		if(type == null)
			return;
		
		String priorityType = null;

		if(type.equalsIgnoreCase("major"))
		{
			priorityType = "Major";
		}
		else if(type.equalsIgnoreCase("minor"))
		{
			priorityType = "Minor";
		}
		else if(type.equalsIgnoreCase("blocker"))
		{
			priorityType = "Blocker";
		}
		else if(type.equalsIgnoreCase("critical"))
		{
			priorityType = "Critical";
		}
		else if(type.equalsIgnoreCase("trivial"))
		{
			priorityType = "Trivial";
		}

		if(priorityType != null)
		{
			Resource resource = model.createResource(nsMpl + priorityType);
			Individual i = model.createIndividual(resource);
			ind.addProperty(model.createObjectProperty(nsMpl + "hasPriority"), i);
		}

	}

	/**
	 * Executes a sparql query on the supplied model
	 * @param model
	 * @param queryString
	 */
	public static void performQuery(Model model, String queryString)
	{
		Query query = QueryFactory.create(queryString);

		// Execute the query and obtain results
		QueryExecution qe = QueryExecutionFactory.create(query, model);
		ResultSet rs = qe.execSelect();
		
		// Output query results			

		// Important - free up resources used running the query

		ResultSetFormatter.out(System.out, rs);
		qe.close();
	
	}

}
