package no.miles.mpl.semantic;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;

import no.miles.mpl.atomextension.issue.IssueModule;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.util.FileManager;
import com.sun.syndication.propono.atom.client.AtomClientFactory;
import com.sun.syndication.propono.atom.client.ClientCollection;
import com.sun.syndication.propono.atom.client.ClientEntry;

public class AtomToBaetleConverter {

	public static String nsMpl = "http://platform.miles.no/ont/1.0/baetlempl.owl#";
	public static String nsDoap = "http://usefulinc.com/ns/doap#";
	public static String nsFoaf = "http://xmlns.com/foaf/0.1/";
	public static String nsSioc = "http://rdfs.org/sioc/ns#";
	
	private static String owlFile = "baetlempl.owl";

	/**
	 * Converts an Atom feed to the baetlempl ontology
	 * @param url
	 * @param user
	 * @param pass
	 * @return
	 */
	public static OntModel convertFeed(String url, String user, String pass)
	{

		OntModel model = null;

		try {
			ClientCollection atomClient = AtomClientFactory.getCollection(url, user, pass);

			//TODO: Remove this..
			String projectName = "mpl";

			Iterator<ClientEntry> entries = atomClient.getEntries();

			model =  ModelFactory.createOntologyModel();
			model.setNsPrefix("doap", nsDoap);
			model.setNsPrefix("foaf", nsFoaf);
			model.setNsPrefix("sioc", nsSioc);

			loadFile(model, owlFile);
			
			Individual project = addProject(model, projectName);
			

			while(entries.hasNext()){

				ClientEntry entry = entries.next();
				IssueModule module = (IssueModule) entry.getModule(IssueModule.URI);

				Resource resource = model.createResource(nsMpl + "Issue");
				
				Individual i = model.createIndividual(resource);
				i.addProperty(model.createDatatypeProperty(nsMpl + "hasSummary"), entry.getSummary().getValue());
				i.addProperty(model.createDatatypeProperty(nsMpl + "hasKey"), entry.getTitle());
//				i.addProperty(model.createDatatypeProperty(nsMpl + "hasUrl"), entry.getId());
				
				i.addProperty(model.createDatatypeProperty(nsMpl + "isType"), module.getType());
				
				//Add project
				i.addProperty(model.createObjectProperty(nsMpl + "belongsToProject"), project);
				
				
				String assignedTo = module.getAssignedTo();
				
				addAssignedTo(model, i, assignedTo);
				
				String reportedBy = module.getReportedBy();
				
				addReportedBy(model, i, reportedBy);
				
				ArrayList<String> components = module.getComponents();
				if(components != null)
				{
					
				}
				
				String priority = module.getPriority();
				
				addPriority(model, priority, i);
				
			}			
			
		}
		catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
		}


		return model;
	}
	
	/**
	 * Load the ontology..
	 * @param model
	 * @param fileName
	 */
	private static void loadFile(OntModel model, String fileName)
	{		
		// use the FileManager to find the input file
		InputStream in = FileManager.get().open( fileName );
		if (in == null) {
			throw new IllegalArgumentException(
					"File: " + fileName + " not found");
		}
		model.read(in, "");
	}
	
	
	private static Individual addProject(OntModel model, String projectKey)
	{
		//Create a project
		Resource projectResource = model.createResource(nsDoap + "Project");
		Individual projectInd = model.createIndividual(projectResource);
		projectInd.addProperty(model.createDatatypeProperty(nsFoaf + "name"), projectKey);
		
		return projectInd;

	}
	
	private static void addAssignedTo(OntModel model, Individual issue, String username)
	{
		Resource assignedToResource = model.createResource(nsSioc + "User");
		Individual assignedTo = model.createIndividual(assignedToResource);
		assignedTo.addProperty(model.createDatatypeProperty(nsFoaf + "accountName"), username);
		issue.addProperty(model.createObjectProperty(nsMpl + "assignedTo"), assignedTo);
	}
	
	private static void addReportedBy(OntModel model, Individual issue, String username)
	{			
		Resource assignedToResource = model.createResource(nsSioc + "User");
		Individual assignedTo = model.createIndividual(assignedToResource);
		assignedTo.addProperty(model.createDatatypeProperty(nsFoaf + "accountName"), username);
		issue.addProperty(model.createObjectProperty(nsMpl + "reportedBy"), assignedTo);
	}
	
	/**
	 * Instantiates the correct priority instance..
	 * @param model
	 * @param type
	 * @param ind
	 */
	private static void addPriority(OntModel model, String type, Individual ind)
	{
		//Nullcheck
		if(type == null)
			return;
		
		String priorityType = null;

		if(type.equalsIgnoreCase("major"))
		{
			priorityType = "Major";
		}
		else if(type.equalsIgnoreCase("minor"))
		{
			priorityType = "Minor";
		}
		else if(type.equalsIgnoreCase("blocker"))
		{
			priorityType = "Blocker";
		}
		else if(type.equalsIgnoreCase("critical"))
		{
			priorityType = "Critical";
		}
		else if(type.equalsIgnoreCase("trivial"))
		{
			priorityType = "Trivial";
		}

		if(priorityType != null)
		{
			Resource resource = model.createResource(nsMpl + priorityType);
			Individual i = model.createIndividual(resource);
			ind.addProperty(model.createObjectProperty(nsMpl + "hasPriority"), i);
		}

	}
}
